/******************************************************************************
 * Licensed Materials - Property of IBM                                       *
 *                                                                            *
 * PID (5697-TKT)                                                             *
 *                                                                            *
 * (C) Copyright by IBM 2000. All Rights Reserved.                            *
 *****************************************************************************/
package com.BobCo.postit;

import com.tivoli.twg.libs.Command;
import com.tivoli.twg.libs.IntelByteBuffer;
import com.tivoli.twg.libs.IntValueSet;
import com.tivoli.twg.log.TWGRas;

/**
 * Command class for retrieving a list of reminder notes for a specific
 * managed object to be displayed in this task bean's association view.
 */
public class BCGetPostItNotesForObjectCmd extends Command
{
  /**
   * Constructor for the retrieving the list of post it notes for
   * an object command.
   *
   * @param oid the object id to get the list of notes for
   */
  public BCGetPostItNotesForObjectCmd( long oid )
  {
    super( BCPostItConstants.GET_POST_ITS_FOR_OBJECT_CMD );
    SetDestinationAddress( BCPostItConstants.POST_IT_SERVER );

    int   iBBLength = 0;
    IntelByteBuffer iBB = null;

    // Parm 0 = object id
    // Talley length of buffer.
    iBBLength = IntelByteBuffer.SIZEOF_LONG64;

    // Allocate the Intel byte buffer.
    iBB = new IntelByteBuffer( iBBLength );

    // Write parameter to buffer and attach to command.
    iBB.WriteLong64( oid );
    AddInputParm( iBB.GetBuffer() );
  }

  /**
   * Get the current list of post it notes that are attached to the object
   * from the command.
   *
   * @return set of note ids attached to the system.
   */
  public IntValueSet getObjectsPostItList()
  {
    if ( NumOutputParms() == 1 )
      return IntelByteBuffer.ReadIntValueSet( OutputParm(0), 0 );
    else
      return null;
  }

}

