/******************************************************************************
 * Licensed Materials - Property of IBM                            *
 *                                                                            *
 * PID (5697-TKT)                                                             *
 *                                                                            *
 * (C) Copyright by IBM 2000. All Rights Reserved. *
 *****************************************************************************/
package com.BobCo.postit;

import com.tivoli.twg.engine.DuplicateObjectIDException;
import com.tivoli.twg.engine.InvalidObjectIDException;
import com.tivoli.twg.engine.TWGExtension;
import com.tivoli.twg.engine.TWGExtensionInitException;
import com.tivoli.twg.engine.TWGExtensionTermException;
import com.tivoli.twg.engine.TWGImageSet;
import com.tivoli.twg.engine.TWGManagedObjectStatusGroup;
import com.tivoli.twg.engine.TWGManagedObjectStatusSetConstants;
import com.tivoli.twg.engine.TWGTask;
import com.tivoli.twg.engine.TWGTaskCreationException;
import com.tivoli.twg.engine.TWGDefaultTask;
import com.tivoli.twg.engine.TWGContentInfoBeans;
import com.tivoli.twg.engine.TWGPersistentObjectSaveException;
import com.tivoli.twg.libs.TWGNLSString;
import com.tivoli.twg.log.TWGRas;
import java.util.Properties;

/**
 * Post It Note extension class : defined by the
 * BCPostIt.TWGExt property file.
 */
public class BCPostItExtension extends TWGExtension
{
    /**
     * Default constructor : usually does nothing, but can access
     * properties loaded from our property file.
     */
    public BCPostItExtension()
    {
        TWGRas.entryExit( TWGRas.EX1, "BCPostItExtension() done");
    }

    /**
     * First initialization method, intended to allow extensions to
     * register classes.
     *
     * @exception TWGExtensionInitException if a fatal error occurs
     * during the initialization method
     */
    public void InitClassRegistration() throws TWGExtensionInitException
    {
        TWGRas.entryExit( TWGRas.EX1, "BCPostItExtension.InitClassRegistration()");
    }

    /**
     * Second initialization method, intended to allow extension to
     * create default instances of classes registered during
     * InitClassRegistration().
     *
     * @exception TWGExtensionInitException if a fatal error occurs
     * during the initialization method.
     */
    public void InitClassInstances() throws TWGExtensionInitException
    {
        TWGRas.entry( TWGRas.EX1, "BCPostItExtension.InitClassInstances()");
        boolean err = false;
        TWGDefaultTask postItTask = null;

        // Instantiate "SystemReminderNotes" task
        if (!TWGTask.isTaskID("BobCo|SystemReminderNotes"))
        {
            try
            {
                postItTask = new TWGDefaultTask( "com/BobCo/postit/BCPostIt.properties" );
            }
            catch (TWGTaskCreationException e)
            {
                TWGRas.error(TWGRas.EX1, "BCPostItExtension.InitClassInstances: exception!", e);
                e.printStackTrace();
                err = true;
            }
        }
        else
        {
            postItTask = (TWGDefaultTask)TWGTask.getTask("BobCo|SystemReminderNotes");
        }

        // Check if the sample content info bean is loaded yet.
        TWGContentInfoBeans cib = TWGContentInfoBeans.getContentInfoBeans( "tivoli.postit.sample" );
        if (cib == null)
        {
            // If not, then lets create it.
            try
            {
                cib = new TWGContentInfoBeans( "tivoli.postit.sample", 
                                               100, 
                                               "com.BobCo.postit.BCPostItResources",
                                               "ReminderNoteAssocViewMenu", 
                                               "com.BobCo.postit.BCPostItContentInfoObjectHandler",
                                               "com.BobCo.postit.BCPostItContentInfoEventHandler" );
            }
            catch (TWGPersistentObjectSaveException ose)
            {
                TWGRas.error( TWGRas.EX1,  "Error creating and saving postit content info bean", ose );
            }
        }

        initializeStatusGroup();

        if (err)
            throw new TWGExtensionInitException( "Error creating Post It default task");

        TWGRas.exit( TWGRas.EX1, "BCPostItExtension.InitClassInstances()");
    }

    /**
     * Final initialization method, intended to allow an extension to
     * do any actions which may have required the other extensions to
     * have completed the previous passes of initialization.
     *
     * @exception TWGExtensionInitException if a fatal error occurs
     * during the initialization method
     */
    public void InitCompletion() throws TWGExtensionInitException
    {
        TWGRas.entryExit( TWGRas.EX1, "BCPostItExtension.InitCompletion()");
        /*- Do task association stuff here */
        boolean err = false;
        if (err)
            throw new TWGExtensionInitException();
    }

    /**
     * First termination method, intended to allow an extension to
     * begin the process of termination.  The only termination
     * processing which should occur here is that which needs to be
     * done before other extensions have executed TermCompletion() pass.
     *
     * @exception TWGExtensionTermException if an error occurs during
     * the termination method.
     */
    public void TermBegin() throws TWGExtensionTermException
    {
        TWGRas.entryExit( TWGRas.EX1, "BCPostItExtension.TermBegin()");
        boolean err = false;
        if (err)
            throw new TWGExtensionTermException();
    }

    /**
     * Final termination method, intended to allow an extension to
     * terminate any processing which may still be active.
     *
     * @exception TWGExtensionTermException if an error occurs during
     * the termination method.
     */
    public void TermComplete() throws TWGExtensionTermException
    {
        TWGRas.entryExit( TWGRas.EX1, "BCPostItExtension.TermComplete()");
        boolean err = false;
        if (err)
            throw new TWGExtensionTermException();
    }

    /**
     * Place necessary information in the TWGServer to register monitor support
     * of threshold status groups.
     */
    private void initializeStatusGroup() 
    {
        TWGRas.entry( TWGRas.EX1, "BCPostItExtension.initializeStatusGroup()");
        TWGManagedObjectStatusGroup group =
        TWGManagedObjectStatusGroup.findStatusGroup("postit.status");

        if (group != null) return;

        String[] images = { "/com/BobCo/postit/images/postit16.gif",
            "/com/BobCo/postit/images/postit16.gif"};

        try
        {
            group = new TWGManagedObjectStatusGroup( "postit.status",
                                                     new TWGNLSString("statusGroup.label", "com.BobCo.postit.BCPostItResources"),
                                                     new TWGNLSString("statusGroup.hover", "com.BobCo.postit.BCPostItResources"),
                                                     null );

            group.addStatusDefinition( "postitStatus",
                                       new TWGNLSString("postitStatus.label", "com.BobCo.postit.BCPostItResources"),
                                       new TWGNLSString("postitStatus.hover", "com.BobCo.postit.BCPostItResources"),
                                       TWGManagedObjectStatusSetConstants.SD_FLAG_PRIVATE_SETTABLE_ONLY,
                                       "BobCo|SystemReminderNotes",
                                       "BCCreatePostItNote",
                                       TWGImageSet.FindImageSet(images) );
            group.save();
        } // endtry
        catch (TWGPersistentObjectSaveException e)
        {
            TWGRas.error( TWGRas.EX1, "Error creating postit status group", e );
        }
        catch (DuplicateObjectIDException e)
        {
            TWGRas.error( TWGRas.EX1, "Error creating postit status group", e );
        }
        TWGRas.exit( TWGRas.EX1, "BCPostItExtension.initializeStatusGroup()" );
    }

}
