/******************************************************************************
 * Licensed Materials - Property of IBM                            *
 *                                                                            *
 * PID (5697-TKT)                                                             *
 *                                                                            *
 * (C) Copyright by IBM 1999. All Rights Reserved. *
 *****************************************************************************/
package com.BobCo.console;

import java.awt.*;
import java.awt.event.*;
import java.util.ListResourceBundle;
import java.util.Locale;

import javax.swing.*;
import javax.swing.border.*;
import javax.swing.event.*;
import javax.swing.text.*;

import com.tivoli.uif.core.UFImageLoader;
import com.tivoli.twg.console.TWGConsoleColor;
import com.tivoli.twg.console.TWGHelpContext;
import com.tivoli.twg.console.TWGTaskFrame;
import com.tivoli.twg.engine.TWGTaskActivator;

/*****************************************************************************
*                         Bob's Software House
*                    A Division of BobCo Enterprises
*****************************************************************************/
/****************************************************************************
* This is an example GUI frame that will provide the task's interface
* to the user to allow them to provide information to the task.
* In this example, a simple frame is displayed, allowing the user
* to interact with the frame.  No real functionality is performed in
* this sample, it is just a vehicle to demonstrate how you can implement
* a Task console extension using just the TWGTaskFrame
* and your own code.
*****************************************************************************/
public class BCSendMsgFrame
       extends TWGTaskFrame
       implements ActionListener
{
  /**
  * This is the constructor for creating the task frame for the user.
  */
  public BCSendMsgFrame()
  {
    // construct the parent frame without a status bar
    super(false);
  }


  /**
   * Initialization routine for setting up non-gui items.
   * Return true to have the frame continue initialization and be
   * displayed.  Return false to stop the frame from coming up.
   */
  public boolean pInit( )
  {
    // Load in the resource bundle for the frame.
    msgResources = (ListResourceBundle)ListResourceBundle.getBundle(
      "com.BobCo.console.BCResources", Locale.getDefault());

    // Set the frame's title and icon.
    setTitle( msgResources.getString( "SendMessageTitle" ));
    setIconImage( UFImageLoader.getImage( "/com/BobCo/console/images/spopup16.gif" ));
    return true;
  }

  /**
   * Called by the console when the server task activation for this instance of
   * the task frame has ended.  All termination processing should be done here,
   * and the main thread should be exited.
   *
   * @param endingType This indicates why the server task ended (i.e. completed
   * successfully, error occurred, etc.
   */
  public void serverTaskEnded( int endingType )
  {
  }


  /**
   * Build the client pane and return it.
   */
  public Container buildView( )
  {
    // Construct the top level client area pane that all panes are
    // placed into for the user interface.
    clientPanel = new JPanel();
    clientPanel.setBorder( new EmptyBorder( 10, 10, 10, 10 ) );
    clientPanel.setLayout( new BorderLayout() );

    // Construct a sample label as the client area.
    JLabel promptLbl = new JLabel( msgResources.getString( "SampleClientMsg" ),
                                   JLabel.CENTER );
    Font ourFont = new Font( "Dialog", Font.BOLD, 36 );
    promptLbl.setFont( ourFont );
    // Use the TWGConsoleColor class for all window colors
    promptLbl.setBackground( TWGConsoleColor.getWindowColor() );
    clientPanel.add( promptLbl );

    // Create the button group for placing the buttons into.
    JPanel btnGroup = new JPanel();
    btnGroup.setLayout( new BoxLayout( btnGroup, BoxLayout.X_AXIS ) );
    btnGroup.setBorder( new EmptyBorder( 10, 10, 10, 10 ) );
    btnGroup.add( Box.createHorizontalGlue() );

    // Add the ok button to the panel.
    okButton = new JButton( msgResources.getString( "SendMsgOkBtn" ));
    okButton.addActionListener( this );
    okButton.setActionCommand( "OKMessage" );
    btnGroup.add( okButton );
    btnGroup.add( Box.createHorizontalStrut( 10 ) );

    // Add the cancel button to the panel.
    cancelButton = new JButton( msgResources.getString( "SendMsgCancelBtn" ));
    cancelButton.addActionListener( this );
    cancelButton.setActionCommand( "CancelMessage" );
    btnGroup.add( cancelButton );

    // Populate the client pane with the panel and the buttons.
    getContentPane().add( "Center", clientPanel );
    getContentPane().add( "South", btnGroup );

    // Add a help topic to the client panel.  Whenever help is requested
    // for this frame via the F1 key, the help window will be launched
    // and the help panel loaded will be the one matching the topic
    // associated here.
    if ( getHelpContext() != null )
      getHelpContext().addHelp( clientPanel, "BCSendMsgConsole.main" );

    return (Container) clientPanel;
  }


  /**
   * Clean up routine.  This method is called by TWGTaskFrame when the
   * task frame is definitely closing (meaning any user verification
   * has already completed and the user indicated it was okay to close).
   * All termination processing should be done here.
   */
  public void pCleanUp( )
  {
    // Allow frames an opportunity to perform some cleanup.  This includes
    // removing the association between the Alt+F4 key, the Send and
    // Cancel pushbuttons of the frame, and their corresponding actions.
    //
    // Proper cleanup ensures no memory leaks.

    // Only clean up once
    if( cleaningup == false )
    {
      cleaningup = true;

      // clean up the panel
      clientPanel.unregisterKeyboardAction( KeyStroke.getKeyStroke(
                                 KeyEvent.VK_F4, InputEvent.ALT_MASK ));
      clientPanel.removeAll();
      clientPanel = null;

      // clean up the buttons
      okButton.removeActionListener( this );
      okButton = null;
      cancelButton.removeActionListener( this );
      cancelButton = null;

      // clean up the resource bundle
      msgResources = null;
    }
  }


  /**
   * Command handler class for actions.  In this sample, we only
   * have two commands, ok message and cancel/close the frame.
   */
  public void actionPerformed( ActionEvent evt )
  {
    String actionCommand = evt.getActionCommand();

    // If the command is ok message, then you would extract
    // the information from the client area.
    // We just do the same as cancel since this is not a real
    // application.  A more robust implementation would actually
    // check the fields for validity.
    if ( actionCommand.equals( "OKMessage" ))
    {
      // We close here, but this would not be the case typically.
      consoleCancel( TWGTaskFrame.USER_REQUESTED_CLOSE );
    }
    // If the command was cancel, then close the task frame down.
    // Calling consoleCancel() is the method to use and note how
    // we pass a parameter to communicate who requested the frame
    // close.
    else if ( actionCommand.equals( "CancelMessage" ))
    {
      consoleCancel( TWGTaskFrame.USER_REQUESTED_CLOSE );
    }
  }


  //
  // private data
  //
  private boolean            cleaningup       = false;
  private ListResourceBundle msgResources;
  private JPanel             clientPanel;
  private JButton            okButton;
  private JButton            cancelButton;

} // end class BCSendMsgFrame
