/******************************************************************************
 * Licensed Materials - Property of IBM                                       *
 *                                                                            *
 * (C) Copyright by IBM 2003.  All Rights Reserved.                           *
 *****************************************************************************/

package com.BobCo.webobject;

// this will need to be fine-tuned
import com.tivoli.twg.engine.InvalidObjectIDException;
import com.tivoli.twg.engine.TWGByteArrayInputStream;
import com.tivoli.twg.engine.TWGByteArrayOutputStream;
import com.tivoli.twg.engine.TWGManagedObject;
import com.tivoli.twg.engine.TWGManagedObjectConstants;
import com.tivoli.twg.engine.TWGManagedObjectFingerprint;
import com.tivoli.twg.engine.TWGObjectDestroyException;
import com.tivoli.twg.engine.TWGPersistentObjectDictionary;
import com.tivoli.twg.engine.TWGPersistentObjectRestoreException;
import com.tivoli.twg.engine.TWGPersistentObjectSaveException;

import com.tivoli.twg.libs.DataValue;
import com.tivoli.twg.libs.IntelByteBuffer;
import com.tivoli.twg.libs.QueueBatch;
import com.tivoli.twg.libs.TWGByteArray;
import com.tivoli.twg.libs.TWGString;
import com.tivoli.twg.libs.TWGStringArray;
import com.tivoli.twg.log.TWGRas;

import java.net.InetAddress;
import java.net.UnknownHostException;
import java.security.AccessController;
import java.sql.*;
import java.util.Collections;
import java.util.Enumeration;
import java.util.HashMap;
import java.util.Locale;

import java.io.NotSerializableException;
import java.io.UTFDataFormatException;


/**
 * This class extends the TWGManagedObject class to create a managed
 * object for systems that are web servers
 *
 * @see com.tivoli.twg.engine.TWGManagedObject
 */

public class BCWebServerManagedObject extends TWGManagedObject {

  /** Our classname */
  public static final String CLASSNAME =
    "com.BobCo.webobject.BCWebServerManagedObject";

  /** Version numbers for saving and restoring */
  private static final int VERSION_THREE = 3;
  private static final int VERSION_TWO   = 2;
  private static final int VERSION_ONE   = 1;

  /** List of supported attribute IDs */
  private static final String[] attrib_ids =
  {
    /* Hostnames for the Web server */
    TWGManagedObjectConstants.ATTRIB_IP_HOSTS,
    /* IP addresses for the Web server */
    TWGManagedObjectConstants.ATTRIB_IP_ADDRS
    //ATTRIB_ADDRESS
  };

  /** List of attribute types */
  private static final int[] attrib_types = {
    /* IP hosts */
    DataValue.STRING_ARRAY_TYPE,
    /* IP addresses */
    DataValue.STRING_ARRAY_TYPE
  };


  /** List of attribute labels */
  private static final String[] attrib_labels =
  {
    /** Web server IP address host names */
    "Host name",
    /** Web server IP Addresses */
    "IP Address"
  };


  /** List of web servers that we've already created */
  protected static HashMap webserverList = new HashMap();

  /** variable for the web server host name */
  private String ws_host;
  /** variable for the web server IP address */
  private String ws_ipaddress;

  //use ObjectQueue to hold the ping requests we need to process
  private static QueueBatch queue = null;


  /**
   * Default constructor for web server managed object
   */
  public BCWebServerManagedObject()
  {
    super();
    // initialize variables to null
    ws_host = null;
    ws_ipaddress = null;

    //create the queue that will hold MOs for us to investigate
    if (queue == null) {
        queue = new QueueBatch("BCWebServerPingQueue");
    }

    // Allow this object to be deleted from the console
    setConsoleDeletable(true);
    // Allow this object to be renamed from the console
    // I'll regret this later.
    setConsoleRenamable(true);

    TWGRas.entryExit(TWGRas.EX7,
                     "BCWebServerManagedObject constructor: object name = "
                     + getName());
  }

  /**
   * Default constructor for web server managed object
   */
  public BCWebServerManagedObject(String name, String ipAddress)
  {
    super();
    InetAddress address = null;
    // set variables
    ws_ipaddress = ipAddress;
    setName(name);
    ws_host = null;

    // The host name might not be the same as the managed object
    // name so set the host name based on the IP address.
    try
    {
      address = InetAddress.getByName(ipAddress);
    }
    catch(UnknownHostException exception) {}

    if (address != null)
    {
      ws_host = address.getHostName();
    }

    //create the queue that will hold MOs for us to investigate
    if (queue == null)
    {
      queue = new QueueBatch("BCWebServerPingQueue");
    }

    // Allow this object to be deleted from the console
    setConsoleDeletable(true);
    // Allow this object to be renamed from the console
    setConsoleRenamable(true);

    try
    {
      TWGRas.entryExit(TWGRas.EX7,
                       "BCWebServerManagedObject constructor: object name = "
                       + getName());
      // save the data in the persistent store
      save();
    }
    catch (TWGPersistentObjectSaveException pose)
    {
      TWGRas.debug(TWGRas.EX7,
                   "BCWebServerManagedObject constructor: TWGPersistentObjectSaveException, object name = "
                   + getName());
    }

    // debug statements
    try
    {
      TWGRas.debug(TWGRas.EX7,
                   "BCWebServerManagedObject constructor: objectID() = " + this.ObjectID());
    }
    catch (InvalidObjectIDException ioie)
    {
      TWGRas.error(TWGRas.EX7,
                   "BCWebServerManagedObject constructor: invalid object ID");
    }

    // add this web server to the list of web servers we've created
    TWGRas.debug(TWGRas.EX7,
                 "BCWebServerManagedObject constructor:  adding web server to the server list: "
                 + this.getName() + ", address: " + ipAddress);
    webserverList.put(ipAddress, this);
  }

  /**
   * Copy instance data to another managed object.
   * @param managedObject target managed object.
   */
  protected void copyTo(TWGManagedObject managedObject)
  {
      super.copyTo(managedObject);
      if (managedObject instanceof BCWebServerManagedObject) {
          BCWebServerManagedObject webServer =
              (BCWebServerManagedObject) managedObject;
          webServer.ws_host = this.ws_host;
          webServer.ws_ipaddress = this.ws_ipaddress;
      }
  }

  /**
   * Destructor for BCWebServerManagedObject: remove object from lists
   */
  public void Delete()
  {
    webserverList.remove(ws_ipaddress);
    super.Delete();
  }

  /**
   * Destroy for BCWebServerManagedObject
   * @exception TWGObjectDestroyException if error during destroy
   */
  public void Destroy() throws TWGObjectDestroyException
  {
    super.Destroy();
  }

  /**
   * Method for saving persistent object data.
   * @param dictionary used to collect persistent object.
   * @exception TWGPersistentObjectSaveException
   *                   thrown to abort save procedure.
   */
  protected void saveData(TWGPersistentObjectDictionary dictionary)
    throws TWGPersistentObjectSaveException
  {
     TWGRas.entryExit(TWGRas.EX7,
                      "BCWebServerManagedObject.saveData(): entry, ipaddress = "
                      + ws_ipaddress);
     super.saveData(dictionary);

     TWGByteArrayOutputStream buffer = new TWGByteArrayOutputStream();

     // Save the version, host name, and IP address
     // to persistent storage.  We'll use this
     // information to rebuild our list of web
     // servers whenever Director is restarted.
     buffer.writeInt(VERSION_ONE);
     buffer.writeUTF(ws_host);
     buffer.writeUTF(ws_ipaddress);

     TWGRas.debug(TWGRas.EX7,
                  "BCWebServerManagedObject.saveData(): bytes saved = " + buffer.size() +
                  ", byte buffer = " + IntelByteBuffer.BytesToString(buffer.toByteArray()));

     /* Add record to dictionary */
     dictionary.put(CLASSNAME, buffer.toByteArray());
  }

  /**
   * Method for restoring persistent object data.
   *
   * @param dictionary used to restore persistent object.
   * @param resolveObjectReferences
   *                   used to indicate if object references should be
   *                   resolved during restore.
   *
   * @exception TWGPersistentObjectRestoreException
   *                   thrown to abort restore procedure.
   */
  protected void restoreData(TWGPersistentObjectDictionary dictionary,
                             boolean resolveObjectReferences)
      throws TWGPersistentObjectRestoreException
  {
     TWGRas.entryExit(TWGRas.EX7, "BCWebServerManagedObject.restoreData(): entry");

     /* Restore base class */
     super.restoreData(dictionary, resolveObjectReferences);

     if (resolveObjectReferences == false)
     {
        TWGByteArrayInputStream buffer = new TWGByteArrayInputStream(dictionary.get(CLASSNAME));

        // Get the version number so we know what other data is stored.
        // If the data that is saved to persistent storage is ever
        // changed, for example, if an additional variable is stored,
        // then the version number must change.
        int version = buffer.readInt();
        switch (version)
        {
          case VERSION_ONE:
          {
            try
            {
               ws_host = new String(buffer.readUTF());
               ws_ipaddress = new String(buffer.readUTF());
               TWGRas.debug(TWGRas.EX7,
                            "BCWebServerManagedObject.restore(): "
                            + getName() + " ipAddress = " + ws_ipaddress);
            }
            catch (UTFDataFormatException exception)
            {
              throw new TWGPersistentObjectRestoreException();
            }
            break;
          }
          case VERSION_TWO:
          case VERSION_THREE:
          default:
          {
            break;
          }
        }
        setState(TWGManagedObjectConstants.MOSTATE_UNKNOWN);
     }
     else
     {
        TWGRas.error(TWGRas.EX7,
                     "BCWebServerManagedObject.restore(): adding restored object to web server list: "
                     + ws_host);
        webserverList.put(ws_ipaddress, this);
        startPing();
     }
  }

  /**
   * Get a web server with a given address.
   * @param address of the web server to find
   * @return BCWebServerManagedObject the web server with the specified address
   */
  protected static BCWebServerManagedObject getWebServer(String address)
  {
    TWGRas.entryExit(TWGRas.EX7,
                     "BCWebServerManagedObject.getWebServer(): entry, address = "
                     + address);

    return (BCWebServerManagedObject) webserverList.get((Object) address);
  }

  /**
   * Get a list of all the web servers.
   * @param address of the web server to find
   * @return BCWebServerManagedObject the web server with the specified address
   */
  protected static Enumeration getWebServerList()
  {
    TWGRas.entryExit(TWGRas.EX7,
                     "BCWebServerManagedObject.getWebServerList(): entry");

    return (Collections.enumeration(webserverList.values()));
  }

  /**
   * Get the IP address for this object
   * @return address the IP address in the form of a String
   */
  protected String getAddress()
  {
    TWGRas.entryExit(TWGRas.EX7,
                     "BCWebServerManagedObject.getAddress(): entry");
    return ws_ipaddress;
  }


  /**
   * Start ping sequence for web server managed objects
   * @return boolean TRUE: Ping function started
   */
  public boolean startPing()
  {
     TWGRas.entryExit(TWGRas.EX7,
                      "BCWebServerManagedObject.startPing(): entry, "
                      + getName() + ", " + getAddress());

     BCWebServerPingRequest qe = new BCWebServerPingRequest(this);
     queue.enqueue(qe);
     return true;
  }

  /**
   * Start ping sequence for web server managed objects
   * @return boolean TRUE: Ping function started
   */
  public static boolean startPing(TWGManagedObject mo)
  {
     TWGRas.entryExit(TWGRas.EX7,
                      "BCWebServerManagedObject.startPing(): entry with one param, "
                      + mo.getName() + ", " + mo.getAddressEntryRecord());

     return TWGManagedObject.startPing(mo);
  }

  /**
   * Start ping sequence for web server managed objects
   * @return boolean TRUE: Ping function started
   */
  public static boolean startPing(TWGManagedObject mo, boolean reauth)
  {
     TWGRas.entryExit(TWGRas.EX7,
                      "BCWebServerManagedObject.startPing(): entry with two params, "
                      + mo.getName() + ", " + mo.getAddressEntryRecord());
     return TWGManagedObject.startPing(mo, reauth);
  }

  /**
   * Default ping with reauthenticate method for SNMP device.
   * If not overridden, just calls SNMP device's startPing method
   * which may have been overridden.
   *
   * @return TRUE: Ping function supported. FALSE: Function not supported
   *         for this instance.
   */
   public boolean startPingWithReauthenticate()
   {
     TWGRas.entryExit(TWGRas.EX7,
                      "BCWebServerManagedObject.startPingWithReauthenticate(), "
                      + getName() + ", " + getAddressEntryRecord());

     return startPing();
   }


   /**
    * Attribute ID enumeration : this method is expected to return a
    * List of strings containing the String names of the attributes
    * supported for a given object.  Subclasses of base classes implementing
    * the interface should report additional attributes by adding them
    * to the list returned by the base class.
    *
    * @returns list of attribute ID strings
    */
   public String[] getAttributeIDList()
   {
      TWGRas.entryExit(TWGRas.EX7,
                       "BCWebServerManagedObject.getAttributeIDList(), name = " + getName());
      String[] s = super.getAttributeIDList();
      String[] new_s = new String[s.length + attrib_ids.length];
      System.arraycopy(s, 0, new_s, 0, s.length);
      System.arraycopy(attrib_ids, 0, new_s, s.length, attrib_ids.length);
      return new_s;
   }

    /**
     * Get value of attribute with given ID
     * @param id - attribute ID requested
     * @param loc - locale to use for value, if applicable
     * @return value object, or null if no value
     */
    public DataValue getAttributeValue(String id, Locale loc)
    {
      TWGRas.entryExit(TWGRas.EX7,
                     "BCWebServerManagedObject.getAttributeValue(): id = " + id +
                      ", name = " + getName());

      if(id.equals(attrib_ids[0]))
      {
        TWGRas.debug(TWGRas.EX7,
                     "BCWebServerManagedObject.getAttributeValue(): returning host name: "
                     + ws_host);
        TWGStringArray hostList = new TWGStringArray();
        hostList.appendValue(ws_host);
        return hostList;
      }
      else if(id.equals(attrib_ids[1]))
      {
        TWGRas.debug(TWGRas.EX7,
                     "BCWebServerManagedObject.getAttributeValue(): returning ip address: "
                     + ws_ipaddress);
        TWGStringArray addressList = new TWGStringArray();
        addressList.appendValue(ws_ipaddress);
        return addressList;
      }
      else
      {
        TWGRas.debug(TWGRas.EX7,
                     "BCWebServerManagedObject.getAttributeValue(): getting attribute from parent class, id = " + id);
        return super.getAttributeValue(id, loc);
      }
    }

    /**
     * Set value of attribute with given ID
     * @param id - attribute ID to be set
     * @param val - attribute value to be set
     * @return true if set successful, false if not successful
     */
    public boolean setAttributeValue(String id, DataValue val)
    {
      return super.setAttributeValue(id, val);
    }

    /**
     * Get type of attribute value for given attribute
     * @param id - attribute ID
     * @return DataValue type of attribute, or DataValue.NONE_TYPE if
     * undefined
     */
    public int getAttributeType(String id)
    {
      for(int i = 0; i < attrib_types.length; i++)
      {
        if(id.equals(attrib_ids[i]))
        {
            return attrib_types[i];
        }
      }
      return super.getAttributeType(id);
    }

    /**
     * Get description string for given attribute in given locale
     * @param id - attribute ID
     * @param loc - locale to use for description
     * @return description string, or null if not available
     */
    public String getAttributeDescription(String id, Locale loc)
    {
      for(int i = 0; i < attrib_types.length; i++)
      {
        if(id.equals(attrib_ids[i]))
        {
          return attrib_labels[i];
        }
      }
      return super.getAttributeDescription(id, loc);
    }

}
