/******************************************************************************
 * Licensed Materials - Property of IBM                                       *
 *                                                                            *
 * (C) Copyright by IBM 2003.  All Rights Reserved.                           *
 *****************************************************************************/

package com.BobCo.webobject;

import com.tivoli.twg.engine.InvalidObjectIDException;
import com.tivoli.twg.engine.TWGExtension;
import com.tivoli.twg.engine.TWGExtensionInitException;
import com.tivoli.twg.engine.TWGExtensionTermException;
import com.tivoli.twg.engine.TWGFilter;
import com.tivoli.twg.engine.TWGFilterConstants;
import com.tivoli.twg.engine.TWGFilterException;
import com.tivoli.twg.engine.TWGFilterParms;
import com.tivoli.twg.engine.TWGImageSet;
import com.tivoli.twg.engine.TWGManagedObject;
import com.tivoli.twg.engine.TWGPersistentObjectSaveException;
import com.tivoli.twg.engine.TWGTask;
import com.tivoli.twg.engine.TWGTaskCreationException;
import com.tivoli.twg.engine.TWGDefaultTask;
import com.tivoli.twg.libs.StringValueSet;
import com.tivoli.twg.log.TWGRas;
import java.util.Properties;


/**
 * Extension registration class for BobCo web server managed
 * object classes.  This class is registered with Director by
 * the <code>BCWebServerManagedObject.TWGExt</code> file.
 *
 * @see com.tivoli.twg.engine.TWGExtension
 */
public class BCWebServerManagedObjectExtension extends TWGExtension

{
  BCWebServerLifeCycleListener listener = null;


  /**
   * Constructor.  This doesn't do much except log a message to
   * indicate that the class was called by Director.
   *
   * @exception TWGExtensionInitException if a fatal error occurs
   * during the initialization method
   */
  public BCWebServerManagedObjectExtension()
  {
    super();

    try
    {
      TWGRas.entryExit(TWGRas.EX7,
                       "BCWebServerManagedObjectExtension constructor: object id = "
                       + ObjectID());
    }
    catch (InvalidObjectIDException ioidx)
    {
      TWGRas.error(TWGRas.EX7, "<invalid>");
    }
  }

  /**
   * First initialization method, intended to allow extensions to
   * register classes.
   *
   * @exception TWGExtensionInitException if a fatal error occurs
   * during the initialization method
   */
  public void InitClassRegistration() throws TWGExtensionInitException
  {
    TWGRas.entryExit(TWGRas.EX7,
                     "BCWebServerManagedObjectExtension.InitClassRegistration(): entry");

    String c = "";

    /* Now register all of our persistent object classes here */
    try
    {
      c = "com.BobCo.webobject.BCWebServerManagedObject";
      RegisterClass(c);
      c = "com.BobCo.webobject.BCWebServerManagedObjectFactory";
      RegisterClass(c);
    }
    catch (ClassNotFoundException exception)
    {
      TWGRas.entryExit(TWGRas.EX7,
                       "BCWebServerManagedObjectExtension.InitClassRegistration(): Class '"
                       + c + "' not found: " + exception);
      throw new TWGExtensionInitException("Class '" + c + "' not found: " + exception);
    }
  }


  /**
   * Second initialization method, intended to allow extension to create
   * default instances of classes registered during InitClassRegistration().
   */
  public void InitClassInstances() throws TWGExtensionInitException
  {
     TWGRas.entryExit(TWGRas.EX7,
                      "BCWebServerManagedObjectExtension.InitClassInstances(): entry");
     if (!TWGTask.isTaskID("BobCo|WebServerManagedObject"))
     {
        try
        {
           TWGDefaultTask testTask =
               new TWGDefaultTask("com/BobCo/webobject/BCWebServerManagedObject.properties");
        }
        catch (TWGTaskCreationException tce)
        {
           tce.printStackTrace ();
           TWGRas.error(TWGRas.EX7,
                        "BCWebServerManagedObjectExtension.InitClassInstances(): TWGTaskCreationException "
                        + tce);
           throw new TWGExtensionInitException("Error creating BCWebServerManagedObject default task");
        }
     }

     TWGRas.debug(TWGRas.EX7, "BCWebServerManagedObjectExtension.InitClassInstances():  created default task");

     // Create our listener for receiving notifications when
     // a new managed object is created or destroyed.
     if (listener == null)
     {
       // lji -- should this be created by our task server instead?
       listener = new BCWebServerLifeCycleListener();

       // Register our listener
       TWGManagedObject.RegisterLifeCycleListener(listener);
       TWGManagedObject.RegisterStateListener(listener);
     }
  }

  /**
   * Final initialization method, intended to allow an extension to do any
   * actions which may have required the other extensions to have completed
   * the previous passes of initialization.
   *
   * @exception TWGExtensionInitException if a fatal error occurs during
   * the initialization method
   */
  public void InitCompletion() throws TWGExtensionInitException
  {
  }

  /**
   * First termination method, intended to allow an extension to begin
   * the process of termination.  The only termination processing which
   * should occur here is that which needs to be done before other
   * extensions have executed TermCompletion() pass.
   *
   * @exception TWGExtensionTermException if an error occurs during the
   * termination method
   */
  public void TermBegin() throws TWGExtensionTermException
  {
    // Deregister our listener
    TWGManagedObject.UnregisterLifeCycleListener(listener);
    TWGManagedObject.UnregisterStateListener(listener);
  }

  /**
   * Final termination method, intended to allow an extension to terminate
   * any processing which may still be active.
   *
   * @exception TWGExtensionTermException if an error occurs during the
   * termination method
   */
  public void TermComplete() throws TWGExtensionTermException
  {
  }
} /* class BCWebServerManagedObjectExtension */

