package com.BobCo.FileExplorer;

import java.io.*;
import java.awt.*;
import java.net.*;
import java.lang.*;
import java.util.*;
import java.beans.*;
import java.applet.*;
import java.awt.event.*;

import javax.swing.*;
import javax.swing.border.*;

import com.tivoli.uif.core.*;
import com.tivoli.uif.controls.*;
import com.tivoli.twg.engine.*;
import com.tivoli.twg.console.*;
import com.tivoli.twg.libs.*;

/**
 * This class is extends TWGTaskFrame to provide the GUI for BobCo's file
 * explorer.  It gets instantiated and launched as appropriate by IT Director's
 * console.
 */
public class BobCoFileExplorerGUI
       extends TWGTaskFrame
       implements ActionListener {

   // create the service node
   ServiceNode sn = null;
   BobCoFileExplorerTree tree = null;

   /**
    * TWGTaskFrame - This is the constructor for the tasks frame.
    */
   public BobCoFileExplorerGUI() {
      super( true );
	  System.out.println("BobCoFileExplorerGUI constructor"
                            +" to log\\com.tivoli.console.ConsoleLauncher.output");
   }

   /**
    * Initialization routine for setting up non-gui items.
    */
   public boolean pInit( ) {
	  System.out.println("BobCoFileExplorerGUI.pInit()"
                            +" to log\\com.tivoli.console.ConsoleLauncher.output");
      boolean rc = true;
      try {
         sn = new ServiceNode();
      } catch (BadServiceNodeImplException snix) {
         System.out.println("BobCo GUI Error:  BadServiceNodeImplException while creating ServiceNode");
         rc = false;
      } catch (ServiceNodeClosedException sncx) {
         System.out.println("BobCo GUI Error:  ServiceNodeClosedException while creating ServiceNode");
         rc = false;
      } catch (ServiceNodeException snx) {
         System.out.println("BobCo GUI Error:  ServiceNodeException while creating ServiceNode");
         rc = false;
      }
      return rc;
   } // end pInit

   /**
    * Routine to allow task frames to clean up.
    */
   public void pCleanUp( ) {
	  System.out.println("BobCoFileExplorerGUI.pCleanUp()");
   }

   /**
    * Notification method to allow the task frame to know when the
    * server portion of a task has died so that it can take any
    * appropriate action.
    */
   public void serverTaskEnded() {
	  System.out.println("BobCoFileExplorerGUI.serverTaskEnded()");
   }

   /**
    * Routine to allow task frames to prompt the user if they
    * should close or not.  Return true to close the frame, return
    * false to stop the frame from closing.
    */
   public boolean verifyOnClose( int cancelType ) {
	  System.out.println("BobCoFileExplorerGUI.verifyOnClose()");
      int confirm = JOptionPane.showConfirmDialog( this,
                                                   "Do you really wish to quit?", "Verify on close",
                                                   JOptionPane.YES_NO_OPTION, JOptionPane.WARNING_MESSAGE );

      if ( confirm == JOptionPane.YES_OPTION )
         return true;

      return false;
   }


   /**
    * Create the client pane and return it.
    */
   public Container buildView( ) {
	  System.out.println("BobCoFileExplorerGUI.buildView()");
      // Set the frame's icon to the flashlight
      setIconImage(UFImageLoader.getImage("/com/BobCo/FileExplorer/images/find16.gif"));
      // Construct the top level client area pane that all panes are
      // placed into.  This code is provided as a temp for frames.
      JPanel returnPanel = new JPanel( );
      returnPanel.setBorder( new BevelBorder( BevelBorder.LOWERED ));
      returnPanel.setLayout(new BorderLayout());

      // Load and set the frame's title from the task resources.
      TWGTaskActivator ta = getTaskActivator();
      setTitle(ta.getTask().getName()+ta.getTargetTitle());
      LongValueSet moidset = ta.getMoid();
	  System.out.println("BobCoFileExplorerGUI.buildView() moidset.Length()="+ moidset.Length());
	  for (int i=0; i<moidset.Length(); i++ ) {
           System.out.println(" moidset.GetValue("+ i +")="+ moidset.GetValue(i));
      }

      /* Put the Tree in a scroller. */
      tree = new BobCoFileExplorerTree(this, moidset.GetValue(0));
      JScrollPane        sp = new JScrollPane();
      sp.setPreferredSize(new Dimension(300, 300));
      sp.getViewport().add(tree);

      /* And show it. */
      returnPanel.setLayout(new BorderLayout());
      returnPanel.add("Center", sp);

      // Create a horizontal panel for the buttons.
      JPanel buttonPane = new JPanel();
      buttonPane.setLayout(new BoxLayout(buttonPane, BoxLayout.X_AXIS));

      // Put glue in first so that all buttons are justified left.
      buttonPane.add(Box.createHorizontalGlue());

      // Create cancel button and add to button panel.
      JButton cancelBtn = new JButton( "Cancel" );
      buttonPane.add( cancelBtn );
      cancelBtn.addActionListener( this );

      returnPanel.add( "South", buttonPane );

      return( returnPanel );
   } // end buildView

   /**
    * Initial frame size to use the first time the frame is
    * created.  Subsequent invocations of the same console
    * will use the cached value (which may be different if
    * the user resized the window.
    */
   public Dimension getFrameSize() {
	  System.out.println("BobCoFileExplorerGUI.getFrameSize()");
      return new Dimension( 200, 240 );
   }

   /**
    * Action handler for any menu or button actions executed.
    */
   public void actionPerformed( ActionEvent evt ) {
	  System.out.println("BobCoFileExplorerGUI.actionPerformed()");
      // Request this window to shutdown for either ok or cancel.
      consoleCancel();

   } // end actionPerformed

   /**
    * A utility method to send a command to the IT Director server.  The
    * command should already be completely initialized.
    * @param  cmd     the Command to send to the server.
    * @return boolean true if no errors were encountered during the SendCommand()
    */
   public boolean sendToServer(Command cmd) {
	  System.out.println("BobCoFileExplorerGUI.sendToServer()");
      boolean rc = true;
      // send the command
      try {
         sn.SendCommand(cmd);
      } catch (BadServiceNodeImplException snix) {
         System.out.println("BobCo GUI Error:  BadServiceNodeImplException while sending command to server");
         rc = false;
      } catch (ServiceNodeClosedException sncx) {
         System.out.println("BobCo GUI Error:  ServiceNodeClosedException while sending command to server");
         rc = false;
      } catch (ServiceNodeException snx) {
         System.out.println("BobCo GUI Error:  ServiceNodeException while sending command to server");
         rc = false;
      } catch (InterruptedException ix) {
         System.out.println("BobCo GUI Error:  InterruptedException while sending command to server");
         rc = false;
      }
      return rc;
   }

   /**
    * A utility method to asynchronously send a command to the IT Director
    * server.  The command should already be completely initialized.
    * @param  cmd     the Command to send to the server.
    * @return boolean true if no errors were encountered during the
    *                 SendAsynchCommand()
    */
   public boolean sendAsynchToServer(Command cmd) {
	  System.out.println("BobCoFileExplorerGUI.sendAsynchToServer()");
      boolean rc = true;
      // send the command
      try {
         sn.SendAsynchCommand(cmd);
      } catch (BadServiceNodeImplException snix) {
         System.out.println("BobCo GUI Error:  BadServiceNodeImplException while sending command to server");
         rc = false;
      } catch (ServiceNodeClosedException sncx) {
         System.out.println("BobCo GUI Error:  ServiceNodeClosedException while sending command to server");
         rc = false;
      } catch (ServiceNodeException snx) {
         System.out.println("BobCo GUI Error:  ServiceNodeException while sending command to server");
         rc = false;
      } catch (InterruptedException ix) {
         System.out.println("BobCo GUI Error:  InterruptedException while sending command to server");
         rc = false;
      }
      return rc;
   }

   /**
    * Get the BobCoFileExplorerTree created by this GUI.
    * @return a BobCoFileExplorerTree
    */
   BobCoFileExplorerTree getTree() {
	  System.out.println("BobCoFileExplorerGUI.getTree()");
      return tree;
   }
} // end class TWGTaskFrame
