package com.BobCo.FileExplorer;
import  com.tivoli.twg.engine.TWGTaskConstants;
import  com.tivoli.twg.engine.TWGTaskServer;
import  com.tivoli.twg.engine.TWGTask;
import  com.tivoli.twg.engine.TWGTaskCreationException;
import  com.tivoli.twg.engine.TWGRemoteTaskManager;
import  com.tivoli.twg.engine.TWGRemoteTaskActivation;
import  com.tivoli.twg.engine.TWGTaskActivation;
import  com.tivoli.twg.engine.TWGLocalTaskClients;
import  com.tivoli.twg.engine.TWGRemoteManagedObject;
import  com.tivoli.twg.engine.RemoteMOLifeCycleListener;
import  com.tivoli.twg.engine.RemoteMOStateChangeListener;
import  com.tivoli.twg.libs.BadServiceNodeImplException;
import  com.tivoli.twg.libs.Command;
import  com.tivoli.twg.libs.CommandCompleteListener;
import  com.tivoli.twg.libs.CommandReceivedListener;
import  com.tivoli.twg.libs.IntelByteBuffer;
import  com.tivoli.twg.libs.LongKeyTable;
import  com.tivoli.twg.libs.LongValueSet;
import  com.tivoli.twg.libs.ServiceNode;
import  com.tivoli.twg.libs.ServiceNodeClosedException;
import  com.tivoli.twg.libs.ServiceNodeException;
import  com.tivoli.twg.libs.ServiceNodeLocalImplFactory;
import  com.tivoli.twg.log.TWGRas;
import  java.io.IOException;
import  java.util.Enumeration;

/**
 * This class is the main class for the BobCo file explorer task server.  It
 * gets launched by the Director server as appropriate 
 */
public class BobCoFileExplorerServer
       implements CommandReceivedListener,
                  TWGTaskServer,
                  BobCoFileExplorerConstants {

   private static boolean keepGoing = true;
   private static ServiceNode sn;

   public BobCoFileExplorerServer() {
      System.out.println("BobCoFileExplorerServer constructor");

      Exception except = null;
      // Bring up the service node
      try {
          sn = new ServiceNode(serviceNodeName);
          BobCoTaskActivation.setServiceNode(sn);
          sn.RegisterDefaultCommandListener(this); 
                                                   
          keepGoing = sn.Create();
          if (keepGoing) {
             System.out.println("BobCoFileExplorerServer constructor Local service node created, name = "+sn.Name());
          } else {
             System.out.println("BobCoFileExplorerServer constructor Local service node create failed.");
          }
      } catch(BadServiceNodeImplException snix) {
         System.out.println("BobCoFileExplorerServer constructor caught BadServiceNodeImplException "+ snix);
         except = snix;
      } catch(ServiceNodeClosedException sncx) {
         System.out.println("BobCoFileExplorerServer constructor caught ServiceNodeClosedException "+ sncx);
         except = sncx;
      } catch(ServiceNodeException snx) {
         System.out.println("BobCoFileExplorerServer constructor caught ServiceNodeException "+ snx);
         except = snx;
      }
      if (except != null) {
         System.out.println("BobCoFileExplorerServer constructor Exception encountered while creating service node");
         except.printStackTrace();
      }
   }

   public boolean serverActivate(TWGTask task) {
      System.out.println("BobCoFileExplorerServer.serverActivate()");
      task.setServiceNode(serviceNodeName);
      return true;
   }

   /**
    * Command received callback.  This listener method is called when a
    * Command is received matching one of the command codes registered for
    * the given listener
    * @param sn - ServiceNode instance which received Command
    * @param cmd - Instance of Command received
    * @return true if ServiceNode should stay active, false if it should
    * terminate
    */
   public boolean CommandReceived(ServiceNode sn, Command cmd) {
      System.out.println("BobCoFileExplorerServer.CommandReceived()\n"
                        +" cmd="+ cmd);

      long actID = IntelByteBuffer.ReadLong64(cmd.InputParm(0), 0);
      System.out.println("BobCoFileExplorerServer.CommandReceived() actID="+ actID);
      BobCoTaskActivation act = BobCoTaskActivation.findActivation(actID);
      System.out.println("BobCoFileExplorerServer.CommandReceived()\n act="+ act);

      if (act != null) {
         switch ((int)cmd.CommandCode()) {
            case consoleRequestLocalFixedDrives:
               new GetClientLocalFixedDrivesCommand(act, cmd);
               break;

            case consoleRequestDirectoryContents:
               new GetClientDirectoryContentsCommand(act, cmd);
               break;

            default:
               System.out.println("Unexpected command code received:  "+cmd.CommandCode());
               cmd.SetReturnCode(RETURN_UNRECOGNIZED_COMMAND_CODE);
               break;
         }
      } else {
         cmd.SetReturnCode(RETURN_UNRECOGNIZED_ACT_ID);
      }

      return true;
   }

   /**
    * This method is called when a subtask activation has occurred.
    * @param act the TWGTaskActivation object containing the task activation
    *            information.
    */
   public int subtaskActivate(TWGTaskActivation act) {
      System.out.println("BobCoFileExplorerServer.subtaskActivate()");

      int rc = TWGTask.SUBTASK_ACT_COMPLETE;
      System.out.println("BobCoFileExplorerServer.subtaskActivate() act.getTitle()="+ act.getTitle());

      String subtaskID = act.getSubtask().getID();
      System.out.println("BobCoFileExplorerServer.subtaskActivate() act.getSubtask().getID()="+ act.getSubtask().getID());

	  
      if (subtaskID.equals("ExploreFiles")) {
         System.out.println("BobCoFileExplorerServer.subtaskActivate() ExploreFiles activation received:");
         TWGLocalTaskClients clients = act.getClients();
		 Object[] allClients = clients.getAllClients();
		 if (allClients.length != 1) {
            System.out.println("Error: activation must be targeted for exactly one client");
		 rc = TWGTask.SUBTASK_ACT_FAILED;
         } else {
            BobCoTaskActivation myAct = new BobCoTaskActivation(act);
         }
      } else {
         System.out.println("Unrecognized subtask ID.");
		 rc = TWGTask.SUBTASK_ACT_FAILED;
      }
	  return rc;
   }

   /**
    * This method is called when a subtask deactivation has occurred.
    * @param act the TWGTaskActivation object containing the task activation
    *            information.  This is the same object that was passed to the
    *            subtaskActivate() method.
    */
   public void subtaskDeactivate(TWGTaskActivation act) {
      long actID = act.getServerActID();
      System.out.println("BobCoFileExplorerServer.subtaskDeactivate() Subtask deactivation received:");
	  System.out.println("imageSet="+ act.getTask().getImageSet() +" serverActID="+ act.getServerActID()
	                                                              +" consoleActID="+ act.getConsoleActID());
	  long[] allTaskIdentifiers = TWGTask.getAllTaskIdentifiers();
	  for (int i=0; i<allTaskIdentifiers.length; i++) {
	      System.out.println("  allTaskIdentifiers["+ i +"]="+ allTaskIdentifiers[i]);
	  }
      System.out.println("taskID = "+act.getTask().getID()+", subtaskID = "+act.getSubtask().getID());
      BobCoTaskActivation.deleteActivation(actID);
   }

   /**
    * The task server has been deactivated and any clean up required should be
    * done during this method.  After this method is called, the task server's
    * JVM will terminate.
    */
   public void serverDeactivate() {
      System.out.println("Server deactivation received:");
      keepGoing = false;
      System.out.println("   keepGoing = "+keepGoing);
      synchronized (this) {
         notifyAll();
      }
      System.out.println("   object notified, main loop should exit now");
   }

   /**
    * Since this task server gets launched in its own JVM by Director's
    * server, it must have a main() method.
    */
   public void run() {
      System.out.println("BobCoFileExplorerServer.run()");
      TWGRas.initialize();

      System.out.println("BobCoFileExplorerServer.run() ready for subcommands...");

      System.out.println("BobCoFileExplorerServer.run() thread waiting...");
      while (keepGoing) {
         synchronized (this) {
            try {
               // This is where the task's run thread would go to work
               wait();
            } catch (InterruptedException ix) {
               System.out.println("BobCoFileExplorerServer.run() InterruptedException encountered during wait.");
            }
         }
      }
      System.out.println("BobCoFileExplorerServer.run() thread wait finished");

      sn.DeregisterDefaultCommandListener();
      try{
         sn.Close();
      }
      catch(ServiceNodeException err){
         System.out.println("BobCoFileExplorerServer.run() Exception encountered while closing service node");
         err.printStackTrace();
      }
      sn = null;
      System.out.println("BobCoFileExplorerServer.run() exiting.");
   }
}


