
package com.BobCo.inventory;


import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.util.Vector;
import com.tivoli.twg.libs.*;
import com.tivoli.twg.log.*;
import com.tivoli.twg.inventory.TWGInvCollector;
import com.tivoli.twg.engine.*;
import java.sql.Timestamp;


public class BobCoSampleInventoryCollector extends TWGInvCollector
{

    BobCoSampleInventoryCollectorSession CurrentSession = null;
    String ipcPath = null;

    // The list of classes we would like the collector to create default
    // queries entries for, show in the ITD inventory GUI.
    private static final String[] makeQueryDatabaseTableClassNames = {
       	"com.BobCo.inventory.BobCoDeviceTable",
    };
    
    // The list of classes that are associated with this MO type.
    private static final String[] databaseTableClassNames = {
	"com.BobCo.inventory.BobCoDeviceTable",
    };
    
    /**
    *  @returns the names of the tables we would like to associate with the managed
    *   object type that this collector refers to. The function will likely be
    *   deprecated, as the paradigm has shifted, and the inventory server doesn't
    *   follow this exactly.  This should be straightened out by release time.
    *   
    */
    public String[] getDatabaseTableClassNames() {
	return databaseTableClassNames;
    }
    
    /**
    * @returns a String array with the names of the database tables for which we would
    *  like a default query created.
    */
    public static String[] getQueryDatabaseTableClassNames() {
	return makeQueryDatabaseTableClassNames;
    }
    
    /**
    *  Default Constructor
    */
    public BobCoSampleInventoryCollector () {

    }

    /**
    *  @returns the pretty name of the Thread, allows for the Server to better track this thread.
    */
   public String getCollectorThreadName() {
      return(new String("BobCoInventory"));
   }


   /**
    *  Called by the server to kick off the inventory session.
    *  If there is a failure, it tells the server what type, and the corresponding
    *  failure message appears in the log & possible pop-up window. 
    */
   public void startInventory(long moid,
                              String className,
                              byte []addrInfo)
    {
	TWGRas.debug(TWGRas.INVENTORY,"Starting BobCo Inventory moid="+moid+" class="+className);
	TWGNativeAddressEntryShadow tmp = new TWGNativeAddressEntryShadow();
	try {
	    if (addrInfo != null) {
		tmp.initAddressEntry(addrInfo,0,addrInfo.length);
		ipcPath = tmp.getIPCPath();
		if (ipcPath != null) {
		    startInventoryInternal((int)moid, ipcPath);
		}
		else {
		    TWGRas.error(TWGRas.INVENTORY,"BobCo Inventory collector unable to get IPCpath info");
		    inventoryComplete(moid,TWGInvCollector.INV_COLLECTION_COMM_ERROR,0);
		    return;
		}
	    }
	    else {
		TWGRas.error(TWGRas.INVENTORY,"BobCo Inventory collector was passed null addrInfo!");
		inventoryComplete(moid,TWGInvCollector.INV_COLLECTION_COMM_ERROR,0);
		return;
	    }
	}
	catch (AddressEntryFormatException aefx)  {
	    TWGRas.error(TWGRas.INVENTORY,"BobCo Inventory collector AddressEntryFormatException",aefx);
	    inventoryComplete(moid,TWGInvCollector.INV_COLLECTION_COMM_ERROR,0);
	    return;
	}
    }
    

    /**
    *  Kicks off the internal  BobCo Inventory Sesssion.
    *
    *  @exception BobCoSampleCollectorException if a fatal error occurs during the BobCo inventory
    *  collection process.   
    */
    public void startInventoryInternal(int objId, String ipcPath) {
	try {
	    TWGRas.debug(TWGRas.INVENTORY,"Creating new BobCo internal inventory");
	CurrentSession = new BobCoSampleInventoryCollectorSession(this, objId, ipcPath);
	} catch (BobCoSampleCollectorException e) {
	    TWGRas.error(TWGRas.INVENTORY, e.toString());
	    inventoryComplete(objId, INV_COLLECTION_GENERAL_FAILURE, System.currentTimeMillis());
	}
    }
     
    /**
    * Intended for any cleanup routines specific to this collector, before it is stopped.
    */

    public void stopInventory(long objId)
    {
	TWGRas.debug(TWGRas.INVENTORY,"Stop BobCo inventory collector - objId="+objId);

    }
    
    /**
    * Intended for any shutdown routines before this collector thread is destroyed. 
    */
    public void shutdown()
    {
	TWGRas.debug(TWGRas.INVENTORY,"Shutdown BobCo inventory collector");
    }
    
}








