/******************************************************************************
 * Licensed Materials - Property of IBM                            *
 *                                                                            *
 * PID (5697-TKT)                                                             *
 *                                                                            *
 * (C) Copyright by IBM 1999. All Rights Reserved. *
 *****************************************************************************/

package com.BobCo.snapins;

import com.tivoli.twg.engine.TWGExtension;
import com.tivoli.twg.engine.TWGExtensionInitException;
import com.tivoli.twg.engine.TWGExtensionTermException;
import java.util.Properties;
import com.tivoli.twg.engine.TWGImageSet;
import com.tivoli.twg.engine.TWGSnapIn;
import com.tivoli.twg.engine.TWGSnapInManager;

/**
 * Extension registration class for BobCo sample snap in classes.
 * Extension class is registered with the <code>BobCoSnapIns.TWGExt</code>
 * file. 
 * 
 * @see com.tivoli.twg.engine.TWGExtension
 */
public class BobCoSnapInsExtension extends TWGExtension 
{
    /**
     * Constructor.
     */
    public BobCoSnapInsExtension() 
    {
        super();
    }

    /**
     * First initialization method, intended to allow extensions to register
     * classes.
     *
     * @exception TWGExtensionInitException if a fatal error occurs during
     * the initialization method
     */
    public void InitClassRegistration() throws TWGExtensionInitException 
    {
    }

    /**
     * Second initialization method, intended to allow extension to create
     * default instances of classes registered during InitClassRegistration().
     * This is where we add the code to register the snap-in's with the
     * TWGSnapInManager.
     *
     * @exception TWGExtensionInitException if a fatal error occurs during
     * the initialization method
     */
    public void InitClassInstances() throws TWGExtensionInitException 
    {
       /* Register the instance browser snap-in for Win32_ProgramGroup data */
       TWGSnapInManager.RegisterSnapIn(TWGSnapIn.SNAPIN_TYPE_TWG_CIM_INSTANCE_BROWSER,
                                       "Win32_ProgramGroup",
                                       "com.BobCo.snapins.ProgramGroupSnapIn");

       /* Register the property browser snap-in for Win32_LogicalDisk data */
       TWGSnapInManager.RegisterSnapIn(TWGSnapIn.SNAPIN_TYPE_TWG_CIM_PROPERTY_BROWSER,
                                       "Win32_LogicalDisk",
                                       "com.BobCo.snapins.LogicalDiskSnapIn");

       /* Register the group browser snap-in for "DMTF|ComponentID|*" */
       TWGSnapInManager.RegisterSnapIn(TWGSnapIn.SNAPIN_TYPE_TWG_DMI_GROUP_BROWSER,
                                       "DMTF|ComponentID|*",
                                       "com.BobCo.snapins.ComponentIDSnapIn");

       /* Register the SNMP info browser snap-in for system */ 
       TWGSnapInManager.RegisterSnapIn(TWGSnapIn.SNAPIN_TYPE_TWG_SNMP_INFO_BROWSER,
                                       "system",
                                       "com.BobCo.snapins.SystemSnapIn");

       /* Register the SNMP info browser snap-in for system */ 
       TWGSnapInManager.RegisterSnapIn(TWGSnapIn.SNAPIN_TYPE_TWG_SNMP_INFO_BROWSER,
                                       "sysDescr",
                                       "com.BobCo.snapins.SysDescrSnapIn");

    }
 
    /**
     * Final initialization method, intended to allow an extension to do any
     * actions which may have required the other extensions to have completed
     * the previous passes of initialization.
     *
     * @exception TWGExtensionInitException if a fatal error occurs during
     * the initialization method
     */
    public void InitCompletion() throws TWGExtensionInitException 
    {
    }

    /**
     * First termination method, intended to allow an extension to begin
     * the process of termination.  The only termination processing which
     * should occur here is that which needs to be done before other
     * extensions have executed TermCompletion() pass.
     *
     * @exception TWGExtensionTermException if an error occurs during the
     * termination method
     */
    public void TermBegin() throws TWGExtensionTermException 
    {
    }

    /**
     * Final termination method, intended to allow an extension to terminate
     * any processing which may still be active.
     *
     * @exception TWGExtensionTermException if an error occurs during the
     * termination method
     */
    public void TermComplete() throws TWGExtensionTermException 
    {
    }
}
