package com.ibm.sysmgt.sdk.CmdTask;

import com.tivoli.twg.engine.InvalidObjectIDException;
import com.tivoli.twg.engine.TWGExtension;
import com.tivoli.twg.engine.TWGExtensionInitException;
import com.tivoli.twg.engine.TWGExtensionTermException;
import com.tivoli.twg.engine.TWGPersistentObjectSaveException;
import com.tivoli.twg.engine.TWGTask;
import com.tivoli.twg.engine.TWGTaskCreationException;
import com.tivoli.twg.engine.TWGDefaultTask;
import com.tivoli.twg.log.TWGRas;
import  java.io.*;
import  com.tivoli.twg.libs.TWGEnvironment;
import java.util.Properties;
import java.util.Hashtable;
import com.tivoli.twg.log.TWGRas;


/**
 * Extension class : defined by the CmdTask.TWGExt property file.
 */
public class CmdTaskExtension extends TWGExtension
{  
   public static final String cmdTaskFileExtension = ".CMDExt";
   private static Hashtable cmdTasks = null;

  /**
   * Default constructor : usually does nothing, but can access
   * properties loaded from our property file.
   */
  public CmdTaskExtension()
  {
    super();
    TWGRas.debug(TWGRas.EX0,"CmdTaskExtension() constructor");
    try
    {
      TWGRas.debug(TWGRas.EX0,"CmdTaskExtension() consructor ObjectID="+ ObjectID() );
    }
    catch (InvalidObjectIDException ioidx)
    {
      TWGRas.debug(TWGRas.EX0,"<invalid>");
    }
    Properties p = getPropertySet();
    cmdTasks = new Hashtable();
    p.list(System.out); /* Dump properties                          */
    TWGRas.debug(TWGRas.EX0,"Leaving CmdTaskExtension() constructor");
  }
  /**
   * First initialization method, intended to allow extensions to register
   * classes.
   *
   * @exception TWGExtensionInitException if a fatal error occurs during
   * the initialization method
   */
  public void InitClassRegistration() throws
  TWGExtensionInitException
  {
    TWGRas.debug(TWGRas.EX0,"CmdTaskExtension.InitClassRegistration()");

    String c = "";
    try
    {    
        c = "com.ibm.sysmgt.sdk.CmdTask.CmdTaskServer";
        RegisterClass( c );
    }
    catch (ClassNotFoundException cnfx)
    {
        TWGRas.debug(TWGRas.EX0,"Class '"+ c +"' not found: throwing ClassNotFoundException");
        throw new TWGExtensionInitException("Class '" + c + "' not found: " + cnfx);
    }
    TWGRas.debug(TWGRas.EX0,"Leaving CmdTaskExtension.InitClassRegistration()");
  }

  /**
   * Second initialization method, intended to allow extension to create
   * default instances of classes registered during InitClassRegistration().
   *
   * @exception TWGExtensionInitException if a fatal error occurs during
   * the initialization method
   */
  public void InitClassInstances() throws TWGExtensionInitException
  {
    TWGRas.debug(TWGRas.EX0,"CmdTaskExtension.InitClassInstances()");
    boolean err = false;

    /*// Instantiate CmdTask server in the same JVM as Director server according to the properties file
    if (!TWGTask.isTaskID("sysmgt|CmdTask"))
    {
      try
      {
        TWGDefaultTask bobCoTask = new TWGDefaultTask("com/ibm/sysmgt/sdk/CmdTask/CmdTaskTask.properties");
      }
      catch (TWGTaskCreationException e)
      {
        TWGRas.debug(TWGRas.EX0,"Task creation exception!!!");
        e.printStackTrace();
        err = true;
      }
    }

    if (err)
    {
      throw new TWGExtensionInitException("error creating TWGDefaultTask(s)");
    }*/
    
    TWGRas.debug(TWGRas.EX0,"Leaving CmdTaskExtension.InitClassInstances()");
  }

  /**
   * Final initialization method, intended to allow an extension to do any
   * actions which may have required the other extensions to have completed
   * the previous passes of initialization.
   *
   * @exception TWGExtensionInitException if a fatal error occurs during
   * the initialization method
   */
  public void InitCompletion() throws TWGExtensionInitException
  {
    TWGRas.debug(TWGRas.EX0,"CmdTaskExtension.InitCompletion()");
    boolean err = false;
    /*- Do task association stuff here */

    
    File directory = new File(TWGEnvironment.getExtensionsDirectory());
    File[] files = directory.listFiles(new ExtensionFilter(cmdTaskFileExtension));
    File file = null;
    int i;
    for( i = 0; i < files.length; i++){
	file = files[i];
	String filename = file.getName();
	TWGRas.debug(TWGRas.EX0,"CmdTask Extensions search found " + filename);
	Properties cmdExtProps = new Properties();
        
	try {
	    FileInputStream fileIn = new FileInputStream(file);
	    cmdExtProps.load(fileIn);
	    fileIn.close();  //make sure we close and free file handle
	    fileIn = null;
	    String temp = null;
	}catch (FileNotFoundException fnfe) {
	    TWGRas.debug(TWGRas.EX0,"Exception while loading " + filename +
			       " from data directory :" + fnfe);
	    err = true;
	} catch (IOException ioe) {
	    TWGRas.debug(TWGRas.EX0,"IOException while loading " + filename + " : "+ ioe);
	    err = true;
	}
	CmdTaskData cmdTaskData = buildCmdTaskData(cmdExtProps, filename);

	if (cmdTaskData != null) {
	    Properties cmdTaskProps = buildCmdTaskProperties(cmdTaskData);

	    // Replace(delete) data in Persistent Store with data from file since file may be updated
	    if (TWGTask.isTaskID(cmdTaskData.getID())) {
		TWGRas.debug(TWGRas.EX0,"CmdTask replacing stored " + cmdTaskData.getID() + " with data from " + filename);
	        
		int rc = TWGTask.deleteTask(cmdTaskData.getID());
		switch (rc) {
		    case -1 :
			TWGRas.error(TWGRas.EX0,"CmdTaskExtension : Error deleting " + cmdTaskData.getID() +" . Task Id not recognized.");
			TWGRas.debug(TWGRas.EX0,"CmdTaskExtension : Error deleting " + cmdTaskData.getID() +" . Task Id not recognized. rc=" + rc);
			break;
		    case 0 :
			TWGRas.debug(TWGRas.EX0,"CmdTaskExtension : " + cmdTaskData.getID() +" deleted. rc=" + rc);
			break;
		    case 1 :
			TWGRas.debug(TWGRas.EX0,"CmdTaskExtension : Error deleting " + cmdTaskData.getID() +" . rc=" + rc);
			break;
		    case 2 :
			TWGRas.debug(TWGRas.EX0,"CmdTaskExtension :" + cmdTaskData.getID() + " has subtasks... Delete deferred . rc=" + rc);
			break;
		}
	    }
	    
	    // Register the parent first if it is defined 
	    if( cmdTaskData.hasParent() ) {
                String parentTaskID = cmdTaskData.getParentTaskID();
		String parentTaskTitle = cmdTaskData.getParentTaskTitle();
		// only register if it does not already exist
		if (!TWGTask.isTaskID(parentTaskID)) {
		    try {
			  Properties parentTaskProps = buildParentTaskProperties(cmdTaskData);
			  TWGDefaultTask task = new TWGDefaultTask(parentTaskProps);
			  // cmdTasks.put(cmdTaskData.getParentID(),parentTaskData);  //needn't store Parent as a Task
			  TWGRas.debug(TWGRas.EX0, "CmdTask registered parent : " + parentTaskID );
		       } catch (TWGTaskCreationException e) {
			  e.printStackTrace();
			  throw new TWGExtensionInitException("CmdTask Error instantiating " + parentTaskID + " : " + e.getMessage());
							    
			  
		       }
		    } else {
			// otherwise use the old parent
			TWGRas.debug(TWGRas.EX0,"CmdTask : ParentTask: " + parentTaskID + " already exists... using pre-existing task.");
			// do not set err = true ... this condition is fine
		    }


	    }

	    if (!TWGTask.isTaskID(cmdTaskData.getID())) {
	       TWGRas.debug(TWGRas.EX0,"CmdTaskExtension trying to register " + cmdTaskData.getID());
	       try {
		  TWGDefaultTask task = new TWGDefaultTask(cmdTaskProps);
		  cmdTasks.put(cmdTaskData.getID(),cmdTaskData);
		  TWGRas.debug(TWGRas.EX0, "CmdTask registered " + cmdTaskData.getID() );
	       } catch (TWGTaskCreationException e) {
		  e.printStackTrace();
		  err = true;
		  throw new TWGExtensionInitException("CmdTask Error instantiating " + cmdTaskData.getID() + " : " + e.getMessage());
	       }
	    } else {
		TWGRas.debug(TWGRas.EX0,"CmdTask Error: " + cmdTaskData.getID() + " Task already exists!!!");
		err = true;
	    }
	}
    }


    
    if (err)
    {
      throw new TWGExtensionInitException();
    }
  }

  /**
   * First termination method, intended to allow an extension to begin
   * the process of termination.  The only termination processing which
   * should occur here is that which needs to be done before other
   * extensions have executed TermCompletion() pass.
   *
   * @exception TWGExtensionTermException if an error occurs during the
   * termination method
   */
  public void TermBegin() throws TWGExtensionTermException
  {
    TWGRas.debug(TWGRas.EX0,"CmdTaskExtension.TermBegin()");
    boolean err = false;
    /*- Do initial termination processing here */
    if (err)
    {
      throw new TWGExtensionTermException();
    }
  }
  /**
   * Final termination method, intended to allow an extension to terminate
   * any processing which may still be active.
   *
   * @exception TWGExtensionTermException if an error occurs during the
   * termination method
   */
  public void TermComplete() throws TWGExtensionTermException
  {
    TWGRas.debug(TWGRas.EX0,"CmdTaskExtension.TermComplete()");
    boolean err = false;
    /*- Do final termination processing here */
    if (err)
    {
      throw new TWGExtensionTermException();
    }
  }

  private CmdTaskData buildCmdTaskData(Properties cmdExtProps,String filename){
    CmdTaskData cmdTaskData = new CmdTaskData();

    // strip beginning path info
    String baseFilename = filename.indexOf(File.separator) > -1 ? 
			    filename.substring(filename.lastIndexOf(File.separator) + 1) : 
			    filename;
    // strip extension
    baseFilename = baseFilename.substring(0,baseFilename.indexOf(cmdTaskFileExtension));


    if ( cmdExtProps.containsKey("Parent") )
	cmdTaskData.setParentTaskTitle((String) cmdExtProps.get("Parent") );
    if ( cmdExtProps.containsKey("TaskID") )
	cmdTaskData.setID((String) cmdExtProps.get("TaskID") );
    else
	cmdTaskData.setID(baseFilename);
    if ( cmdExtProps.containsKey("Title") )
	cmdTaskData.setTitle((String) cmdExtProps.get("Title") );
    else
	cmdTaskData.setTitle(baseFilename);

    if ( cmdExtProps.containsKey("CommandString") ){
	cmdTaskData.setWindowsCommandString((String) cmdExtProps.get("CommandString") );
	cmdTaskData.setUnixCommandString((String) cmdExtProps.get("CommandString") );
    }
    if ( cmdExtProps.containsKey("CommandString.Windows") )
	cmdTaskData.setWindowsCommandString((String) cmdExtProps.get("CommandString.Windows") );
    if ( cmdExtProps.containsKey("CommandString.Unix") )
	cmdTaskData.setUnixCommandString((String) cmdExtProps.get("CommandString.Unix") );

    if ( cmdExtProps.containsKey("ShellRequired") )
	cmdTaskData.setShellRequired( (new Boolean( (String) cmdExtProps.get("ShellRequired")) ).booleanValue() );
    else
	cmdTaskData.setShellRequired(false);

    if ( cmdExtProps.containsKey("Cwd") ){
	cmdTaskData.setWindowsCwd((String) cmdExtProps.get("Cwd") );
	cmdTaskData.setUnixCwd((String) cmdExtProps.get("Cwd") );
    }
    if ( cmdExtProps.containsKey("Cwd.Windows") )
	cmdTaskData.setWindowsCwd((String) cmdExtProps.get("Cwd.Windows") );
    if ( cmdExtProps.containsKey("Cwd.Unix") )
	cmdTaskData.setUnixCwd((String) cmdExtProps.get("Cwd.Unix") );

    if ( cmdExtProps.containsKey("URL.Type") )
	cmdTaskData.setUrlType((String) cmdExtProps.get("URL.Type") );
    if ( cmdExtProps.containsKey("URL.Port") )
	cmdTaskData.setUrlPort((String) cmdExtProps.get("URL.Port") );
    if ( cmdExtProps.containsKey("URL.File") )
	cmdTaskData.setUrlFile((String) cmdExtProps.get("URL.File") );

    if ( cmdExtProps.containsKey("CommandString.Agent.Windows") )
	cmdTaskData.setWindowsAgentCommandString((String) cmdExtProps.get("CommandString.Agent.Windows") );
    if ( cmdExtProps.containsKey("CommandString.Agent.Unix") )
	cmdTaskData.setUnixAgentCommandString((String) cmdExtProps.get("CommandString.Agent.Unix") );

    if ( cmdExtProps.containsKey("Cwd.Agent") ){
	cmdTaskData.setWindowsAgentCwd((String) cmdExtProps.get("Cwd.Agent") );
	cmdTaskData.setUnixAgentCwd((String) cmdExtProps.get("Cwd.Agent") );
    }
    if ( cmdExtProps.containsKey("Cwd.Agent.Windows") )
	cmdTaskData.setWindowsAgentCwd( (String) cmdExtProps.get("Cwd.Agent.Windows") );
    if ( cmdExtProps.containsKey("Cwd.Agent.Unix") )
	cmdTaskData.setUnixAgentCwd( (String) cmdExtProps.get("Cwd.Agent.Unix") );

    if ( cmdExtProps.containsKey("Timeout.Agent") )
	cmdTaskData.setAgentTimeout((Integer.parseInt((String) cmdExtProps.get("Timeout.Agent"))) );

    return cmdTaskData;
  }

  private Properties buildCmdTaskProperties(CmdTaskData cmdTaskData){
      Properties p = new Properties();

      p.put("TaskIconHidden","false");
      p.put("TaskUnrestricted","true");
      p.put("Icon.Small","/com/ibm/sysmgt/sdk/CmdTask/images/document.gif");
      p.put("Icon.Small.Selected","/com/ibm/sysmgt/sdk/CmdTask/images/document.gif");
      
      p.put("GUI","class:com.ibm.sysmgt.sdk.CmdTask.CmdTaskGUI");
      
      p.put("Client.0.Class","com.tivoli.twg.engine.TWGManagedObject");
      p.put("Subtask.0.Context","interactive:true | server:true | client:ignore | targeted:one");
      p.put("Client.0.Bind", "Class.soft");
      
      p.put("Subtask.0.MenuLocation","ObjectIcon");
      p.put("Subtask.0.Actions","drag-drop");
      p.put("ServiceNode.Forward","command:12561 | subtask:false | lifecycle:false | state:false");
      p.put("Server","class:com.ibm.sysmgt.sdk.CmdTask.CmdTaskServer | auto-load:true | sameJVM:true");
      String id = cmdTaskData.getID();
      String title = cmdTaskData.getTitle();
      
      p.put("TaskID", id);
      p.put("Title",title);
      p.put("Label.Menu",title + ",1");
      p.put("Label.Task.Pane",title+",1");
      p.put("Subtask.0.ID",id.substring(id.lastIndexOf('|')+1) );

      if(cmdTaskData.hasParent()){
	  p.put("ParentTaskID",cmdTaskData.getParentTaskID());
	  p.put("Subtask.0.Menu",cmdTaskData.getParentTaskTitle() + ", 450 | " + cmdTaskData.getTitle() + ",450");
      } else {
	  p.put("Subtask.0.Menu",cmdTaskData.getTitle()+",450");
      }

      return p;
  }

  private Properties buildParentTaskProperties(CmdTaskData cmdTaskData){
      Properties p = new Properties();
      String parentTaskID = cmdTaskData.getParentTaskID();
      String parentTaskTitle = cmdTaskData.getParentTaskTitle();

      p.put("TaskIconHidden","false");
      p.put("TaskUnrestricted","true");
      p.put("Icon.Small","/com/ibm/sysmgt/sdk/CmdTask/images/document.gif");
      p.put("Icon.Small.Selected","/com/ibm/sysmgt/sdk/CmdTask/images/document.gif");
      p.put("GUI","class:com.ibm.sysmgt.sdk.CmdTask.CmdTaskGUI");
      p.put("Client.0.Class","com.tivoli.twg.engine.TWGManagedObject");
      p.put("Client.0.Bind" , "Class.soft");
      p.put("Subtask.0.Menu", parentTaskTitle + ",450");
      
      p.put("Server","class:com.ibm.sysmgt.sdk.CmdTask.CmdTaskServer | auto-load:true | sameJVM:true");
      
      p.put("TaskID",parentTaskID);
      p.put("Title",parentTaskTitle);
      p.put("Label.Menu",parentTaskTitle+",1");
      p.put("Label.Task.Pane",parentTaskTitle+",1");
      p.put("Subtask.0.ID",parentTaskTitle );

      /*NO More than two levels now
      if(cmdTaskData.hasParent()){
	  p.put("ParentTaskID",cmdTaskData.getParent());
      }*/

      return p;
  }

  public static CmdTaskData getCmdTaskData(String taskID){
      if( cmdTasks.containsKey(taskID) )
	  return ( (CmdTaskData) cmdTasks.get(taskID) );
      else
	  return null;
  }

}

class ExtensionFilter implements FilenameFilter {
  String extension;
  ExtensionFilter(String extension) { 
      this.extension = extension; 
  }

  public boolean accept(File dir, String name) {
    // Strip path information:
    String f = new File(name).getName();
    return f.endsWith(extension);
  }
}

