package com.BobCo.FileExplorer;
import com.tivoli.twg.libs.Command;
import com.tivoli.twg.libs.CommandCompleteListener;
import com.tivoli.twg.libs.IntelByteBuffer;
import javax.swing.SwingUtilities;

/**
 * This subclass of Command contains fields used for correlation so that
 * the command can be sent asynchronously and the results can be handled
 * properly when they are returned.  It also implements the Runnable and
 * CommandCompleteListener interfaces so that command completion processing
 * can be handled asynchronously and back under Swing's dispatch thread.
 */
public class ConsoleRequestDirectoryContentsCommand
       extends Command
       implements BobCoFileExplorerConstants,
                  CommandCompleteListener,
                  Runnable {

   String path;
   BobCoFileExplorerTreeNode node;

   /**
    * Constructor
    * @param node   the node of the tree representing the directory
    *               that will be updated with the content information.
    * @param actID  the activation ID to send down to the server so it can
    *               find the right client to request information from.
    */
   ConsoleRequestDirectoryContentsCommand(BobCoFileExplorerTreeNode node, long actID, String path) {
      super(consoleRequestDirectoryContents);
      SetDestinationAddress(serviceNodeName);
      setCommandCompleteListener(this);
      this.path = path;
      this.node = node;
      byte[] buffer = new byte[IntelByteBuffer.SIZEOF_LONG64 +
                               IntelByteBuffer.GetCompUnicodeLength(path)];
      IntelByteBuffer.WriteLong64(buffer, actID, 0);
      IntelByteBuffer.WriteCompUnicode(buffer, path, IntelByteBuffer.SIZEOF_LONG64);
      AttachInputParm(buffer);
   }

   /**
    * Command completion callback.  This listener method is called when a Command instance
    * registered to this listener is completed.
    * @param cmd - Command instance of command which has completed
    */
   public void CommandComplete(Command cmd) {
      System.out.println("*** BobCo ConsoleRequestDirectoryContentsCommand complete, rc="+ReturnCode()+", NumOutputParms="+NumOutputParms());
      SwingUtilities.invokeLater(this);
   }

   /**
    * Implement run() so that we can invoke the node's update method safely
    * under Swing's dispatch thread via SwingUtilities.invokeLater().
    */
   public void run() {
      System.out.println("*** BobCo ConsoleRequestDirectoryContentsCommand invokeLater callback");
      node.directoryContentsCommandComplete(this);
   }
}
