package com.BobCo.FileExplorer;
import com.tivoli.twg.engine.TWGRemoteManagedObject;
import com.tivoli.twg.libs.Command;
import com.tivoli.twg.libs.CommandCompleteListener;

/**
 * This subclass of Command contains fields used for correlation so that
 * the command can be sent asynchronously and the results can be handled
 * properly when they are returned.  It also implements the
 * CommandCompleteListener interface so that it can handle its own command
 * complete processing
 */
public class GetClientLocalFixedDrivesCommand
       extends Command
       implements BobCoFileExplorerConstants,
                  CommandCompleteListener {
   BobCoTaskActivation    act;
   Command                consoleCmd;

   /**
    * Constructor
    * @param act         the BobCoTaskActivation that this command is being
    *                    issued under
    * @param consoleCmd  the ConsoleRequestDirectoryContentsCommand received
    *                    from the console that is causing this command to be
    *                    sent to the client.  Input parameters are taken from
    *                    this received Command, and output parameters received
    *                    from the client are sent back as output parameters in
    *                    the consoleCmd.
    */
   GetClientLocalFixedDrivesCommand(BobCoTaskActivation act, Command consoleCmd) {
      super(getClientLocalFixedDrives);
      this.act = act;
      this.consoleCmd = consoleCmd;
      consoleCmd.PostponeReply();
      setCommandCompleteListener(this);

      System.out.println("BobCo sending GetClientLocalFixedDrivesCommand to client");
      act.sendAsynchToClient(this);
   }

   /**
    * Command completion callback.  This listener method is called when a Command instance
    * registered to this listener is completed.
    * @param cmd - Command instance of command which has completed
    */
   public void CommandComplete(Command cmd) {
      long rc = ReturnCode();
      System.out.println("BobCo GetClientLocalFixedDrivesCommand complete, rc="+rc+", NumOutputParms="+NumOutputParms());
      consoleCmd.SetReturnCode(rc);
      if (rc == RETURN_OKAY) {
         for (int i=0; i<NumOutputParms(); i++) {
            consoleCmd.AddOutputParm(OutputParm(i));
         }
      }
      act.sendPostponedReply(consoleCmd);
   }
}
