package com.tivoli.twg.mscs;

import com.tivoli.twg.libs.*;

/**
 * Base class for MSCS (Microsoft Clustering Service) agent IPC commands
 * @author Mike Primm
 * @version 1.1 - 6/8/05
 */
public class MSCSAgentCommand extends Command {
    /** Agent service node name                                                             */
    public static final String CLUSTAGT_SVCNODE_NAME = "MSCS_Agent";
    /** Error code - not a clustered system                                                 */
    public static final long MSCSAGT_ERROR_NOT_CLUSTER = 	0x000A;

    /** MSCS Agent Command - ClusterEnum                                                    */
    public static final long MSCSAGT_ENUM_CLUSTER = 		0x4401;
    /** MSCS Agent Command - GetClusterInformation                                          */
    public static final long MSCSAGT_GET_CLUSTER_INFO = 	0x4402;
    /** MSCS Agent Command - GetClusterQuorumResource                                       */
    public static final long MSCSAGT_GET_CLUSTER_QUORUM_RES = 	0x4403;
    /** MSCS Agent Command - SetClusterName                                                 */
    public static final long MSCSAGT_SET_CLUSTER_NAME = 	0x4404;
    /** MSCS Agent Command - SetClusterQuorumResource                                       */
    public static final long MSCSAGT_SET_CLUSTER_QUORUM_RES = 	0x4405;
    /** MSCS Agent Command - ClusterNodeEnum operation                                      */
    public static final long MSCSAGT_ENUM_NODE_CLUSTER = 	0x4406;
    /** MSCS Agent Command - EvictClusterNode                                               */
    public static final long MSCSAGT_EVICT_NODE = 		0x4407;
    /** MSCS Agent Command - GetClusterNodeId                                               */
    public static final long MSCSAGT_GET_NODE_ID = 		0x4408;
    /** MSCS Agent Command - GetClusterNodeState                                            */
    public static final long MSCSAGT_GET_NODE_STATE = 		0x4409;
    /** MSCS Agent Command - GetClusterNodeState                                            */
    public static final long MSCSAGT_GET_NODE_CLUSTER_STATE = 	0x440A;
    /** MSCS Agent Command - PauseClusterNode                                               */
    public static final long MSCSAGT_PAUSE_NODE	= 		0x440B;
    /** MSCS Agent Command - ResumeClusterNode                                              */
    public static final long MSCSAGT_RESUME_NODE = 		0x440C;
    /** MSCS Agent Command - ClusterNetworkEnum                                             */
    public static final long MSCSAGT_ENUM_NETWORKS =        	0x440D;
    /** MSCS Agent Command - GetClusterNetworkId                                            */
    public static final long MSCSAGT_GET_NETWORK_ID =         	0x440E;
    /** MSCS Agent Command - GetClusterNetworkState                                         */
    public static final long MSCSAGT_GET_NETWORK_STATE = 	0x440F;
    /** MSCS Agent Command - SetClusterNetworkName                                          */
    public static final long MSCSAGT_SET_NETWORK_NAME =         0x4410;
    /** MSCS Agent Command - SetClusterNetworkPriorityOrder                                 */
    public static final long MSCSAGT_SET_NETWORK_PRIORITY_ORDER = 	0x4411;
    /** MSCS Agent Command - GetClusterNetInterface                                         */
    public static final long MSCSAGT_GET_NET_INTERFACE = 	0x4412;
    /** MSCS Agent Command - GetClusterNetInterfaceState                                    */
    public static final long MSCSAGT_GET_NET_INTERFACE_STATE = 	0x4413;
    /** MSCS Agent Command - ClusterGroupEnum operation                                     */
    public static final long MSCSAGT_ENUM_CLUSTER_GROUP = 	0x4414;
    /** MSCS Agent Command - CreateClusterGroup                                             */
    public static final long MSCSAGT_CREATE_GROUP = 		0x4415;
    /** MSCS Agent Command - DeleteClusterGroup                                             */
    public static final long MSCSAGT_DELETE_GROUP = 		0x4416;
    /** MSCS Agent Command - GetClusterGroupState                                           */
    public static final long MSCSAGT_GET_GROUP_STATE = 		0x4417;
    /** MSCS Agent Command - MoveClusterGroup                                               */
    public static final long MSCSAGT_MOVE_GROUP =		0x4418;
    /** MSCS Agent Command - OnlineClusterGroup                                             */
    public static final long MSCSAGT_ONLINE_GROUP =             0x4419;
    /** MSCS Agent Command - OfflineClusterGroup                                            */
    public static final long MSCSAGT_OFFLINE_GROUP =       	0x441A;
    /** MSCS Agent Command - SetClusterGroupName                                            */
    public static final long MSCSAGT_SET_GROUP_NAME =           0x441B;
    /** MSCS Agent Command - SetClusterGroupNodeList                                        */
    public static final long MSCSAGT_SET_GROUP_NODE_LIST =      0x441C;
    /** MSCS Agent Command - ClusterResourceEnum                                            */
    public static final long MSCSAGT_ENUM_RESOURCES =        	0x441D;
    /** MSCS Agent Command - AddClusterResourceDependency                                   */
    public static final long MSCSAGT_ADD_RESOURCE_DEPEND =      0x441E;
    /** MSCS Agent Command - AddClusterResourceNode                                         */
    public static final long MSCSAGT_ADD_RESOURCE_NODE =        0x441F;
    /** MSCS Agent Command - CanResourceBeDependent                                         */
    public static final long MSCSAGT_CAN_RESOURCE_BE_DEPEND =   0x4420;
    /** MSCS Agent Command - ChangeClusterResourceGroup                                     */
    public static final long MSCSAGT_CHANGE_RESOURCE_GROUP =    0x4421;
    /** MSCS Agent Command - CreateClusterResource                                          */
    public static final long MSCSAGT_CREATE_RESOURCE = 		0x4422;
    /** MSCS Agent Command - DeleteClusterResource                                          */
    public static final long MSCSAGT_DELETE_RESOURCE =          0x4423;
    /** MSCS Agent Command - FailClusterResource                                            */
    public static final long MSCSAGT_FAIL_RESOURCE =            0x4424;
    /** MSCS Agent Command - OnlineClusterResource                                          */
    public static final long MSCSAGT_ONLINE_RESOURCE =          0x4425;
    /** MSCS Agent Command - OfflineClusterResource                                         */
    public static final long MSCSAGT_OFFLINE_RESOURCE =         0x4426;
    /** MSCS Agent Command - RemoveClusterResourceDependency                                */
    public static final long MSCSAGT_REMOVE_RESOURCE_DEPEND =   0x4427;
    /** MSCS Agent Command - RemoveClusterResourceNode                                      */
    public static final long MSCSAGT_REMOVE_RESOURCE_NODE =     0x4428;
    /** MSCS Agent Command - GetClusterResourceNetworkName                                  */
    public static final long MSCSAGT_GET_RESOURCE_NET_NAME = 	0x4429;
    /** MSCS Agent Command - GetClusterResourceState                                        */
    public static final long MSCSAGT_GET_RESOURCE_STATE =	0x442A;
    /** MSCS Agent Command - SetClusterResourceName                                         */
    public static final long MSCSAGT_SET_RESOURCE_NAME = 	0x442B;
    /** MSCS Agent Command - GetClusterNotify history                                       */
    public static final long MSCSAGT_GET_NOTIFIES =      	0x442C;
    /** MSCS Agent Command - exit agent                                                     */
    public static final long MSCSAGT_EXIT = 			0x442D;
    /** MSCS Agent Command - ClusterNodeControl                                             */
    public static final long MSCSAGT_NODE_CONTROL = 		0x442E;
    /** MSCS Agent Command - ClusterNetworkControl                                          */
    public static final long MSCSAGT_NETWORK_CONTROL = 		0x442F;
    /** MSCS Agent Command - ClusterNetInterfaceControl                                     */
    public static final long MSCSAGT_NETINTERFACE_CONTROL = 	0x4430;
    /** MSCS Agent Command - ClusterGroupControl                                            */
    public static final long MSCSAGT_GROUP_CONTROL = 		0x4431;
    /** MSCS Agent Command - ClusterResourceControl                                         */
    public static final long MSCSAGT_RESOURCE_CONTROL = 	0x4432;
    /** MSCS Agent Command - ClusterResourceTypeControl                                     */
    public static final long MSCSAGT_RESOURCE_TYPE_CONTROL = 	0x4433;
    /** MSCS Agent Command - Enumerate all resource information                             */
    public static final long MSCSAGT_GET_FULL_CONFIG_INFO = 	0x4434;

    /** Major version - Windows NT V4                                                       */
    public static final long NT4_MAJOR_VERSION = 		1;
    /** Major version - Windows NT V4 SP4                                                   */
    public static final long NT4SP4_MAJOR_VERSION =     	2;
    /** Major version - Windows NT V5                                                       */
    public static final long NT5_MAJOR_VERSION = 		3;

    /** Cluster node state flag - installed                                                 */
    public static final long CLUSTER_INSTALLED = 		0x00000001;
    /** Cluster node state flag - running                                                   */
    public static final long CLUSTER_RUNNING =                  0x00000010;

    /** Cluster node state - not installed                                                  */
    public static final long ClusterStateNotInstalled =         0x00000000;
    /** Cluster node state - not running                                                    */
    public static final long ClusterStateNotRunning =           CLUSTER_INSTALLED;
    /** Cluster node state - running                                                        */
    public static final long ClusterStateRunning =              CLUSTER_INSTALLED | CLUSTER_RUNNING;

    /** Cluster state changes                                                               */
    public static final long CLUSTER_CHANGE_NODE_STATE               = 0x00000001;
    public static final long CLUSTER_CHANGE_NODE_DELETED             = 0x00000002;
    public static final long CLUSTER_CHANGE_NODE_ADDED               = 0x00000004;
    public static final long CLUSTER_CHANGE_NODE_PROPERTY            = 0x00000008;
    public static final long CLUSTER_CHANGE_REGISTRY_NAME            = 0x00000010;
    public static final long CLUSTER_CHANGE_REGISTRY_ATTRIBUTES      = 0x00000020;
    public static final long CLUSTER_CHANGE_REGISTRY_VALUE           = 0x00000040;
    public static final long CLUSTER_CHANGE_REGISTRY_SUBTREE         = 0x00000080;
    public static final long CLUSTER_CHANGE_RESOURCE_STATE           = 0x00000100;
    public static final long CLUSTER_CHANGE_RESOURCE_DELETED         = 0x00000200;
    public static final long CLUSTER_CHANGE_RESOURCE_ADDED           = 0x00000400;
    public static final long CLUSTER_CHANGE_RESOURCE_PROPERTY        = 0x00000800;
    public static final long CLUSTER_CHANGE_GROUP_STATE              = 0x00001000;
    public static final long CLUSTER_CHANGE_GROUP_DELETED            = 0x00002000;
    public static final long CLUSTER_CHANGE_GROUP_ADDED              = 0x00004000;
    public static final long CLUSTER_CHANGE_GROUP_PROPERTY           = 0x00008000;
    public static final long CLUSTER_CHANGE_RESOURCE_TYPE_DELETED    = 0x00010000;
    public static final long CLUSTER_CHANGE_RESOURCE_TYPE_ADDED      = 0x00020000;
    public static final long CLUSTER_CHANGE_NETWORK_STATE            = 0x00100000;
    public static final long CLUSTER_CHANGE_NETWORK_DELETED          = 0x00200000;
    public static final long CLUSTER_CHANGE_NETWORK_ADDED            = 0x00400000;
    public static final long CLUSTER_CHANGE_NETWORK_PROPERTY         = 0x00800000;
    public static final long CLUSTER_CHANGE_NETINTERFACE_STATE       = 0x01000000;
    public static final long CLUSTER_CHANGE_NETINTERFACE_DELETED     = 0x02000000;
    public static final long CLUSTER_CHANGE_NETINTERFACE_ADDED       = 0x04000000;
    public static final long CLUSTER_CHANGE_NETINTERFACE_PROPERTY    = 0x08000000;
    public static final long CLUSTER_CHANGE_QUORUM_STATE             = 0x10000000;
    public static final long CLUSTER_CHANGE_CLUSTER_STATE            = 0x20000000;
    public static final long CLUSTER_CHANGE_CLUSTER_PROPERTY         = 0x40000000;
    public static final long CLUSTER_CHANGE_HANDLE_CLOSE             = 0x80000000;
    public static final long CLUSTER_CHANGE_ALL                      = 
                                              (CLUSTER_CHANGE_NODE_STATE                |
                                               CLUSTER_CHANGE_NODE_DELETED              |
                                               CLUSTER_CHANGE_NODE_ADDED                |
                                               CLUSTER_CHANGE_NODE_PROPERTY             |
                                               CLUSTER_CHANGE_REGISTRY_NAME             |
                                               CLUSTER_CHANGE_REGISTRY_ATTRIBUTES       |
                                               CLUSTER_CHANGE_REGISTRY_VALUE            |
                                               CLUSTER_CHANGE_REGISTRY_SUBTREE          |
                                               CLUSTER_CHANGE_RESOURCE_STATE            |
                                               CLUSTER_CHANGE_RESOURCE_DELETED          |
                                               CLUSTER_CHANGE_RESOURCE_ADDED            |
                                               CLUSTER_CHANGE_RESOURCE_PROPERTY         |
                                               CLUSTER_CHANGE_GROUP_STATE               |
                                               CLUSTER_CHANGE_GROUP_DELETED             |
                                               CLUSTER_CHANGE_GROUP_ADDED               |
                                               CLUSTER_CHANGE_GROUP_PROPERTY            |
                                               CLUSTER_CHANGE_RESOURCE_TYPE_DELETED     |
                                               CLUSTER_CHANGE_RESOURCE_TYPE_ADDED       |
                                               CLUSTER_CHANGE_NETWORK_STATE             |
                                               CLUSTER_CHANGE_NETWORK_DELETED           |
                                               CLUSTER_CHANGE_NETWORK_ADDED             |
                                               CLUSTER_CHANGE_NETWORK_PROPERTY          |
                                               CLUSTER_CHANGE_NETINTERFACE_STATE        |
                                               CLUSTER_CHANGE_NETINTERFACE_DELETED      |
                                               CLUSTER_CHANGE_NETINTERFACE_ADDED        |
                                               CLUSTER_CHANGE_NETINTERFACE_PROPERTY     |
                                               CLUSTER_CHANGE_QUORUM_STATE              |
                                               CLUSTER_CHANGE_CLUSTER_STATE             |
                                               CLUSTER_CHANGE_CLUSTER_PROPERTY          |
                                               CLUSTER_CHANGE_HANDLE_CLOSE);

    /** Cluster enumeration values                                                          */
    public static final long CLUSTER_ENUM_NODE               = 0x00000001;
    public static final long CLUSTER_ENUM_RESTYPE            = 0x00000002;
    public static final long CLUSTER_ENUM_RESOURCE           = 0x00000004;
    public static final long CLUSTER_ENUM_GROUP              = 0x00000008;
    public static final long CLUSTER_ENUM_NETWORK            = 0x00000010;
    public static final long CLUSTER_ENUM_NETINTERFACE       = 0x00000020;
    public static final long CLUSTER_ENUM_INTERNAL_NETWORK   = 0x80000000;
    public static final long CLUSTER_ENUM_ALL                = 
                                      (CLUSTER_ENUM_NODE      |
                                       CLUSTER_ENUM_RESTYPE   |
                                       CLUSTER_ENUM_RESOURCE  |
                                       CLUSTER_ENUM_GROUP     |
                                       CLUSTER_ENUM_NETWORK   |
                                       CLUSTER_ENUM_NETINTERFACE);

    /** Cluster node enumeration values                                                     */
    public static final long CLUSTER_NODE_ENUM_NETINTERFACES = 0x00000001;
    public static final long CLUSTER_NODE_ENUM_ALL           = (CLUSTER_NODE_ENUM_NETINTERFACES);

    /** Cluster node state constants                                                        */
    public static final long ClusterNodeStateUnknown = -1;
    public static final long ClusterNodeUp = 0;
    public static final long ClusterNodeDown = 1;
    public static final long ClusterNodePaused = 2;
    public static final long ClusterNodeJoining = 3;

    /** Cluster group enumeration constants                                                 */
    public static final long CLUSTER_GROUP_ENUM_CONTAINS     = 0x00000001;
    public static final long CLUSTER_GROUP_ENUM_NODES        = 0x00000002;
    public static final long CLUSTER_GROUP_ENUM_ALL          = 
                                      (CLUSTER_GROUP_ENUM_CONTAINS |
                                       CLUSTER_GROUP_ENUM_NODES);

    /** Cluster group state constants                                                       */
    public static final long ClusterGroupStateUnknown = -1;
    public static final long ClusterGroupOnline = 0;
    public static final long ClusterGroupOffline = 1;
    public static final long ClusterGroupFailed = 2;
    public static final long ClusterGroupPartialOnline = 3;

    /** Autofailback type constants                                                         */
    public static final long ClusterGroupPreventFailback = 0;
    public static final long ClusterGroupAllowFailback = 1;
    public static final long ClusterGroupFailbackTypeCount = 2;

    /** Cluster resource state constants                                                    */
    public static final long ClusterResourceStateUnknown = -1;
    public static final long ClusterResourceInherited = 0;
    public static final long ClusterResourceInitializing = 1;
    public static final long ClusterResourceOnline = 2;
    public static final long ClusterResourceOffline = 3;
    public static final long ClusterResourceFailed = 4;
    public static final long ClusterResourcePending = 128;
    public static final long ClusterResourceOnlinePending = 129;
    public static final long ClusterResourceOfflinePending = 130;

    /** Cluster resource restart action constants                                           */
    public static final long ClusterResourceDontRestart = 0;
    public static final long ClusterResourceRestartNoNotify = 1;
    public static final long ClusterResourceRestartNotify = 2;
    public static final long ClusterResourceRestartActionCount = 3;

    /** Cluster resource create flags                                                       */
    public static final long CLUSTER_RESOURCE_SEPARATE_MONITOR   = 1;
    public static final long CLUSTER_RESOURCE_VALID_FLAGS        = CLUSTER_RESOURCE_SEPARATE_MONITOR;

    /** Cluster property type                                                               */
    public static final long CLUSPROP_TYPE_ENDMARK = 0;
    public static final long CLUSPROP_TYPE_LIST_VALUE = 1;
    public static final long CLUSPROP_TYPE_RESCLASS = 2;
    public static final long CLUSPROP_TYPE_RESERVED1 = 3;
    public static final long CLUSPROP_TYPE_NAME = 4;
    public static final long CLUSPROP_TYPE_SIGNATURE = 5;
    public static final long CLUSPROP_TYPE_SCSI_ADDRESS = 6;
    public static final long CLUSPROP_TYPE_DISK_NUMBER = 7;
    public static final long CLUSPROP_TYPE_PARTITION_INFO = 8;
    public static final long CLUSPROP_TYPE_FTSET_INFO = 9;
    public static final long CLUSPROP_TYPE_USER = 32768;

    /** Cluster property formats                                                            */
    public static final long CLUSPROP_FORMAT_UNKNOWN = 0;
    public static final long CLUSPROP_FORMAT_BINARY = 1;
    public static final long CLUSPROP_FORMAT_DWORD = 2;
    public static final long CLUSPROP_FORMAT_SZ = 3;
    public static final long CLUSPROP_FORMAT_EXPAND_SZ = 4;
    public static final long CLUSPROP_FORMAT_MULTI_SZ = 5;
    public static final long CLUSPROP_FORMAT_ULARGE_INTEGER = 6;
    public static final long CLUSPROP_FORMAT_LONG = 7;
    public static final long CLUSPROP_FORMAT_USER = 32768;

    /** 
     * Compute syntax value
     */
    public static long CLUSPROP_SYNTAX_VALUE(long type, long format ) {
        return ((type << 16) | format);
    }

    /** Cluster property syntax constants                                                   */
    public static final long CLUSPROP_SYNTAX_ENDMARK         = CLUSPROP_SYNTAX_VALUE( CLUSPROP_TYPE_ENDMARK, CLUSPROP_FORMAT_UNKNOWN );
    public static final long CLUSPROP_SYNTAX_NAME            = CLUSPROP_SYNTAX_VALUE( CLUSPROP_TYPE_NAME, CLUSPROP_FORMAT_SZ );
    public static final long CLUSPROP_SYNTAX_RESCLASS        = CLUSPROP_SYNTAX_VALUE( CLUSPROP_TYPE_RESCLASS, CLUSPROP_FORMAT_DWORD );
    public static final long CLUSPROP_SYNTAX_LIST_VALUE_SZ           = CLUSPROP_SYNTAX_VALUE( CLUSPROP_TYPE_LIST_VALUE, CLUSPROP_FORMAT_SZ );
    public static final long CLUSPROP_SYNTAX_LIST_VALUE_EXPAND_SZ    = CLUSPROP_SYNTAX_VALUE( CLUSPROP_TYPE_LIST_VALUE, CLUSPROP_FORMAT_EXPAND_SZ );
    public static final long CLUSPROP_SYNTAX_LIST_VALUE_DWORD        = CLUSPROP_SYNTAX_VALUE( CLUSPROP_TYPE_LIST_VALUE, CLUSPROP_FORMAT_DWORD );
    public static final long CLUSPROP_SYNTAX_LIST_VALUE_BINARY       = CLUSPROP_SYNTAX_VALUE( CLUSPROP_TYPE_LIST_VALUE, CLUSPROP_FORMAT_BINARY );
    public static final long CLUSPROP_SYNTAX_LIST_VALUE_MULTI_SZ     = CLUSPROP_SYNTAX_VALUE( CLUSPROP_TYPE_LIST_VALUE, CLUSPROP_FORMAT_MULTI_SZ );
    public static final long CLUSPROP_SYNTAX_LIST_VALUE_LONG         = CLUSPROP_SYNTAX_VALUE( CLUSPROP_TYPE_LIST_VALUE, CLUSPROP_FORMAT_LONG );
    public static final long CLUSPROP_SYNTAX_DISK_SIGNATURE  = CLUSPROP_SYNTAX_VALUE( CLUSPROP_TYPE_SIGNATURE, CLUSPROP_FORMAT_DWORD );
    public static final long CLUSPROP_SYNTAX_SCSI_ADDRESS    = CLUSPROP_SYNTAX_VALUE( CLUSPROP_TYPE_SCSI_ADDRESS, CLUSPROP_FORMAT_DWORD );
    public static final long CLUSPROP_SYNTAX_DISK_NUMBER     = CLUSPROP_SYNTAX_VALUE( CLUSPROP_TYPE_DISK_NUMBER, CLUSPROP_FORMAT_DWORD );
    public static final long CLUSPROP_SYNTAX_PARTITION_INFO  = CLUSPROP_SYNTAX_VALUE( CLUSPROP_TYPE_PARTITION_INFO, CLUSPROP_FORMAT_BINARY );
    public static final long CLUSPROP_SYNTAX_FTSET_INFO      = CLUSPROP_SYNTAX_VALUE( CLUSPROP_TYPE_FTSET_INFO, CLUSPROP_FORMAT_BINARY );

    /** Cluster control code access methods                                                 */
    public static final long CLUS_ACCESS_ANY = 0;
    public static final long CLUS_ACCESS_READ = 0x01;
    public static final long CLUS_ACCESS_WRITE = 0x02;

    /** Define Cluster Control Code modification actions                                    */
    public static final long CLUS_NO_MODIFY = 0;
    public static final long CLUS_MODIFY = 0x01;

    /** Define Cluster Control Code Global actions                                          */
    public static final long CLUS_NOT_GLOBAL = 0;
    public static final long CLUS_GLOBAL = 0x01;

    /** Define Cluster Control Code target objects                                          */
    public static final long CLUS_OBJECT_INVALID = 0;
    public static final long CLUS_OBJECT_RESOURCE = 1;
    public static final long CLUS_OBJECT_RESOURCE_TYPE = 2;
    public static final long CLUS_OBJECT_GROUP = 3;
    public static final long CLUS_OBJECT_NODE = 4;
    public static final long CLUS_OBJECT_NETWORK = 5;
    public static final long CLUS_OBJECT_NETINTERFACE = 6;
    public static final long CLUS_OBJECT_USER = 128;

    /** Define control code shifts                                                          */
    public static final long CLUSCTL_ACCESS_SHIFT = 0;
    public static final long CLUSCTL_FUNCTION_SHIFT = 2;
    public static final long CLCTL_INTERNAL_SHIFT = 20;
    public static final long CLCTL_USER_SHIFT = 21;
    public static final long CLCTL_MODIFY_SHIFT = 22;
    public static final long CLCTL_GLOBAL_SHIFT = 23;
    public static final long CLUSCTL_OBJECT_SHIFT = 24;

    /** Define control code masks                                                           */
    public static final long CLCTL_INTERNAL_MASK = (1<<CLCTL_INTERNAL_SHIFT);
    public static final long CLCTL_USER_MASK = (1<<CLCTL_USER_SHIFT);
    public static final long CLCTL_MODIFY_MASK = (1<<CLCTL_MODIFY_SHIFT);
    public static final long CLCTL_GLOBAL_MASK = (1<<CLCTL_GLOBAL_SHIFT);
    public static final long CLUSCTL_CONTROL_CODE_MASK = 0x3FFFFF;
    public static final long CLUSCTL_OBJECT_MASK = 0xFF;
    public static final long CLUSCTL_ACCESS_MODE_MASK = 0x03;

    /** Cluster Control function codes (a DWORD)                                            */
    public static final long CLCTL_CLUSTER_BASE = 0;
    public static final long CLCTL_USER_BASE = (1<<CLCTL_USER_SHIFT);

    /** Compute cluster control function external code                                      */
    public static long CLCTL_EXTERNAL_CODE(long Function, long Access, long Modify ) {
        return ((Access) << CLUSCTL_ACCESS_SHIFT) | 
            ((CLCTL_CLUSTER_BASE + Function) << CLUSCTL_FUNCTION_SHIFT) | 
            ((Modify) << CLCTL_MODIFY_SHIFT);
    }
    /** Compute cluster control function internal code                                      */
    public static long CLCTL_INTERNAL_CODE(long Function, long Access, long Modify ) {
        return ((Access) << CLUSCTL_ACCESS_SHIFT) | CLCTL_INTERNAL_MASK | 
            ((CLCTL_CLUSTER_BASE + Function) << CLUSCTL_FUNCTION_SHIFT) | 
            ((Modify) << CLCTL_MODIFY_SHIFT);
    }
    /** External control codes                                                              */
    public static final long CLCTL_UNKNOWN = CLCTL_EXTERNAL_CODE( 0, CLUS_ACCESS_ANY, CLUS_NO_MODIFY );
    public static final long CLCTL_GET_CHARACTERISTICS = CLCTL_EXTERNAL_CODE( 1, CLUS_ACCESS_READ, CLUS_NO_MODIFY );
    public static final long CLCTL_GET_FLAGS = CLCTL_EXTERNAL_CODE( 2, CLUS_ACCESS_READ, CLUS_NO_MODIFY );
    public static final long CLCTL_GET_CLASS_INFO = CLCTL_EXTERNAL_CODE( 3, CLUS_ACCESS_READ, CLUS_NO_MODIFY );
    public static final long CLCTL_GET_REQUIRED_DEPENDENCIES = CLCTL_EXTERNAL_CODE( 4, CLUS_ACCESS_READ, CLUS_NO_MODIFY );
    public static final long CLCTL_GET_NAME = CLCTL_EXTERNAL_CODE( 10, CLUS_ACCESS_READ, CLUS_NO_MODIFY );
    public static final long CLCTL_GET_RESOURCE_TYPE = CLCTL_EXTERNAL_CODE( 11, CLUS_ACCESS_READ, CLUS_NO_MODIFY );
    public static final long CLCTL_GET_NODE = CLCTL_EXTERNAL_CODE( 12, CLUS_ACCESS_READ, CLUS_NO_MODIFY );
    public static final long CLCTL_GET_NETWORK = CLCTL_EXTERNAL_CODE( 13, CLUS_ACCESS_READ, CLUS_NO_MODIFY );
    public static final long CLCTL_GET_ID = CLCTL_EXTERNAL_CODE( 14, CLUS_ACCESS_READ, CLUS_NO_MODIFY );
    public static final long CLCTL_ENUM_COMMON_PROPERTIES = CLCTL_EXTERNAL_CODE( 20, CLUS_ACCESS_READ, CLUS_NO_MODIFY );
    public static final long CLCTL_GET_RO_COMMON_PROPERTIES = CLCTL_EXTERNAL_CODE( 21, CLUS_ACCESS_READ, CLUS_NO_MODIFY );
    public static final long CLCTL_GET_COMMON_PROPERTIES = CLCTL_EXTERNAL_CODE( 22, CLUS_ACCESS_READ, CLUS_NO_MODIFY );
    public static final long CLCTL_SET_COMMON_PROPERTIES = CLCTL_EXTERNAL_CODE( 23, CLUS_ACCESS_WRITE, CLUS_MODIFY );
    public static final long CLCTL_VALIDATE_COMMON_PROPERTIES = CLCTL_EXTERNAL_CODE( 24, CLUS_ACCESS_READ, CLUS_NO_MODIFY );
    public static final long CLCTL_ENUM_PRIVATE_PROPERTIES = CLCTL_EXTERNAL_CODE( 30, CLUS_ACCESS_READ, CLUS_NO_MODIFY );
    public static final long CLCTL_GET_RO_PRIVATE_PROPERTIES = CLCTL_EXTERNAL_CODE( 31, CLUS_ACCESS_READ, CLUS_NO_MODIFY );
    public static final long CLCTL_GET_PRIVATE_PROPERTIES = CLCTL_EXTERNAL_CODE( 32, CLUS_ACCESS_READ, CLUS_NO_MODIFY );
    public static final long CLCTL_SET_PRIVATE_PROPERTIES = CLCTL_EXTERNAL_CODE( 33, CLUS_ACCESS_WRITE, CLUS_MODIFY );
    public static final long CLCTL_VALIDATE_PRIVATE_PROPERTIES = CLCTL_EXTERNAL_CODE( 34, CLUS_ACCESS_READ, CLUS_NO_MODIFY );
    public static final long CLCTL_ADD_REGISTRY_CHECKPOINT = CLCTL_EXTERNAL_CODE( 40, CLUS_ACCESS_WRITE, CLUS_MODIFY );
    public static final long CLCTL_DELETE_REGISTRY_CHECKPOINT = CLCTL_EXTERNAL_CODE( 41, CLUS_ACCESS_WRITE, CLUS_MODIFY );
    public static final long CLCTL_GET_REGISTRY_CHECKPOINTS = CLCTL_EXTERNAL_CODE( 42, CLUS_ACCESS_READ, CLUS_NO_MODIFY );
    public static final long CLCTL_GET_LOADBAL_PROCESS_LIST = CLCTL_EXTERNAL_CODE( 50, CLUS_ACCESS_READ, CLUS_NO_MODIFY );
    public static final long CLCTL_STORAGE_GET_DISK_INFO = CLCTL_EXTERNAL_CODE( 100, CLUS_ACCESS_READ, CLUS_NO_MODIFY );
    public static final long CLCTL_STORAGE_GET_AVAILABLE_DISKS = CLCTL_EXTERNAL_CODE( 101, CLUS_ACCESS_READ, CLUS_NO_MODIFY );
    public static final long CLCTL_STORAGE_IS_PATH_VALID = CLCTL_EXTERNAL_CODE( 102, CLUS_ACCESS_READ, CLUS_NO_MODIFY );
    public static final long CLCTL_STORAGE_GET_ALL_AVAILABLE_DISKS = (CLCTL_EXTERNAL_CODE( 103, CLUS_ACCESS_READ, CLUS_NO_MODIFY ) | CLCTL_GLOBAL_MASK);

    /** Internal control codes                                                              */
    public static final long CLCTL_DELETE = CLCTL_INTERNAL_CODE( 1, CLUS_ACCESS_WRITE, CLUS_MODIFY );
    public static final long CLCTL_INSTALL_NODE = CLCTL_INTERNAL_CODE( 2, CLUS_ACCESS_WRITE, CLUS_MODIFY );
    public static final long CLCTL_EVICT_NODE = CLCTL_INTERNAL_CODE( 3, CLUS_ACCESS_WRITE, CLUS_MODIFY );
    public static final long CLCTL_ADD_DEPENDENCY = CLCTL_INTERNAL_CODE( 4, CLUS_ACCESS_WRITE, CLUS_MODIFY );
    public static final long CLCTL_REMOVE_DEPENDENCY = CLCTL_INTERNAL_CODE( 5, CLUS_ACCESS_WRITE, CLUS_MODIFY );
    public static final long CLCTL_ADD_OWNER = CLCTL_INTERNAL_CODE( 6, CLUS_ACCESS_WRITE, CLUS_MODIFY );
    public static final long CLCTL_REMOVE_OWNER = CLCTL_INTERNAL_CODE( 7, CLUS_ACCESS_WRITE, CLUS_MODIFY );
    public static final long CLCTL_SET_NAME = CLCTL_INTERNAL_CODE( 9, CLUS_ACCESS_WRITE, CLUS_MODIFY );
    public static final long CLCTL_CLUSTER_NAME_CHANGED = CLCTL_INTERNAL_CODE( 10, CLUS_ACCESS_WRITE, CLUS_MODIFY );
    public static final long CLCTL_CLUSTER_VERSION_CHANGED = CLCTL_INTERNAL_CODE( 11, CLUS_ACCESS_WRITE, CLUS_MODIFY);

    /** Define macros to generate object specific control codes                             */
    public static final long CLUSCTL_RESOURCE_CODE(long Function ) {
        return ((CLUS_OBJECT_RESOURCE << CLUSCTL_OBJECT_SHIFT) | Function);
    }
    public static final long CLUSCTL_RESOURCE_TYPE_CODE(long Function ) {
        return ((CLUS_OBJECT_RESOURCE_TYPE << CLUSCTL_OBJECT_SHIFT) | Function);
    }
    public static final long CLUSCTL_GROUP_CODE(long Function ) {
        return ((CLUS_OBJECT_GROUP << CLUSCTL_OBJECT_SHIFT) | Function);
    }
    public static final long CLUSCTL_NODE_CODE(long Function ) {
        return ((CLUS_OBJECT_NODE << CLUSCTL_OBJECT_SHIFT) | Function);
    }
    public static final long CLUSCTL_NETWORK_CODE(long Function ) {
        return ((CLUS_OBJECT_NETWORK << CLUSCTL_OBJECT_SHIFT) | Function);
    }
    public static final long CLUSCTL_NETINTERFACE_CODE(long Function ) {
        return ((CLUS_OBJECT_NETINTERFACE << CLUSCTL_OBJECT_SHIFT) | Function);
    }
    public static final long CLUSCTL_USER_CODE(long Function, long Object ) {
        return ((Object) << CLUSCTL_OBJECT_SHIFT) | (CLCTL_USER_BASE + Function);
    }
    /** Define macros to get the function or access mode out of a control code              */
    public static final long CLUSCTL_GET_CONTROL_FUNCTION(long ControlCode ) {
        return ((ControlCode >> CLUSCTL_ACCESS_SHIFT) & CLUSCTL_CONTROL_CODE_MASK);
    }
    public static final long CLUSCTL_GET_ACCESS_MODE(long ControlCode ) {
        return ((ControlCode >> CLUSCTL_ACCESS_SHIFT) & CLUSCTL_ACCESS_MODE_MASK);
    }
    public static final long CLUSCTL_GET_CONTROL_OBJECT(long ControlCode ) {
        return ((ControlCode >> CLUSCTL_OBJECT_SHIFT) & CLUSCTL_OBJECT_MASK);
    }
    /** Cluster Control Codes for Resources                                                 */
    public static final long CLUSCTL_RESOURCE_UNKNOWN =
        CLUSCTL_RESOURCE_CODE( CLCTL_UNKNOWN );
    public static final long CLUSCTL_RESOURCE_GET_CHARACTERISTICS =
        CLUSCTL_RESOURCE_CODE( CLCTL_GET_CHARACTERISTICS );
    public static final long CLUSCTL_RESOURCE_GET_FLAGS =
        CLUSCTL_RESOURCE_CODE( CLCTL_GET_FLAGS );
    public static final long CLUSCTL_RESOURCE_GET_CLASS_INFO =
        CLUSCTL_RESOURCE_CODE( CLCTL_GET_CLASS_INFO );
    public static final long CLUSCTL_RESOURCE_GET_REQUIRED_DEPENDENCIES =
        CLUSCTL_RESOURCE_CODE( CLCTL_GET_REQUIRED_DEPENDENCIES );
    public static final long CLUSCTL_RESOURCE_GET_NAME =
        CLUSCTL_RESOURCE_CODE( CLCTL_GET_NAME );
    public static final long CLUSCTL_RESOURCE_GET_ID =
        CLUSCTL_RESOURCE_CODE( CLCTL_GET_ID );
    public static final long CLUSCTL_RESOURCE_GET_RESOURCE_TYPE =
        CLUSCTL_RESOURCE_CODE( CLCTL_GET_RESOURCE_TYPE );
    public static final long CLUSCTL_RESOURCE_ENUM_COMMON_PROPERTIES =
        CLUSCTL_RESOURCE_CODE( CLCTL_ENUM_COMMON_PROPERTIES );
    public static final long CLUSCTL_RESOURCE_GET_RO_COMMON_PROPERTIES =
        CLUSCTL_RESOURCE_CODE( CLCTL_GET_RO_COMMON_PROPERTIES );
    public static final long CLUSCTL_RESOURCE_GET_COMMON_PROPERTIES =
        CLUSCTL_RESOURCE_CODE( CLCTL_GET_COMMON_PROPERTIES );
    public static final long CLUSCTL_RESOURCE_SET_COMMON_PROPERTIES =
        CLUSCTL_RESOURCE_CODE( CLCTL_SET_COMMON_PROPERTIES );
    public static final long CLUSCTL_RESOURCE_VALIDATE_COMMON_PROPERTIES =
        CLUSCTL_RESOURCE_CODE( CLCTL_VALIDATE_COMMON_PROPERTIES );
    public static final long CLUSCTL_RESOURCE_ENUM_PRIVATE_PROPERTIES =
        CLUSCTL_RESOURCE_CODE( CLCTL_ENUM_PRIVATE_PROPERTIES );
    public static final long CLUSCTL_RESOURCE_GET_RO_PRIVATE_PROPERTIES =
        CLUSCTL_RESOURCE_CODE( CLCTL_GET_RO_PRIVATE_PROPERTIES );
    public static final long CLUSCTL_RESOURCE_GET_PRIVATE_PROPERTIES =
        CLUSCTL_RESOURCE_CODE( CLCTL_GET_PRIVATE_PROPERTIES );
    public static final long CLUSCTL_RESOURCE_SET_PRIVATE_PROPERTIES =
        CLUSCTL_RESOURCE_CODE( CLCTL_SET_PRIVATE_PROPERTIES );
    public static final long CLUSCTL_RESOURCE_VALIDATE_PRIVATE_PROPERTIES =
        CLUSCTL_RESOURCE_CODE( CLCTL_VALIDATE_PRIVATE_PROPERTIES );
    public static final long CLUSCTL_RESOURCE_ADD_REGISTRY_CHECKPOINT =
        CLUSCTL_RESOURCE_CODE( CLCTL_ADD_REGISTRY_CHECKPOINT );
    public static final long CLUSCTL_RESOURCE_DELETE_REGISTRY_CHECKPOINT =
        CLUSCTL_RESOURCE_CODE( CLCTL_DELETE_REGISTRY_CHECKPOINT );
    public static final long CLUSCTL_RESOURCE_GET_REGISTRY_CHECKPOINTS =
        CLUSCTL_RESOURCE_CODE( CLCTL_GET_REGISTRY_CHECKPOINTS );
    public static final long CLUSCTL_RESOURCE_GET_LOADBAL_PROCESS_LIST =
        CLUSCTL_RESOURCE_CODE( CLCTL_GET_LOADBAL_PROCESS_LIST );
    public static final long CLUSCTL_RESOURCE_STORAGE_GET_DISK_INFO =
        CLUSCTL_RESOURCE_CODE( CLCTL_STORAGE_GET_DISK_INFO );
    public static final long CLUSCTL_RESOURCE_STORAGE_IS_PATH_VALID =
        CLUSCTL_RESOURCE_CODE( CLCTL_STORAGE_IS_PATH_VALID );
    public static final long CLUSCTL_RESOURCE_DELETE =
        CLUSCTL_RESOURCE_CODE( CLCTL_DELETE );
    public static final long CLUSCTL_RESOURCE_INSTALL_NODE =
        CLUSCTL_RESOURCE_CODE( CLCTL_INSTALL_NODE );
    public static final long CLUSCTL_RESOURCE_EVICT_NODE =
        CLUSCTL_RESOURCE_CODE( CLCTL_EVICT_NODE );
    public static final long CLUSCTL_RESOURCE_ADD_DEPENDENCY =
        CLUSCTL_RESOURCE_CODE( CLCTL_ADD_DEPENDENCY );
    public static final long CLUSCTL_RESOURCE_REMOVE_DEPENDENCY =
        CLUSCTL_RESOURCE_CODE( CLCTL_REMOVE_DEPENDENCY );
    public static final long CLUSCTL_RESOURCE_ADD_OWNER =
        CLUSCTL_RESOURCE_CODE( CLCTL_ADD_OWNER );
    public static final long CLUSCTL_RESOURCE_REMOVE_OWNER =
        CLUSCTL_RESOURCE_CODE( CLCTL_REMOVE_OWNER );
    public static final long CLUSCTL_RESOURCE_SET_NAME =
        CLUSCTL_RESOURCE_CODE( CLCTL_SET_NAME );
    public static final long CLUSCTL_RESOURCE_CLUSTER_NAME_CHANGED =
        CLUSCTL_RESOURCE_CODE( CLCTL_CLUSTER_NAME_CHANGED );
    public static final long CLUSCTL_RESOURCE_CLUSTER_VERSION_CHANGED =
        CLUSCTL_RESOURCE_CODE( CLCTL_CLUSTER_VERSION_CHANGED );

    /** Cluster Control Codes for Resource Types                                            */
    public static final long CLUSCTL_RESOURCE_TYPE_UNKNOWN =
        CLUSCTL_RESOURCE_TYPE_CODE( CLCTL_UNKNOWN );
    public static final long CLUSCTL_RESOURCE_TYPE_GET_CHARACTERISTICS  =
        CLUSCTL_RESOURCE_TYPE_CODE( CLCTL_GET_CHARACTERISTICS );
    public static final long CLUSCTL_RESOURCE_TYPE_GET_FLAGS =
        CLUSCTL_RESOURCE_TYPE_CODE( CLCTL_GET_FLAGS );
    public static final long CLUSCTL_RESOURCE_TYPE_GET_CLASS_INFO =
        CLUSCTL_RESOURCE_TYPE_CODE( CLCTL_GET_CLASS_INFO );
    public static final long CLUSCTL_RESOURCE_TYPE_GET_REQUIRED_DEPENDENCIES =
        CLUSCTL_RESOURCE_TYPE_CODE( CLCTL_GET_REQUIRED_DEPENDENCIES );
    public static final long CLUSCTL_RESOURCE_TYPE_ENUM_COMMON_PROPERTIES =
        CLUSCTL_RESOURCE_TYPE_CODE( CLCTL_ENUM_COMMON_PROPERTIES );
    public static final long CLUSCTL_RESOURCE_TYPE_GET_RO_COMMON_PROPERTIES =
        CLUSCTL_RESOURCE_TYPE_CODE( CLCTL_GET_RO_COMMON_PROPERTIES );
    public static final long CLUSCTL_RESOURCE_TYPE_GET_COMMON_PROPERTIES =
        CLUSCTL_RESOURCE_TYPE_CODE( CLCTL_GET_COMMON_PROPERTIES );
    public static final long CLUSCTL_RESOURCE_TYPE_VALIDATE_COMMON_PROPERTIES =
        CLUSCTL_RESOURCE_TYPE_CODE( CLCTL_VALIDATE_COMMON_PROPERTIES );
    public static final long CLUSCTL_RESOURCE_TYPE_SET_COMMON_PROPERTIES =
        CLUSCTL_RESOURCE_TYPE_CODE( CLCTL_SET_COMMON_PROPERTIES );
    public static final long CLUSCTL_RESOURCE_TYPE_ENUM_PRIVATE_PROPERTIES =
        CLUSCTL_RESOURCE_TYPE_CODE( CLCTL_ENUM_PRIVATE_PROPERTIES );
    public static final long CLUSCTL_RESOURCE_TYPE_GET_RO_PRIVATE_PROPERTIES =
        CLUSCTL_RESOURCE_TYPE_CODE( CLCTL_GET_RO_PRIVATE_PROPERTIES );
    public static final long CLUSCTL_RESOURCE_TYPE_GET_PRIVATE_PROPERTIES =
        CLUSCTL_RESOURCE_TYPE_CODE( CLCTL_GET_PRIVATE_PROPERTIES );
    public static final long CLUSCTL_RESOURCE_TYPE_SET_PRIVATE_PROPERTIES =
        CLUSCTL_RESOURCE_TYPE_CODE( CLCTL_SET_PRIVATE_PROPERTIES );
    public static final long CLUSCTL_RESOURCE_TYPE_VALIDATE_PRIVATE_PROPERTIES =
        CLUSCTL_RESOURCE_TYPE_CODE( CLCTL_VALIDATE_PRIVATE_PROPERTIES );
    public static final long CLUSCTL_RESOURCE_TYPE_GET_REGISTRY_CHECKPOINTS =
        CLUSCTL_RESOURCE_TYPE_CODE( CLCTL_GET_REGISTRY_CHECKPOINTS );
    public static final long CLUSCTL_RESOURCE_TYPE_STORAGE_GET_AVAILABLE_DISKS =
        CLUSCTL_RESOURCE_TYPE_CODE( CLCTL_STORAGE_GET_AVAILABLE_DISKS );
    public static final long CLUSCTL_RESOURCE_TYPE_INSTALL_NODE =
        CLUSCTL_RESOURCE_TYPE_CODE( CLCTL_INSTALL_NODE );
    public static final long CLUSCTL_RESOURCE_TYPE_EVICT_NODE =
        CLUSCTL_RESOURCE_TYPE_CODE( CLCTL_EVICT_NODE );

    /** Cluster Control Codes for Groups                                                    */
    public static final long CLUSCTL_GROUP_UNKNOWN =
        CLUSCTL_GROUP_CODE( CLCTL_UNKNOWN );
    public static final long CLUSCTL_GROUP_GET_CHARACTERISTICS =
        CLUSCTL_GROUP_CODE( CLCTL_GET_CHARACTERISTICS );
    public static final long CLUSCTL_GROUP_GET_FLAGS =
        CLUSCTL_GROUP_CODE( CLCTL_GET_FLAGS );
    public static final long CLUSCTL_GROUP_GET_NAME =
        CLUSCTL_GROUP_CODE( CLCTL_GET_NAME );
    public static final long CLUSCTL_GROUP_GET_ID =
        CLUSCTL_GROUP_CODE( CLCTL_GET_ID );
    public static final long CLUSCTL_GROUP_ENUM_COMMON_PROPERTIES =
        CLUSCTL_GROUP_CODE( CLCTL_ENUM_COMMON_PROPERTIES );
    public static final long CLUSCTL_GROUP_GET_RO_COMMON_PROPERTIES =
        CLUSCTL_GROUP_CODE( CLCTL_GET_RO_COMMON_PROPERTIES );
    public static final long CLUSCTL_GROUP_GET_COMMON_PROPERTIES =
        CLUSCTL_GROUP_CODE( CLCTL_GET_COMMON_PROPERTIES );
    public static final long CLUSCTL_GROUP_SET_COMMON_PROPERTIES =
        CLUSCTL_GROUP_CODE( CLCTL_SET_COMMON_PROPERTIES );
    public static final long CLUSCTL_GROUP_VALIDATE_COMMON_PROPERTIES =
        CLUSCTL_GROUP_CODE( CLCTL_VALIDATE_COMMON_PROPERTIES );
    public static final long CLUSCTL_GROUP_ENUM_PRIVATE_PROPERTIES =
        CLUSCTL_GROUP_CODE( CLCTL_ENUM_PRIVATE_PROPERTIES );
    public static final long CLUSCTL_GROUP_GET_RO_PRIVATE_PROPERTIES =
        CLUSCTL_GROUP_CODE( CLCTL_GET_RO_PRIVATE_PROPERTIES );
    public static final long CLUSCTL_GROUP_GET_PRIVATE_PROPERTIES =
        CLUSCTL_GROUP_CODE( CLCTL_GET_PRIVATE_PROPERTIES );
    public static final long CLUSCTL_GROUP_SET_PRIVATE_PROPERTIES =
        CLUSCTL_GROUP_CODE( CLCTL_SET_PRIVATE_PROPERTIES );
    public static final long CLUSCTL_GROUP_VALIDATE_PRIVATE_PROPERTIES =
        CLUSCTL_GROUP_CODE( CLCTL_VALIDATE_PRIVATE_PROPERTIES );

    /** Cluster Control Codes for Nodes                                                     */
    public static final long CLUSCTL_NODE_UNKNOWN =
        CLUSCTL_NODE_CODE( CLCTL_UNKNOWN );
    public static final long CLUSCTL_NODE_GET_CHARACTERISTICS =
        CLUSCTL_NODE_CODE( CLCTL_GET_CHARACTERISTICS );
    public static final long CLUSCTL_NODE_GET_FLAGS =
        CLUSCTL_NODE_CODE( CLCTL_GET_FLAGS );
    public static final long CLUSCTL_NODE_GET_NAME =
        CLUSCTL_NODE_CODE( CLCTL_GET_NAME );
    public static final long CLUSCTL_NODE_GET_ID =
        CLUSCTL_NODE_CODE( CLCTL_GET_ID );
    public static final long CLUSCTL_NODE_ENUM_COMMON_PROPERTIES =
        CLUSCTL_NODE_CODE( CLCTL_ENUM_COMMON_PROPERTIES );
    public static final long CLUSCTL_NODE_GET_RO_COMMON_PROPERTIES =
        CLUSCTL_NODE_CODE( CLCTL_GET_RO_COMMON_PROPERTIES );
    public static final long CLUSCTL_NODE_GET_COMMON_PROPERTIES =
        CLUSCTL_NODE_CODE( CLCTL_GET_COMMON_PROPERTIES );
    public static final long CLUSCTL_NODE_SET_COMMON_PROPERTIES =
        CLUSCTL_NODE_CODE( CLCTL_SET_COMMON_PROPERTIES );
    public static final long CLUSCTL_NODE_VALIDATE_COMMON_PROPERTIES =
        CLUSCTL_NODE_CODE( CLCTL_VALIDATE_COMMON_PROPERTIES );
    public static final long CLUSCTL_NODE_ENUM_PRIVATE_PROPERTIES =
        CLUSCTL_NODE_CODE( CLCTL_ENUM_PRIVATE_PROPERTIES );
    public static final long CLUSCTL_NODE_GET_RO_PRIVATE_PROPERTIES =
        CLUSCTL_NODE_CODE( CLCTL_GET_RO_PRIVATE_PROPERTIES );
    public static final long CLUSCTL_NODE_GET_PRIVATE_PROPERTIES =
        CLUSCTL_NODE_CODE( CLCTL_GET_PRIVATE_PROPERTIES );
    public static final long CLUSCTL_NODE_SET_PRIVATE_PROPERTIES =
        CLUSCTL_NODE_CODE( CLCTL_SET_PRIVATE_PROPERTIES );
    public static final long CLUSCTL_NODE_VALIDATE_PRIVATE_PROPERTIES =
        CLUSCTL_NODE_CODE( CLCTL_VALIDATE_PRIVATE_PROPERTIES );

    /** Cluster Control Codes for Networks                                                  */
    public static final long CLUSCTL_NETWORK_UNKNOWN =
        CLUSCTL_NETWORK_CODE( CLCTL_UNKNOWN );
    public static final long CLUSCTL_NETWORK_GET_CHARACTERISTICS =
        CLUSCTL_NETWORK_CODE( CLCTL_GET_CHARACTERISTICS );
    public static final long CLUSCTL_NETWORK_GET_FLAGS =
        CLUSCTL_NETWORK_CODE( CLCTL_GET_FLAGS );
    public static final long CLUSCTL_NETWORK_GET_NAME =
        CLUSCTL_NETWORK_CODE( CLCTL_GET_NAME );
    public static final long CLUSCTL_NETWORK_GET_ID =
        CLUSCTL_NETWORK_CODE( CLCTL_GET_ID );
    public static final long CLUSCTL_NETWORK_ENUM_COMMON_PROPERTIES =
        CLUSCTL_NETWORK_CODE( CLCTL_ENUM_COMMON_PROPERTIES );
    public static final long CLUSCTL_NETWORK_GET_RO_COMMON_PROPERTIES =
        CLUSCTL_NETWORK_CODE( CLCTL_GET_RO_COMMON_PROPERTIES );
    public static final long CLUSCTL_NETWORK_GET_COMMON_PROPERTIES =
        CLUSCTL_NETWORK_CODE( CLCTL_GET_COMMON_PROPERTIES );
    public static final long CLUSCTL_NETWORK_SET_COMMON_PROPERTIES =
        CLUSCTL_NETWORK_CODE( CLCTL_SET_COMMON_PROPERTIES );
    public static final long CLUSCTL_NETWORK_VALIDATE_COMMON_PROPERTIES =
        CLUSCTL_NETWORK_CODE( CLCTL_VALIDATE_COMMON_PROPERTIES );
    public static final long CLUSCTL_NETWORK_ENUM_PRIVATE_PROPERTIES =
        CLUSCTL_NETWORK_CODE( CLCTL_ENUM_PRIVATE_PROPERTIES );
    public static final long CLUSCTL_NETWORK_GET_RO_PRIVATE_PROPERTIES =
        CLUSCTL_NETWORK_CODE( CLCTL_GET_RO_PRIVATE_PROPERTIES );
    public static final long CLUSCTL_NETWORK_GET_PRIVATE_PROPERTIES =
        CLUSCTL_NETWORK_CODE( CLCTL_GET_PRIVATE_PROPERTIES );
    public static final long CLUSCTL_NETWORK_SET_PRIVATE_PROPERTIES =
        CLUSCTL_NETWORK_CODE( CLCTL_SET_PRIVATE_PROPERTIES );
    public static final long CLUSCTL_NETWORK_VALIDATE_PRIVATE_PROPERTIES =
        CLUSCTL_NETWORK_CODE( CLCTL_VALIDATE_PRIVATE_PROPERTIES );

    /** Cluster Control Codes for Network Interfaces                                        */
    public static final long CLUSCTL_NETINTERFACE_UNKNOWN =
        CLUSCTL_NETINTERFACE_CODE( CLCTL_UNKNOWN );
    public static final long CLUSCTL_NETINTERFACE_GET_CHARACTERISTICS =
        CLUSCTL_NETINTERFACE_CODE( CLCTL_GET_CHARACTERISTICS );
    public static final long CLUSCTL_NETINTERFACE_GET_FLAGS =
        CLUSCTL_NETINTERFACE_CODE( CLCTL_GET_FLAGS );
    public static final long CLUSCTL_NETINTERFACE_GET_NAME =
        CLUSCTL_NETINTERFACE_CODE( CLCTL_GET_NAME );
    public static final long CLUSCTL_NETINTERFACE_GET_ID =
        CLUSCTL_NETINTERFACE_CODE( CLCTL_GET_ID );
    public static final long CLUSCTL_NETINTERFACE_GET_NODE =
        CLUSCTL_NETINTERFACE_CODE( CLCTL_GET_NODE );
    public static final long CLUSCTL_NETINTERFACE_GET_NETWORK =
        CLUSCTL_NETINTERFACE_CODE( CLCTL_GET_NETWORK );
    public static final long CLUSCTL_NETINTERFACE_ENUM_COMMON_PROPERTIES =
        CLUSCTL_NETINTERFACE_CODE( CLCTL_ENUM_COMMON_PROPERTIES );
    public static final long CLUSCTL_NETINTERFACE_GET_RO_COMMON_PROPERTIES =
        CLUSCTL_NETINTERFACE_CODE( CLCTL_GET_RO_COMMON_PROPERTIES );
    public static final long CLUSCTL_NETINTERFACE_GET_COMMON_PROPERTIES =
        CLUSCTL_NETINTERFACE_CODE( CLCTL_GET_COMMON_PROPERTIES );
    public static final long CLUSCTL_NETINTERFACE_SET_COMMON_PROPERTIES =
        CLUSCTL_NETINTERFACE_CODE( CLCTL_SET_COMMON_PROPERTIES );
    public static final long CLUSCTL_NETINTERFACE_VALIDATE_COMMON_PROPERTIES =
        CLUSCTL_NETINTERFACE_CODE( CLCTL_VALIDATE_COMMON_PROPERTIES );
    public static final long CLUSCTL_NETINTERFACE_ENUM_PRIVATE_PROPERTIES =
        CLUSCTL_NETINTERFACE_CODE( CLCTL_ENUM_PRIVATE_PROPERTIES );
    public static final long CLUSCTL_NETINTERFACE_GET_RO_PRIVATE_PROPERTIES =
        CLUSCTL_NETINTERFACE_CODE( CLCTL_GET_RO_PRIVATE_PROPERTIES );
    public static final long CLUSCTL_NETINTERFACE_GET_PRIVATE_PROPERTIES =
        CLUSCTL_NETINTERFACE_CODE( CLCTL_GET_PRIVATE_PROPERTIES );
    public static final long CLUSCTL_NETINTERFACE_SET_PRIVATE_PROPERTIES =
        CLUSCTL_NETINTERFACE_CODE( CLCTL_SET_PRIVATE_PROPERTIES );
    public static final long CLUSCTL_NETINTERFACE_VALIDATE_PRIVATE_PROPERTIES =
        CLUSCTL_NETINTERFACE_CODE( CLCTL_VALIDATE_PRIVATE_PROPERTIES );

    /** Cluster Resource Class types                                                        */
    public static final long CLUS_RESCLASS_UNKNOWN = 0;
    public static final long CLUS_RESCLASS_STORAGE = 1;
    public static final long CLUS_RESCLASS_USER = 32768;

    /** Define Resource SubClass bits                                                       */
    public static final long CLUS_RESSUBCLASS_SHARED = 0x80000000;

    /** Cluster Characteristics                                                             */
    public static final long CLUS_CHAR_UNKNOWN                   = 0x00000000;
    public static final long CLUS_CHAR_QUORUM                    = 0x00000001;
    public static final long CLUS_CHAR_DELETE_REQUIRES_ALL_NODES = 0x00000002;

    /** Cluster Flags                                                                       */
    public static final long CLUS_FLAG_CORE          = 0x00000001;

    /** Cluster resource enumeration types                                                  */
    public static final long CLUSTER_RESOURCE_ENUM_DEPENDS   = 0x00000001;
    public static final long CLUSTER_RESOURCE_ENUM_PROVIDES  = 0x00000002;
    public static final long CLUSTER_RESOURCE_ENUM_NODES     = 0x00000004;
    public static final long CLUSTER_RESOURCE_ENUM_ALL       = 
                                        (CLUSTER_RESOURCE_ENUM_DEPENDS  |
                                         CLUSTER_RESOURCE_ENUM_PROVIDES |
                                         CLUSTER_RESOURCE_ENUM_NODES);

    /** Define enumerable group types                                                       */
    public static final long CLUSTER_NETWORK_ENUM_NETINTERFACES  = 0x00000001;
    public static final long CLUSTER_NETWORK_ENUM_ALL            = CLUSTER_NETWORK_ENUM_NETINTERFACES;

    /** Network state constants                                                             */
    public static final long ClusterNetworkStateUnknown = -1;
    public static final long ClusterNetworkUnavailable = 0;
    public static final long ClusterNetworkDown = 1;
    public static final long ClusterNetworkPartitioned = 2;
    public static final long ClusterNetworkUp = 3;

    /** Network roles constants                                                             */
    public static final long ClusterNetworkRoleNone              = 0;
    public static final long ClusterNetworkRoleInternalUse       = 0x00000001;
    public static final long ClusterNetworkRoleClientAccess      = 0x00000002;
    public static final long ClusterNetworkRoleInternalAndClient = 0x00000003;

    /** Network interface state constants                                                   */
    public static final long ClusterNetInterfaceStateUnknown = -1;
    public static final long ClusterNetInterfaceUnavailable = 0;
    public static final long ClusterNetInterfaceFailed = 1;
    public static final long ClusterNetInterfaceUnreachable = 2;
    public static final long ClusterNetInterfaceUp = 3;

    /**
     * Base constructor for MSCSAgentCommand
     * @param cmd_code - command code
     * @param ipc_path - IPC path to target
     */
    public MSCSAgentCommand(long cmd_code, String ipc_path) {
        super(cmd_code);
        if(ipc_path == null)
            ipc_path = CLUSTAGT_SVCNODE_NAME;
        if(!ipc_path.endsWith(CLUSTAGT_SVCNODE_NAME)) {
            ipc_path += "::" + CLUSTAGT_SVCNODE_NAME;
        }
        SetDestinationAddress(ipc_path);
    }
    /**
     * Add DWORD input parameter
     * @param val - value to be set
     */
    protected void addDWORDInputParam(long val) {
        byte[] v = new byte[IntelByteBuffer.SIZEOF_ULONG];
        IntelByteBuffer.WriteULONG(v, val, 0);
        AttachInputParm(v);
    }
    /**
     * Read DWORD output parameter
     * @param index - parameter index to read
     * @param off - offset of DWORD field
     * @returns value returned, or -1 if not available
     */
    protected long readDWORDOutputParam(int index, int off) {
        if(OutputParmLength(index) < (off + IntelByteBuffer.SIZEOF_ULONG))
            return -1;
        else 
            return IntelByteBuffer.ReadULONG(OutputParm(index), off);
    }
    /**
     * Read WORD output parameter
     * @param index - parameter index to read
     * @param off - offset of WORD field
     * @returns value returned, or -1 if not available
     */
    protected int readWORDOutputParam(int index, int off) {
        if(OutputParmLength(index) < (off + IntelByteBuffer.SIZEOF_USHORT))
            return -1;
        else 
            return IntelByteBuffer.ReadUSHORT(OutputParm(index), off);
    }
    /**
     * Read DWORD output parameter
     * @param index - parameter index to read
     * @returns value returned, or -1 if not available
     */
    protected long readDWORDOutputParam(int index) {
        return readDWORDOutputParam(index, 0);
    }
    /**
     * Read CompUnicode from output parameter
     * @param index - parameter index to read
     * @param off - offset within parameter to read
     * @returns string read, or null if not available
     */
    protected String readCompUnicodeOutputParam(int index, int off) {
        if(OutputParmLength(index) < (off + 1)) {
            return null;
        } else {
            return IntelByteBuffer.ReadCompUnicode(OutputParm(index), off);
        }
    }
    /**
     * Read CompUnicode from output parameter
     * @param index - parameter index to read
     * @returns string read, or null if not available
     */
    protected String readCompUnicodeOutputParam(int index) {
        return readCompUnicodeOutputParam(index, 0);
    }
    /**
     * Add CompUnicode input parameter
     * @param val - string value to write
     */
    protected void addCompUnicodeInputParam(String val) {
        byte[] v = new byte[IntelByteBuffer.GetCompUnicodeLength(val)];
        IntelByteBuffer.WriteCompUnicode(v, val, 0);
        AttachInputParm(v);
    }
    /**
     * Read CompUnicode list from output parameter
     * @param index - parameter index to read
     * @param len - number of strings to read
     * @param off - offset within parameter to read
     * @returns strings read, or null if not available
     */
    protected String[] readCompUnicodeListOutputParam(int index, int len, int off) {
        if(OutputParmLength(index) < (off + len)) {
            return null;
        } else {
            String[] v = new String[len];
            byte[] buf = OutputParm(index);
            for(int i = 0; i < len; i++) {
                v[i] = IntelByteBuffer.ReadCompUnicode(buf, off);
                off += IntelByteBuffer.GetCompUnicodeLength(v[i]);
            }
            return v;
        }
    }
    /**
     * Read CompUnicode list from output parameter
     * @param index - parameter index to read
     * @param len - number of strings to read
     * @returns strings read, or null if not available
     */
    protected String[] readCompUnicodeListOutputParam(int index, int len) {
        return readCompUnicodeListOutputParam(index, len, 0);
    }
    /**
     * Read unicode string from output parameter
     * @param index - parameter index to read
     * @param off - offset of string to read
     * @returns string read, or null if not available
     */
    protected String readUnicodeOutputParam(int index, int off) {
        int cnt;
        byte[] v = OutputParm(index);
        if(OutputParmLength(index) < (off + 1)) {
            return null;
        }
        for(cnt = 0; IntelByteBuffer.ReadUnicode(v, off+2*cnt) != '\0'; cnt++) {}
        char[] buf = new char[cnt];
        for(int i = 0; i < cnt; i++) {
            buf[i] = IntelByteBuffer.ReadUnicode(v, off+2*i);
        }
        return new String(buf);
    }
}

