package com.tivoli.twg.mscs;

import java.awt.*;
import java.awt.event.*;
import com.tivoli.twg.inventory.*;
import com.tivoli.twg.libs.*;
import com.tivoli.twg.engine.*;
import com.tivoli.twg.log.*;
import com.tivoli.twg.inventory.*;
import com.tivoli.twg.engine.cluster.TWGClusterMemberTable;
import java.util.Hashtable;
import com.tivoli.twg.netipc.TWGSvcMgrServiceNodeConstants;

/**
 * MSCS Inventory Collector
 * @author Mike Primm
 * @version 1.1 - 6/8/05
 */
public class MSCSInvCollector extends TWGInvCollector {
   private ServiceNode sn = null;

   private static final String[] databaseTableClassNames = {
      "com.tivoli.twg.mscs.MSCSResourceTable",
      "com.tivoli.twg.mscs.MSCSGroupTable",
      "com.tivoli.twg.mscs.MSCSGroupResTable",
      "com.tivoli.twg.mscs.MSCSNetIntTable",
      "com.tivoli.twg.mscs.MSCSClusterInfoTable"
   };
   /**
    * Return list of database tables
    * @returns database table names
    */
   public String[] getDatabaseTableClassNames() {
       return databaseTableClassNames;
   }
   /** 
    * Get name for collectory thread
    * @returns thread name
    */
   public String getCollectorThreadName() {
      return "MSCSManagedObject";
   }
   /**
    * Start inventory collection on given object
    * @param moid - managed object ID of entity to inventory
    * @param className - class name of object to inventory
    * @param addrInfo - raw address entry information for object to inventory
    */
   public void startInventory(long moid, String className, byte[] addrInfo) {
       TWGRas.debug(TWGRas.INVENTORY,"Start MSCS Inventory moid="+moid+" class="+className);
       System.out.println("Start MSCS Inventory moid="+moid+" class="+className);
       MSCSAddressEntryShadow tmp = new MSCSAddressEntryShadow();
       try {
           if(addrInfo != null) {       /* If address block provided                       */
               /* Fill in address for target                                               */
               tmp.initAddressEntry(addrInfo, 0, addrInfo.length);
               String ipcPath = tmp.getIPCPath();
               TWGRas.debug(TWGRas.INVENTORY,"Start MSCS Inventory - objId="+moid+"  IPC addr="+ipcPath);
               System.out.println("Start MSCS Inventory - objId="+moid+"  IPC addr="+ipcPath);
               if (ipcPath != null) {
                    startInventoryInternal((int)moid, ipcPath);
               } else {
                   TWGRas.error(TWGRas.INVENTORY,"Unable to get MSCS IPCPath info!");
                   System.out.println("Unable to get MSCS IPCPath info!");
                   inventoryComplete(moid,TWGInvCollector.INV_COLLECTION_COMM_ERROR,0);
               }
           } else {
               TWGRas.error(TWGRas.INVENTORY,"MSCS addrInfo null!");
               System.out.println("MSCS addrInfo null!");
               inventoryComplete(moid,TWGInvCollector.INV_COLLECTION_COMM_ERROR,0);
           }
       } catch (AddressEntryFormatException aefx) {
           TWGRas.error(TWGRas.INVENTORY,"AddressEntryFormatException",aefx);
           System.out.println("AddressEntryFormatException-" + aefx);
           inventoryComplete(moid,TWGInvCollector.INV_COLLECTION_COMM_ERROR,0);
       }
   }
   /**
    * Internal inventory start point
    * @param moid - managed object ID to be inventoried
    * @param ipcPath - IPC path to object
    */
   private void startInventoryInternal(int moid, String ipcPath) {
       System.out.println("Doing MSCS Inventory!!!!");
       try {
           if(sn == null) {
               sn = new ServiceNode();
           }
           /* Start agent on target                                                         */
           MSCSStartInvCommand cmd = new MSCSStartInvCommand(moid, ipcPath);
           sn.SendAsynchCommand(cmd);
       } catch (ServiceNodeException snx) {
           inventoryComplete(moid,TWGInvCollector.INV_COLLECTION_COMM_ERROR,0);               
       } catch (InterruptedException ix) {
           inventoryComplete(moid,TWGInvCollector.INV_COLLECTION_COMM_ERROR,0);               
       }
   }
   private class MSCSStartInvCommand extends Command {
       private int moid;
       private String path;
       private MSCSStartInvCommand(int objId, String ipcPath) {
           super(TWGSvcMgrServiceNodeConstants.SVCMGR_START_SVC);
           SetDestinationAddress(ipcPath + "::" +TWGSvcMgrServiceNodeConstants.SERVICE_NODE);
           byte[] p0 = new byte[IntelByteBuffer.GetASCIIZLength(
               MSCSAgentCommand.CLUSTAGT_SVCNODE_NAME)];
           IntelByteBuffer.WriteASCIIZ(p0, MSCSAgentCommand.CLUSTAGT_SVCNODE_NAME, 0);
           AttachInputParm(p0);
           moid = objId;
           path = ipcPath;
       }
       public void CommandComplete() {
           /* If successful, send get-all command                                           */
           if(ReturnCode() == 0) {
               MSCSInvGetFullConfigCommand cmd = new MSCSInvGetFullConfigCommand(moid,
                   path);
               try {
                   sn.SendAsynchCommand(cmd);
               } catch (ServiceNodeException snx) {
                   inventoryComplete(moid,TWGInvCollector.INV_COLLECTION_COMM_ERROR,0);
               } catch (InterruptedException ix) {
                   inventoryComplete(moid,TWGInvCollector.INV_COLLECTION_COMM_ERROR,0);
               }
           } else {
               inventoryComplete(moid,TWGInvCollector.INV_COLLECTION_COMM_ERROR,0);
           }
       }
   }
   private class MSCSInvGetFullConfigCommand extends MSCSGetFullConfigCommand {
       private int moid;
       private String path;
       private MSCSInvGetFullConfigCommand(int objId, String ipcPath) {
           super(ipcPath);
           moid = objId;
           path = ipcPath;
       }
       public void CommandComplete() {
           /* If successful, process results                                               */
           if(ReturnCode() == 0) {
               System.out.println("Results received for MSCS command!!!!");
               processInventoryData(this, moid);

               inventoryComplete(moid, TWGInvCollector.INV_COLLECTION_SUCCESS, System.currentTimeMillis());
           } else {
               System.out.println("Error getting results : rc=" + Long.toHexString(ReturnCode()));
               inventoryComplete(moid,TWGInvCollector.INV_COLLECTION_COMM_ERROR,0);
           }
           /* Always send service stop                                                     */
           try {
               MSCSStopInvCommand cmd = new MSCSStopInvCommand(moid, path);
               sn.SendAsynchCommand(cmd);
           } catch (ServiceNodeException snx) {
           } catch (InterruptedException ix) {
           }
       }
   }
   private class MSCSStopInvCommand extends Command {
       private int moid;
       private String path;
       private MSCSStopInvCommand(int objId, String ipcPath) {
           super(TWGSvcMgrServiceNodeConstants.SVCMGR_STOP_SVC);
           SetDestinationAddress(ipcPath + "::" + TWGSvcMgrServiceNodeConstants.SERVICE_NODE);
           byte[] p0 = new byte[IntelByteBuffer.GetASCIIZLength(
               MSCSAgentCommand.CLUSTAGT_SVCNODE_NAME)];
           IntelByteBuffer.WriteASCIIZ(p0, MSCSAgentCommand.CLUSTAGT_SVCNODE_NAME, 0);
           AttachInputParm(p0);
           moid = objId;
           path = ipcPath;
       }
   }
   /**
    * Abort method for inventory
    * @param objID - object ID of inventory to abort
    */
   public void stopInventory(long objId) {
      TWGRas.debug(TWGRas.INVENTORY, "Stop MSCS Inventory - objId="+objId);
   }
   /**
    * Collector shutdown method
    */
   public void shutdown() {
       TWGRas.debug(TWGRas.INVENTORY,"Shutdown MSCS Inventory");
       if(sn != null) {
           try { sn.Close(); } catch (ServiceNodeException snx) {}
           sn = null;
       }
   }
   /**
    * Base handler method for processing received inventory data
    */
   private void processInventoryData(MSCSGetFullConfigCommand cmd, int moid) {
       /* Add resource data to database                                           */
       processResourceInventoryData(cmd, moid);
       /* Add group data to database                                              */
       processGroupInventoryData(cmd, moid);
       /* Add network interface data to database                                  */
       processNetIntInventoryData(cmd, moid);
       /* Add group-resource map data to database                                 */
       processGroupResInventoryData(cmd, moid);
       /* Add cluster member table                                                */
       processMemberInventoryData(cmd, moid);
       /* Add cluster info table                                                  */
       processClusterInfoInventoryData(cmd, moid);
   }
   /**
    * Handler for resource table contributions
    */
   private void processResourceInventoryData(MSCSGetFullConfigCommand cmd, int moid) {
       TWGDbStoreRow row;
       /* Create row replace command                                               */
       TWGDbReplaceManagedObjCmd dbCmd = new TWGDbReplaceManagedObjCmd(moid);
       int num_parms = cmd.getNumberOfElements();       /* Get number of records   */

       for(int i=0; i < num_parms; i++) {
           /* If not resource, skip it                                             */
           if(cmd.getElementType(i) != cmd.CLUSTER_ENUM_RESOURCE) {
               continue;
           }
           // Store row in database
           row = new TWGDbStoreRow(MSCSResourceTable.TABLE_NAME);
           row.addColumnValue(MSCSResourceTable.MANAGED_OBJ_ID, moid);
           String val = cmd.getElementResourceType(i);
           row.addColumnValue(MSCSResourceTable.MSCS_RES_TYPE, 
               ((val != null)?val:" "));
           val = cmd.getElementName(i);
           row.addColumnValue(MSCSResourceTable.MSCS_RES_NAME, 
               ((val != null)?val:" "));
           val = cmd.getElementResourceDependentNetwork(i);
           row.addColumnValue(MSCSResourceTable.MSCS_RES_DEP_NET, 
               ((val != null)?val:" "));
           val = cmd.getElementResourceCurrentOwner(i);
           row.addColumnValue(MSCSResourceTable.MSCS_RES_CUR_OWN, 
               ((val != null)?val:" "));
           val = cmd.getElementResourceOwnerNode(i);
           row.addColumnValue(MSCSResourceTable.MSCS_RES_NODE, 
               ((val != null)?val:" "));
           dbCmd.addRow(row);
       }
       sendToDb(TWGDbReplaceManagedObjCmd.TWG_DB_REPLACE_MANAGEDOBJ_CMD, dbCmd);
   }
   /**
    * Handler for group table contributions
    */
   private void processGroupInventoryData(MSCSGetFullConfigCommand cmd, int moid) {
       TWGDbStoreRow row;
       /* Create row replace command                                               */
       TWGDbReplaceManagedObjCmd dbCmd = new TWGDbReplaceManagedObjCmd(moid);
       int num_parms = cmd.getNumberOfElements();       /* Get number of records   */

       for(int i=0; i < num_parms; i++) {
           /* If not resource, skip it                                             */
           if(cmd.getElementType(i) != cmd.CLUSTER_ENUM_GROUP) {
               continue;
           }
           // Store row in database
           row = new TWGDbStoreRow(MSCSGroupTable.TABLE_NAME);
           row.addColumnValue(MSCSGroupTable.MANAGED_OBJ_ID, moid);
           String val = cmd.getElementName(i);
           row.addColumnValue(MSCSGroupTable.MSCS_GRP_NAME, 
               ((val != null)?val:" "));
           val = cmd.getElementGroupNodeName(i);
           row.addColumnValue(MSCSGroupTable.MSCS_GRP_NODE, 
               ((val != null)?val:" "));
           dbCmd.addRow(row);
       }
       sendToDb(TWGDbReplaceManagedObjCmd.TWG_DB_REPLACE_MANAGEDOBJ_CMD, dbCmd);
   }
   /**
    * Handler for network interface table contributions
    */
   private void processNetIntInventoryData(MSCSGetFullConfigCommand cmd, int moid) {
       TWGDbStoreRow row;
       Hashtable net_by_int = new Hashtable();
       Hashtable node_by_int = new Hashtable();
       StringValueSet interfaces = new StringValueSet();

       int num_parms = cmd.getNumberOfElements();       /* Get number of records   */

       for(int i=0; i < num_parms; i++) {
           if(cmd.getElementType(i) == cmd.CLUSTER_ENUM_NODE) { /* If node         */
               /* Get interfaces associated with node                              */
               String[] ni_per_node = cmd.getElementNodeNetworkInterfaceList(i);
               String nodename = cmd.getElementName(i);
               for(int j = 0; j < ni_per_node.length; j++) {
                   node_by_int.put(ni_per_node[j], nodename);
                   interfaces.InsertValue(ni_per_node[j]);
               }
           }
           else if(cmd.getElementType(i) == cmd.CLUSTER_ENUM_NETWORK) { /* If net  */
               String[] ni_per_net = cmd.getElementNetworkNetworkInterfaceList(i);
               String netname = cmd.getElementName(i);
               for(int j = 0; j < ni_per_net.length; j++) {
                   net_by_int.put(ni_per_net[j], netname);
                   interfaces.InsertValue(ni_per_net[j]);
               }
           }
           else if(cmd.getElementType(i) == cmd.CLUSTER_ENUM_NETINTERFACE) {
               interfaces.InsertValue(cmd.getElementName(i));
           }
       }
       /* Create row replace command                                               */
       TWGDbReplaceManagedObjCmd dbCmd = new TWGDbReplaceManagedObjCmd(moid);
       /* Add rows for each network interface found                                 */
       for(int i = 0; i < interfaces.Length(); i++) {
           // Store row in database
           row = new TWGDbStoreRow(MSCSNetIntTable.TABLE_NAME);
           row.addColumnValue(MSCSNetIntTable.MANAGED_OBJ_ID, moid);
           String intf = interfaces.GetValue(i);
           String val = (String)net_by_int.get(intf);
           row.addColumnValue(MSCSNetIntTable.MSCS_NET_NAME, 
               ((val != null)?val:" "));
           val = intf;
           row.addColumnValue(MSCSNetIntTable.MSCS_NETINT_NAME, 
               ((val != null)?val:" "));
           val = (String)node_by_int.get(intf);
           row.addColumnValue(MSCSNetIntTable.MSCS_NODE_NAME, 
               ((val != null)?val:" "));
           dbCmd.addRow(row);
       }
       sendToDb(TWGDbReplaceManagedObjCmd.TWG_DB_REPLACE_MANAGEDOBJ_CMD, dbCmd);
   }
   /**
    * Handler for group resource mapping table contributions
    */
   private void processGroupResInventoryData(MSCSGetFullConfigCommand cmd, int moid) {
       TWGDbStoreRow row;
       /* Create row replace command                                               */
       TWGDbReplaceManagedObjCmd dbCmd = new TWGDbReplaceManagedObjCmd(moid);
       int num_parms = cmd.getNumberOfElements();       /* Get number of records   */

       for(int i=0; i < num_parms; i++) {
           /* If not resource, skip it                                             */
           if(cmd.getElementType(i) != cmd.CLUSTER_ENUM_GROUP) {
               continue;
           }
           String grp_name = cmd.getElementName(i);
           String[] res_list = cmd.getElementGroupResourceList(i);
           for(int j = 0; j < res_list.length; j++) {
               // Store row in database
               row = new TWGDbStoreRow(MSCSGroupResTable.TABLE_NAME);
               row.addColumnValue(MSCSGroupResTable.MANAGED_OBJ_ID, moid);
               row.addColumnValue(MSCSGroupResTable.MSCS_GRP_NAME, 
                   ((grp_name != null)?grp_name:" "));
               row.addColumnValue(MSCSGroupResTable.MSCS_RES_NAME, 
                   res_list[j]);
               dbCmd.addRow(row);
           }
       }
       sendToDb(TWGDbReplaceManagedObjCmd.TWG_DB_REPLACE_MANAGEDOBJ_CMD, dbCmd);
   }
   /**
    * Handler for cluster member table contributions
    */
   private void processMemberInventoryData(MSCSGetFullConfigCommand cmd, int moid) {
       TWGDbStoreRow row;
       /* Create row replace command                                               */
       TWGDbReplaceManagedObjCmd dbCmd = new TWGDbReplaceManagedObjCmd(moid);
       int num_parms = cmd.getNumberOfElements();       /* Get number of records   */

       for(int i=0; i < num_parms; i++) {
           String elem_type = TWGClusterMemberTable.CLUS_MEMBER_TYPE_GENERIC;
           long type = cmd.getElementType(i);
           int elem_moid = 0;
           if(type == cmd.CLUSTER_ENUM_NODE) {
               elem_type = TWGClusterMemberTable.CLUS_MEMBER_TYPE_NODE;
           }
           else if(type == cmd.CLUSTER_ENUM_NETWORK) {
               elem_type = TWGClusterMemberTable.CLUS_MEMBER_TYPE_NET;
           }
           else if(type == cmd.CLUSTER_ENUM_NETINTERFACE) {
               elem_type = TWGClusterMemberTable.CLUS_MEMBER_TYPE_NETINT;
           }
           else if(type == cmd.CLUSTER_ENUM_GROUP) {
               elem_type = TWGClusterMemberTable.CLUS_MEMBER_TYPE_RESGRP;
           }
           else if(type == cmd.CLUSTER_ENUM_RESOURCE) {
               String res_type = cmd.getElementResourceType(i);
               if(res_type.equals("File Share")) {
                   elem_type = TWGClusterMemberTable.CLUS_MEMBER_TYPE_FILESHARE;
               }
               else if(res_type.equals("Generic Application")) {
                   elem_type = TWGClusterMemberTable.CLUS_MEMBER_TYPE_APP;
               }
               else if((res_type.equals("Generic Service")) ||
                   (res_type.equals("Time Service")) ||
                   (res_type.equals("Distributed Transaction Coordinator"))) {
                   elem_type = TWGClusterMemberTable.CLUS_MEMBER_TYPE_SVC;
               }
               else if(res_type.equals("IP Address")) {
                   elem_type = TWGClusterMemberTable.CLUS_MEMBER_TYPE_IPADDR;
               }
               else if(res_type.equals("Microsoft Message Queue Server")) {
                   elem_type = TWGClusterMemberTable.CLUS_MEMBER_TYPE_QUEUE;
               }
               else if(res_type.equals("Network Name")) {
                   elem_type = TWGClusterMemberTable.CLUS_MEMBER_TYPE_NETNAME;
               }
               else if(res_type.equals("Physical Disk")) {
                   elem_type = TWGClusterMemberTable.CLUS_MEMBER_TYPE_PHYSDISK;
               }
               else if(res_type.equals("Print Spooler")) {
                   elem_type = TWGClusterMemberTable.CLUS_MEMBER_TYPE_PRINTSPOOL;
               }
               else {
                   elem_type = TWGClusterMemberTable.CLUS_MEMBER_TYPE_RES;
               }
           }
           else {
               elem_type = null;
           }
           if(elem_type != null) {
               // Store row in database
               row = new TWGDbStoreRow(TWGClusterMemberTable.TABLE_NAME);
               row.addColumnValue(TWGClusterMemberTable.MANAGED_OBJ_ID, moid);
               row.addColumnValue(TWGClusterMemberTable.CLUS_MEMBER_NAME, 
                   cmd.getElementName(i));
               row.addColumnValue(TWGClusterMemberTable.CLUS_MEMBER_TYPE, elem_type);
               row.addColumnValue(TWGClusterMemberTable.MEMBER_MANAGED_OBJ_ID, elem_moid);
               dbCmd.addRow(row);
           }
       }
       sendToDb(TWGDbReplaceManagedObjCmd.TWG_DB_REPLACE_MANAGEDOBJ_CMD, dbCmd);
   }
   /**
    * Handler for cluster information table contributions
    */
   private void processClusterInfoInventoryData(MSCSGetFullConfigCommand cmd, int moid) {
       TWGDbStoreRow row;
       /* Create row replace command                                               */
       TWGDbReplaceManagedObjCmd dbCmd = new TWGDbReplaceManagedObjCmd(moid);

       // Store row in database
       row = new TWGDbStoreRow(MSCSClusterInfoTable.TABLE_NAME);
       row.addColumnValue(MSCSClusterInfoTable.MANAGED_OBJ_ID, moid);
       String val = cmd.getClusterName();
       row.addColumnValue(MSCSClusterInfoTable.MSCS_NAME,
           ((val != null)?val:" "));
       row.addColumnValue(MSCSClusterInfoTable.MSCS_MAJ_VER, cmd.getClusterMajorVersion());
       row.addColumnValue(MSCSClusterInfoTable.MSCS_MIN_VER, cmd.getClusterMinorVersion());
       row.addColumnValue(MSCSClusterInfoTable.MSCS_BUILD, cmd.getClusterBuildNumber());
       val = cmd.getClusterVendorID();
       row.addColumnValue(MSCSClusterInfoTable.MSCS_VENDOR, ((val != null)?val:" "));
       val = cmd.getClusterCSDVersion();
       row.addColumnValue(MSCSClusterInfoTable.MSCS_CSD, ((val != null)?val:" "));
       dbCmd.addRow(row);
       sendToDb(TWGDbReplaceManagedObjCmd.TWG_DB_REPLACE_MANAGEDOBJ_CMD, dbCmd);
   }
}

