package com.tivoli.twg.mscs;

import com.tivoli.twg.engine.*;
import java.util.Vector;

/**
 * <P>SQL database table for MSCS Network-Interface Table information
 *
 * <p>Primary key - MANAGED_OBJ_ID, MSCS_NET_NAME, MSCS_NETINT_NAME
 *
 * <p>
 * <table width=450 border=5 cols=14>
 * <!- Table Name>
 * <tr><td colspan=14 align=center><strong>MSCS_NET_INT</strong></td>
 * <!- Table description >
 * <tr><td><strong>Field Name</strong></td>
 * <td><strong>Data Type</strong></td>
 * <td><strong>Description</strong></td>
 * <td><strong>NT/2000</strong></td>
 * <td><strong>95/98</strong></td>
 * <td><strong>OS/2</strong></td>
 * <td><strong>Netware</strong></td>
 * <td><strong>AS/400</strong></td>
 * <td><strong>SNMP</strong></td>
 * <td><strong>AIX</strong></td>
 * <td><strong>HPUX</strong></td>
 * <td><strong>Solaris</strong></td>
 * <td><strong>Unixware</strong></td>
 * <td><strong>Linux</strong></td>
 * <!- Table rows >
 * <tr><td>MANAGED_OBJ_ID</td>         <td>INT</td>       <td>Managed object id (of cluster)</td>
 *     <td>X</td> <td> </td> <td> </td> <td> </td> <td> </td> <td> </td> <td> </td> <td> </td> <td> </td> <td> </td> <td> </td> 
 * <tr><td>MSCS_NET_NAME</td>          <td>CHAR(64)</td>  <td>Network Name</td>
 *     <td>X</td> <td> </td> <td> </td> <td> </td> <td> </td> <td> </td> <td> </td> <td> </td> <td> </td> <td> </td> <td> </td> 
 * <tr><td>MSCS_NETINT_NAME</td>       <td>CHAR(64)</td>  <td>Network Interface Name</td>
 *     <td>X</td> <td> </td> <td> </td> <td> </td> <td> </td> <td> </td> <td> </td> <td> </td> <td> </td> <td> </td> <td> </td> 
 * <tr><td>MSCS_NODE_NAME</td>         <td>CHAR(64)</td>  <td>Node Name</td>
 *     <td>X</td> <td> </td> <td> </td> <td> </td> <td> </td> <td> </td> <td> </td> <td> </td> <td> </td> <td> </td> <td> </td> 
 * </table>
 */
public class MSCSNetIntTable extends TWGDatabaseTable {
    /* SQL Table Name                                                                       */
    public static final String TABLE_NAME = "MSCS_NET_INT";

    /* SQL Table Attributes                                                                 */
    public static final String MANAGED_OBJ_ID     = "MANAGED_OBJ_ID";
    public static final String MSCS_NET_NAME      = "MSCS_NET_NAME";
    public static final String MSCS_NETINT_NAME   = "MSCS_NETINT_NAME";
    public static final String MSCS_NODE_NAME     = "MSCS_NODE_NAME";

    /* table version                                                                        */
    public static final String TABLE_VERSION                = "1.0";
    /**
     * Default constructor for MSCS Network-Interface table
     */
    public MSCSNetIntTable() {
        super();
    }
    /**
     * Table create method
     * @exception TWGDbException if error creating table
     */
    public void create() throws TWGDbException {
        //-define SQL database name
        String db = TWGDatabase.TWG_DB_NAME;
        String tableName = TABLE_NAME;
        String tableTokenName = TABLE_NAME;
        //----------------------------------------------------------------------
        //-Create the columns
        //----------------------------------------------------------------------
        TWGDbColumn column;
        TWGDbIntegerColumn intColumn;
        TWGDbColumnData column_data = new TWGDbColumnData();

        intColumn = new TWGDbIntegerColumn(MANAGED_OBJ_ID, MANAGED_OBJ_ID);
        intColumn.setFilterColumn(false);
        column_data.addColumn(intColumn);

        column = new TWGDbCharColumn(MSCS_NET_NAME, MSCS_NET_NAME, 64);
        column.setDefaultValue("");
        column_data.addColumn(column);

        column = new TWGDbCharColumn(MSCS_NETINT_NAME, MSCS_NETINT_NAME, 64);
        column.setDefaultValue("");
        column_data.addColumn(column);

        column = new TWGDbCharColumn(MSCS_NODE_NAME, MSCS_NODE_NAME, 64);
        column.setDefaultValue("");
        column_data.addColumn(column);

        //---------------------------------------------------------------------
        // parms object to hold optional parms (key, key unique, and additional indices
        //---------------------------------------------------------------------
        TWGDbTableParms parms = new TWGDbTableParms();
        parms.setKey(MANAGED_OBJ_ID + "," + MSCS_NET_NAME + "," + MSCS_NETINT_NAME);
        parms.setKeyUnique(true);
        parms.setGraphicName("com/tivoli/twg/inventory/images/database_group_s.gif");

        //---------------------------------------------------------------------
        // NLS bundle for the table
        //---------------------------------------------------------------------
        String tableBundle = "com.tivoli.twg.mscs.MSCSNLS";

        //---------------------------------------------------------------------
        // Negotiate the SQL database name
        //---------------------------------------------------------------------
        int i = 1;
        String saveTableName      = tableName;
        String saveTableTokenName = tableTokenName;

        boolean tableNegotiationComplete = false;

        while (tableNegotiationComplete == false) {
            try {
                this.TWGInitTable(db, tableTokenName, tableName, column_data, parms,
                    tableBundle);
                tableNegotiationComplete = true;
            } catch (TWGDbRealTableNameNotUniqueException e) {
                tableName = saveTableName + String.valueOf(i);
                tableTokenName = saveTableTokenName + String.valueOf(i);
                i++;
            } catch (TWGDbTokenTableNameNotUniqueException e) {
                tableName = saveTableName + String.valueOf(i);
                tableTokenName = saveTableTokenName + String.valueOf(i);
                i++;
            }
        }
    }
    /**
     * Get the version number for the table
     * @returns the table version
     */
    public String getTableVersion() {
      return TABLE_VERSION;
    }
}
