/////////////////////////////////////////////////////////
// Module		MatchesPanel.java
// Version		3.10
// Language		Java
// Author		Scott Riley
// History
// 06/09/00    Release 3.10
// 10/26/99    Initial release
//
// Copyright (C) 1999-2000 Tivoli Systems, an IBM Company  
//
////////////////////////////////////////////////////////
import java.awt.*;
import java.awt.event.*;
import java.net.*;
import java.util.Vector;

public class MatchesPanel extends Panel implements ItemListener {
    FindFrame myFrame;
    List resultsList;
    Vector urls = new Vector();
    Label numOfResults;
    int numOfHits = 0;
    int numSearchedPages = 0;
    int numPagesToSearch = 0;

    MatchesPanel(FindFrame mf)
    {
        myFrame = mf;
        urls.ensureCapacity(mf.sdkFind.totalPages);
        Label headingLabel = new Label("Search Results", Label.LEFT);
        numOfResults = new Label("", Label.LEFT);
        resultsList = new List(7, false);
        Font listFont = new Font("Monospaced",12,12); 
        if (listFont != null) {
            resultsList.setFont(listFont);
        }

        resultsList.setForeground(Color.blue);      
        resultsList.repaint();                        
        // setLayout(new GridLayout(3,1));
        setLayout(new BorderLayout(10,10));

        add("North",headingLabel);
        add("Center",resultsList);
        add("South",numOfResults);
        resultsList.addItemListener(this);
    }

    // This method is invoked when a new search
    // is initiated or when the 'Clear' button is clicked.
    public void clearList() 
    {
        urls.removeAllElements();
        numOfHits = 0;
        numOfResults.setText(" ");
        resultsList.removeAll();

        numSearchedPages = 0;
        numPagesToSearch = 0;
    }

    private void updateResults()
    {
        String resultsMsg = "";
        if (1 == numOfHits)
            resultsMsg = "Match found on 1 page.";
        else
            resultsMsg = "Match found on " + numOfHits + " pages.";


        if (numSearchedPages > 0) {
            resultsMsg += "  Searched " + numSearchedPages + " of " 
                + numPagesToSearch + " pages.";
        }

        numOfResults.setText(resultsMsg);


    }


    // This method is invoked when a search match is found
    public void addResult(String match, String url, int searched, int totalPages)
    {

        numSearchedPages = searched;
        numPagesToSearch = totalPages;

        numOfHits ++;
        urls.addElement(new String(url));

        resultsList.add(match);
        updateResults();
    }


    // This method is invoked when a search match is found
    public void updateProgress(int searched, int totalPages)
    {
        numSearchedPages = searched;
        numPagesToSearch = totalPages;
        updateResults();
    }


    // Draw a 3-D box around this panel.
    public void paint(Graphics g)
    {
        Dimension d = getSize();
        Color bg = getBackground();

        g.setColor(bg);
        g.draw3DRect(0,0,d.width-1,d.height-1,true);
        g.draw3DRect(2,2,d.width-5,d.height-5,false);
    }


    // The panel should at least be this size
    public Dimension getMinimumSize()
    {
        Dimension d = new Dimension(400,120);
        return d;
    }


    // The padding for the chrome
    public Insets getInsets()
    {
        return new Insets(3,15,3,15);
    }


    // Would prefer the panel to be this size
    public Dimension getPreferredSize()
    {
        Dimension d = new Dimension(400,120);
        return d;
    }

    transient ItemListener itemListener;

    // Event handler
    protected void processItemEvent (ItemEvent evt) {
        if (itemListener != null) {
            itemListener.itemStateChanged(evt);
        }
    }

    public void itemStateChanged(ItemEvent evt) {
        // A match in the list was selected.
        switch (evt.getStateChange()) {
        case ItemEvent.SELECTED:
            int sIndex = ((Integer)evt.getItem()).intValue();
            URL dest;
            try {
                dest = new URL((String) urls.elementAt(sIndex));
                myFrame.sdkFind.getAppletContext().showDocument(dest, "right");
            } catch (MalformedURLException excep) {
                System.out.println("SDKFind: Bad URL - Ignoring.");
            }
        }
    }
}
