package com.BobCo.inventory;

import java.util.Vector;
import com.tivoli.twg.inventory.*;
import com.tivoli.twg.engine.*;
import com.tivoli.twg.log.TWGRas;


public class SampleInventoryCollectorExtension extends TWGExtension {

   public static final String INV_AGENT_SVC_NAME      = "NativeMO.BobCoInventoryServiceNode";
   public static final String INV_RESOURCE_BUNDLE     = "";
   public static final String CLASSNAME_NATIVE_MO     = "com.tivoli.twg.engine.TWGNativeManagedObject";
   public static final String CLASSNAME_INV_COLLECTOR = "com.BobCo.inventory.BobCoSampleInventoryCollector";

   private static String bundleClassName = "com.BobCo.inventory.BobCoTableResources";

    /**
     * Default constructor
     */
    public SampleInventoryCollectorExtension() {
        super();
    }

    /**
     * First initialization method, intended to allow extensions to register
     * classes using RegisterClass method.
     *
     * @exception TWGExtensionInitException if a fatal error occurs during
     * the initialization method
     */
    public void InitClassRegistration() throws
        TWGExtensionInitException {
	String  errmsg = "";
        try {
            // now register all of our persistent object classes here
            // Register UNIX inventory collector associated with Native MO and
            // the collector's service name on the agent
            TWGInvCollector.registerCollectorForService(this,
                                                        CLASSNAME_INV_COLLECTOR,
                                                        CLASSNAME_NATIVE_MO, INV_AGENT_SVC_NAME);


	
        TWGRas.exit(TWGRas.ENGINE,"Extension initialized classes");
        } catch (ClassNotFoundException cnfx) {
            throw new TWGExtensionInitException("Exception: " +
                cnfx);
        } catch (IllegalAccessException e) {
           throw new TWGExtensionInitException(e.toString());
        } catch (InstantiationException e) {
           throw new TWGExtensionInitException(e.toString());
        }
    }

    /**
     * Second initialization method, intended to allow extension to create
     * default instances of classes registered during InitClassRegistration().
     *
     * @exception TWGExtensionInitException if a fatal error occurs during
     * the initialization method
     */
    public void InitClassInstances() throws TWGExtensionInitException {
       String tableName = "";
       String errmsg = "";
       if (TWGDatabase.isActive()) {
	try {
           // Add inventory collector's tables to base filter factory
           TWGInvCollectorConfig.UpdateFilterFactory(CLASSNAME_INV_COLLECTOR,
                                                     "com.tivoli.twg.inventory",
                                                     INV_RESOURCE_BUNDLE,
                                                     "FactoryName.Inventory.BobCo");

           // Create default queries for inventory collector's tables
	   String[] createqueryclasses = BobCoSampleInventoryCollector.getQueryDatabaseTableClassNames();
	   if (createqueryclasses != null) {
	       for (int i=0;i<createqueryclasses.length;i++) {
		       TWGInvCollectorConfig.GetCreateDefaultQuery(TWGDatabaseTable.findTableByClassName(
								createqueryclasses[i]), INV_RESOURCE_BUNDLE);
	       }
	   }




	//---------------------------------------------------------------
        // Create a default filters for our extension.  This example uses IT
        // Director inventory tables to create a filter of all systems that
        // have Pentium III processors and Motorola BobCo Devices
        //---------------------------------------------------------------
                                                                 
        // The filter id string is used to identify and search for the filter.  It must be
        // unique across all filters created by all Director extensions.
        String filterId = "BobCo.Filter.PentiumIII_and_Motorola";

        // see if the filter we want to create already exists.  If not, create it
        if (TWGFilter.findByIDString(filterId) == null) try
        {
           // create the parameters object for the filter, specifying the bundle key that will be
           // used to search for the translated name of the filter
           TWGFilterParms parms = new TWGFilterParms(("FilterName.PentiumIII_and_Motorola"));
           parms.setBundleName(bundleClassName);

           // create the parent constraint (constraint 0) and add it to the parameters.
           // This constraint tells what operator  to use to combine the constraints
           // below it (constraints 0.1 and 0.2)
           parms.addConstraint( new TWGFilterConstraint( "0" , TWGFilterConstraint.ALL ) );


           // get the token name of the processor table and the operating system table
           tableName = "com.tivoli.twg.inventory.TWGProcessorTable";
           String processorToken = TWGDatabaseTable.findTableByClassName(tableName).getTokenName();
           tableName = "com.BobCo.inventory.BobCoDeviceTable";
	   String deviceTableToken = TWGDatabaseTable.findTableByClassName(tableName).getTokenName();

           // create a column filter specifying the processor table, the processor type column, the operator,
           // and the value of PENTIUMIII
           TWGDbStringColumnFilter fil =
              new TWGDbStringColumnFilter(processorToken, TWGProcessorTable.FAMILY , TWGDbColumnOperators.EQUAL);
           fil.addColumnValue(TWGProcessorTable.FAMILY_PENTIUMIII);

           // create a constraint using that column filter and add it to the filter parameters.
           // Specify the filter factory this table can be found in
           parms.addConstraint(new TWGFilterConstraint( "0.1" , "com.tivoli.twg.inventory", fil) );

           // create a column filter specifying the BobCo Device table, the BobCo Device chipset
           // column, the operator, and the value of MOTOROLA
           //  create a second column filter  and a second constraint with it ("0.2")
           fil = new TWGDbStringColumnFilter(deviceTableToken , BobCoDeviceTable.DEVICE_CHIPSET, TWGDbColumnOperators.EQUAL);
           fil.addColumnValue(BobCoDeviceTable.DEVICE_CHIPSET_MOTOROLA);

           // create a constraint using the second columnn filter and add it to the filter parameters
           parms.addConstraint( new TWGFilterConstraint( "0.2" , "com.tivoli.twg.inventory", fil) );

           // create the filter
           TWGFilter filter = new TWGFilter(parms);
           // filter is read only
           filter.setReadOnly(true);
           // set id string that uniquely identifies this filter
           filter.setIDString(filterId);
           // save the filter
           filter.save();

           try
           {
              System.out.println("new filter created. Persistent id is "  + filter.PersistID());
           }
           catch (Exception e)
           {
              System.out.println(e.getMessage());
           }

        } 
	catch (TWGDbException e)
        {
           errmsg = "TWGDbException: SampleInventoryCollectorExtension: exception initializing inventory filters for "
													  + tableName;
           TWGRas.error( TWGRas.EX1, errmsg , e);
        }
        catch (TWGFilterInitException e)
        {
           errmsg = "TWGFileterInitException: SampleInventoryCollectorExtension: exception initializing inventory filters for " 
													           + tableName;
           TWGRas.error( TWGRas.EX1, errmsg , e);
        }
        catch (TWGFilterException e)
        {
           errmsg = "TWGFilterExection: SampleInventoryCollectorExtension: exception initializing inventory filters for " 
													     + tableName;
           TWGRas.error( TWGRas.EX1, errmsg , e);
        }
        catch ( TWGPersistentObjectSaveException e )
        {
           errmsg = "TWGPersistentObjectSaveException: SampleInventoryCollectorExtension: exception saving inventory filters for " 
														      + tableName;
           TWGRas.error( TWGRas.EX1, errmsg , e);
	}


	} catch (TWGDbException e) {
	    throw new TWGExtensionInitException(e.toString());
	}
      }
    }
    /*
     * Final initialization method, intended to allow an extension to do any
     * actions which may have required the other extensions to have completed
     * the previous passes of initialization.
     *
     * @exception TWGExtensionInitException if a fatal error occurs during
     * the initialization method
     */
    public void InitCompletion() throws TWGExtensionInitException {
    }

    /**
     * First termination method, intended to allow an extension to begin
     * the process of termination.  The only termination processing which
     * should occur here is that which needs to be done before other
     * extensions have executed TermCompletion() pass.
     *
     * @exception TWGExtensionTermException if an error occurs during the
     * termination method
     */
    public void TermBegin() throws TWGExtensionTermException {
    }

    /**
     * Final termination method, intended to allow an extension to terminate
     * any processing which may still be active.
     *
     * @exception TWGExtensionTermException if an error occurs during the
     * termination method
     */
    public void TermComplete() throws TWGExtensionTermException {
    }

}
