package com.BobCo.subagent;

/**
 * Sample IT Director Agent Extension (Subagent).
 * This sample subagent is a ServiceNode subclass.
 *
 * This class logs trace messages using flag TWGRas.EX0 (reserved for OEMs).
 */
import com.tivoli.twg.libs.*;
import com.tivoli.twg.agent.*;
import com.tivoli.twg.log.TWGRas;
import java.util.*;

public class SampleInventorySubagent extends ServiceNode
{
   private static final String SERVICE_NODE_NAME = "BobCoInventoryServiceNode";

   private static final int MY_INVENTORY_CMD                     =  0x200;

   /**
    * Default constructor
    */
   public SampleInventorySubagent()
   throws BadServiceNodeImplException, ServiceNodeException
   {
      super(SERVICE_NODE_NAME);
      TWGRas.entryExit(TWGRas.EX0,"Sample Inventory Subagent default constructor");
   }

   public boolean CommandReceived(Command cmd) {
      int rc;
      int cc = (int)cmd.CommandCode();
      TWGRas.entryExit(TWGRas.EX0,"Sample Inventory Subagent received command code " + cc);
      switch (cc) {
      case MY_INVENTORY_CMD:
	    TWGRas.entryExit(TWGRas.INVENTORY,"SampleSubagent received MY_INVENTORY_CMD command");
	    //  This is where you would call whatever code you need to gather the appropriate inventory data
	    //  in This Example, well just gather a few bits of data straight from java code.
	    //  You may wish to use JNI/C-code to gather data more specific data on your platform. 
	    GatherInventoryData(cmd);
	    rc = 0;
	    break;
      default:
	    rc = 1;
	    TWGRas.entryExit(TWGRas.INVENTORY,"SampleSubagent received unknown command code " + cc);
	    break;
      }
      cmd.SetReturnCode(rc);
      return true;
   }


   /**
   *  Sample Gathering of data....
   */
    public int GatherInventoryData(Command cmd) {

        // a Few fields for the Existing Operating System Table...

	int index = 1; // Only one OS on this system.....
        String osname = new String("OS Name"); // System.getProperty("os.name");
	String ostype = new String("OS Type"); // System.getProperty("os.type");
	String primary = new String("TRUE");
        String version = new String("My OS value"); // System.getProperty("os.value");
        String revision = new String("os.revision");


        int length = osname.length() + ostype.length() + primary.length() + version.length() + revision.length() + 4 + 10;

	TWGRas.debug(TWGRas.INVENTORY,"The length is: " + length);

        IntelByteBuffer databuf = new IntelByteBuffer(length);
	
        databuf.WriteULONG(index);
	databuf.WriteASCIIZ(osname);
	databuf.WriteASCIIZ(ostype);
	databuf.WriteASCIIZ(primary);
        databuf.WriteASCIIZ(version);
        databuf.WriteASCIIZ(revision);
        cmd.AddOutputParm(databuf.GetBuffer());
	TWGRas.debug(TWGRas.INVENTORY,"Finished with the Operating System table");

        // Now a few fields for the new Bobco Device Table....

        int    device_index = 1;
	String device_type = new String("BobCo Device");
	Date now = new Date();
	long device_date = now.getTime();
	String chipset = new String("MOTOROLA");
	double size = 1234567.890;


        IntelByteBuffer device_buf = new IntelByteBuffer(4 + device_type.length()+ 4 + chipset.length() + 10);
	device_buf.WriteULONG(device_index);
        device_buf.WriteASCIIZ(device_type);
	device_buf.WriteULONG(device_date);
	device_buf.WriteASCIIZ(chipset);
	device_buf.WriteDouble(size);
	TWGRas.debug(TWGRas.INVENTORY,"Finished with the BobCo Device table");
        cmd.AddOutputParm(device_buf.GetBuffer());

      return 0;  // everything went A_OK

    }
}

