package com.tivoli.twg.mscs;

import com.tivoli.twg.mscs.*;
import com.tivoli.twg.libs.*;
import java.util.*;
import java.awt.*;


//import java.io.*;
//import java.net.*;
//import java.lang.*;
import java.beans.*;
//import java.applet.*;
//import java.awt.event.*;


import javax.swing.*;
import javax.swing.border.*;
import javax.swing.event.*;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import com.tivoli.twg.guilibs.WindowAttributes;
import com.tivoli.twg.guilibs.WindowAttributeListener;
import com.tivoli.twg.guilibs.WindowAttributeEvent;
import com.tivoli.twg.guilibs.*;
import javax.swing.SwingConstants;
import java.awt.event.*;
import java.lang.Thread;
import com.tivoli.uif.core.*;
import com.tivoli.uif.controls.*;
import com.tivoli.twg.console.*;
import com.tivoli.twg.engine.*;

public class TWGMSCSBrowser extends TWGTaskFrame
                           implements ActionListener,
                                      PropertyChangeListener,
                                      WindowAttributeListener
{

   private   TWGMSCSClassBrowser  Browser     = null;
   private   MultiLocaleBundle   mlb         = null;
   private   Container           Panel       = null;

   public static final int BR_INITIAL_WIDTH  = 500;
   public static final int BR_INITIAL_HEIGHT = 500;
   //menus
   private JMenuBar mb = null;
   private JMenu file = null;
   private JMenu help = null;
   private JMenuItem fRefresh = null;
   private JMenuItem fExit = null;
   private JMenuItem hContents = null;
   private JMenuItem hIndex = null;

  // Name of our NLS Bundle class
  private static final String ADDRESSNLSCLASS =
       "com.tivoli.twg.mscs.TWGMSCSResources";


  public String getNlsString(String ID) {

     String name = null;
     if (mlb!=null) {
         try
         {
            name = mlb.getString(Locale.getDefault(),ID);
         } catch (MissingResourceException mrx) {}
     }
     return name;
  }


  /**
   * Initialization routine for setting up non-gui items.
   */
  public boolean pInit( )
  {
     boolean RC = true;

        try
        {
            mlb = MultiLocaleBundle.getSharedBundle(ADDRESSNLSCLASS);
        } catch (MissingResourceException mrx)
        {
            mlb = null;
        }
        RC = BuilditNow();
        return RC;
  } // end pInit

  /**
   * Routine to allow task frames to clean up.
   */
  public void pCleanUp( )
  {
     if ( Browser != null)
     {
        Browser.pCleanUp();
        Browser = null;
     }

     mb = null;
     file = null;
     if (fRefresh != null) {
         fRefresh.removeActionListener(this);
         fRefresh = null;
     }
     if (fExit != null) {
         fExit.removeActionListener(this);
         fExit = null;
     }

     help = null;
     if (hContents != null) {
        getHelpContext().removeHelp( hContents );
        hContents = null;
     }
     if (hIndex != null) {
        getHelpContext().removeHelp( hIndex );
        hIndex = null;
     }
     dispose();
  }
  /**
   * This method is used to clean-up resources when the
   * window is disposed.
   */
  public void dispose ()
  {
     // cleanup
     Panel           = null;
     super.dispose();
  }

   /**
    * Returns the frame size that is used when the frame is
    * initially created.  This is an overridden  method to
    * provide a large enough frame.
    */
   public Dimension getFrameSize()
   {
      if ( Browser != null )
      {
         return new Dimension(BR_INITIAL_WIDTH, BR_INITIAL_HEIGHT);
      }
      return new Dimension(TWGTaskFrame.INITIAL_WIDTH, TWGTaskFrame.INITIAL_HEIGHT);
   }

   private void createMenus()
   {
      mb = new JMenuBar();

      this.setJMenuBar(mb);

      // create file menu
      file = new JMenu(getNlsString("mFile"));

      fRefresh = new JMenuItem(getNlsString("miRefreshTree"));
      fRefresh.setActionCommand("refresh");
      fRefresh.addActionListener(this);
      file.add(fRefresh);

      fExit = new JMenuItem(getNlsString("miExit"));
      fExit.setActionCommand("exit");
      fExit.addActionListener(this);

      file.add(fExit);
      mb.add(file);

      //create help menu
      help = new JMenu(getNlsString("mHelp"));
      hContents = new JMenuItem(getNlsString("TWGMSCSBrowseFrameTitle"));

      // Add help action to hContents menuitem.  Specify the topic id
      this.getHelpContext().addHelp( hContents, "TWGMSCS.Browser");

      hIndex = new JMenuItem(getNlsString("TWGHelpIndex"));
      this.getHelpContext().addHelp( hIndex, "Director.index");

      help.add(hContents);
      help.add(hIndex);

      mb.add(help);

   }//end method

   public void actionPerformed( ActionEvent evt )
   {
     // Place menu and/or button handling code here.
     // Remove listener interface if neither of these are required.
     //         ....................
     String                   cmd = evt.getActionCommand();

     if (cmd.equals("refresh")) {
         if ( Browser != null ) {
             Browser.populateClustersTree(false);
         }

     } else if (cmd.equals("exit")) {
        if ( Browser != null ) {
           Browser.setStatus( TWGMSCSConstants.CLOSE );
           pCleanUp();
        }
     }
   } // end actionPerformed

   public void propertyChange( PropertyChangeEvent pevt )
   {
      if ( pevt.getPropertyName() == TWGMSCSConstants.STATUS_CHANGE )
      {

         if (Browser != null)
         {
            setStatusText(Browser.getStatusText());
         }
         switch (((Integer)pevt.getNewValue()).intValue())
         {
            case TWGMSCSConstants.IN_PROGRESS:
               startStatusIndicator();
               //hourglass off
               setBusy(false);
               break;

            case TWGMSCSConstants.BUSY:
               startStatusIndicator();
               //hourglass on
               setBusy(true);
               break;
            case TWGMSCSConstants.CLOSE:
               stopStatusIndicator();
               break;
            default:  //READY AND UNKNOWN
               stopStatusIndicator();
               //hourglass off
               setBusy(false);
               break;
         }
      }

      return;
   }

   public TWGMSCSBrowser(){
       super(true);
   }

   public Container buildView()
   {
      return Panel;
   }

   private boolean BuilditNow()
   {
      boolean RC = true;

      TWGTaskActivator ta = getTaskActivator();
      Panel = BuildBrowser(ta);

      if(Panel == null)
      {
         RC = false;
      }
      else
      {
         setStatusText(getNlsString("Ready"));
      }
      return RC;

   }


   private Container BuildBrowser(TWGTaskActivator ta)
   {
      this.setIconImage(ta.getTask().getImage( TWGTaskConstants.INDEX_ICON_SMALL ).getImage());
      this.setTitle( getNlsString( "TWGMSCSBrowseFrameTitle" ));

      createMenus(); //for the frame

      LongValueSet targetSet = ta.getMoid();
      Browser = new TWGMSCSClassBrowser(ta, targetSet);
      Browser.addPropertyChangeListener(this);
      return Browser;

   }

}

