package com.tivoli.twg.mscs;

import java.io.*;
import java.net.*;
import java.lang.*;
import java.util.*;
import java.beans.*;
import java.text.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;

import com.tivoli.twg.engine.*;
import com.tivoli.twg.libs.*;
import com.tivoli.twg.libs.TWGConVersion;
import com.tivoli.twg.mscs.*;

/** 
 * Object to represent a network for the cluster.
 */
public class TWGMSCSClusterNetworkObject extends TWGMSCSClusterObject
                                         implements ActionListener
                           {
    String  role;
    String  mask;
    private JMenuItem     fPopupRename = null;

    public TWGMSCSClusterNetworkObject(String nname, String nstate, String ndesc, String nrole, String nmask) {
        super(nname,nstate,ndesc);
        role=nrole;
        mask=nmask;
    }

    public void setRole(String nrole) {
        role=nrole;
    }

    public String getRole() {
        return role;
    }

    public void setMask(String nmask) {
        mask=nmask;
    }

    public String getMask() {
        return mask;
    }

    public int      getColumns() {
        return 5;
    }

    public String   getColumnHeader(int column) {
        switch (column) {
            case 0:
                return getNlsString("colnameName");
            case 1:
                return getNlsString("colnameState");
            case 2:
                return getNlsString("colnameRole");
            case 3:
                return getNlsString("colnameMask");
            case 4:
                return getNlsString("colnameDesc");
            default:
                return getNlsString("stateUnknown");
        }
    }

    public String   getColumnString(int column) {
        switch (column) {
            case 0:
                if (getName()!=null) {
                    return getName();
                }
                break;
            case 1:
                if (getState()!=null) {
                    return getState();
                }
                break;
            case 2:
                if (getRole()!=null) {
                    return getRole();
                }
                break;
            case 3:
                if (getMask()!=null) {
                    return getMask();
                }
                break;
            case 4:
                if (getDescription()!=null) {
                    return getDescription();
                } else {
                    return new String("");
                }
            default:
                break;
        }
        return getNlsString("stateUnknown");
    }

    public int      getPopupItemCount() {
        if (super.getPopupItemCount()>0) {
            return super.getPopupItemCount()+2;
        } else {
            return 1;
        }
    }

    public JMenuItem getPopupItem(int index) {
        switch (index) {
            case 0:
                if (fPopupRename==null) {
                    fPopupRename=new JMenuItem(getNlsString("miRename"));
                    fPopupRename.setActionCommand("rename");
                    fPopupRename.addActionListener(this);
                    if ((TWGConVersion.getServerVersion()/100000)<3000 || !(TWGConUser.currentHasPrivilege(TWGMSCSConstants.MODIFY_CLUSTER_ACCESS_TOKEN))) {
                        fPopupRename.setEnabled(false);
                    }
                }
                return fPopupRename;
            case 1:
                // return null to have the adder add a separator instead
                return null;
            default:
                break;
        }
        if (index<super.getPopupItemCount()+2) {
            return super.getPopupItem(index-2);
        } else {
            // don't know what to add, so add a separator, I guess
            return null;
        }
    }

    public void actionPerformed( ActionEvent evt )
    {
      // Place menu and/or button handling code here.
      // Remove listener interface if neither of these are required.
      //         ....................
      String                   cmd = evt.getActionCommand();
      TWGMSCSClusterObjectWithServiceNode  owsn = null;
      ServiceNode                          sn = null;
      MSCSAgentCommand                     agtcmd = null;

      owsn=getParentWithServiceNode();
      if (owsn!=null) {
          sn=owsn.getServiceNode();
      }

      if (cmd.equals("rename")) {
          // rename the network object
          String newname;

          newname=JOptionPane.showInputDialog(null,
					      getNlsString("renameNewName"),
                                              getNlsString("renameTitle"),
                                              JOptionPane.QUESTION_MESSAGE);
	  if (newname != null) {
	      if (!isNameValid(newname)) {
		  Object[] badname = {getName(), newname};
		  JOptionPane.showMessageDialog(null,
						MessageFormat.format(getNlsString("errNameBad"),badname),
						getNlsString("TWGMSCSBrowseFrameTitle"),
						JOptionPane.ERROR_MESSAGE);
	      } else {
		  Object[] nameargs = {getName(), newname};
		  // verify that they Really want to rename the network object
		  if (JOptionPane.showConfirmDialog(null,
						    MessageFormat.format(getNlsString("areYouSureRename"),nameargs),
						    getNlsString("TWGMSCSBrowseFrameTitle"),
						    JOptionPane.YES_NO_OPTION,
						    JOptionPane.QUESTION_MESSAGE) ==
                      JOptionPane.YES_OPTION) {
		      // they said yes, do the rename
		      agtcmd = new MSCSAgentCommand(MSCSAgentCommand.MSCSAGT_SET_NETWORK_NAME, owsn.GetMyDestination());
		      agtcmd.addCompUnicodeInputParam(getName());
		      agtcmd.addCompUnicodeInputParam(newname);
		      if (sn != null) {
			  // Send command to server
			  try{
			      sn.SendAsynchCommand(agtcmd);
			  }
			  catch(InterruptedException x){
			      return;
			  } 
			  catch(ServiceNodeClosedException x){
			      return;
			  }
			  catch(ServiceNodeException x){
			      return;
			  }
		      }
		  }
	      }
	  }
      } else {
          super.actionPerformed(evt);
      }
    } // end actionPerformed
}

