/******************************************************************************
 * Licensed Materials - Property of IBM                                       *
 *                                                                            *
 * (C) Copyright by IBM 2003.  All Rights Reserved.                           *
 *****************************************************************************/

package com.ibm.sysmgt.sdk.tools;

// this will need to be fine-tuned
import com.tivoli.twg.engine.InvalidObjectIDException;
import com.tivoli.twg.engine.TWGByteArrayInputStream;
import com.tivoli.twg.engine.TWGByteArrayOutputStream;
import com.tivoli.twg.engine.TWGManagedObject;
import com.tivoli.twg.engine.TWGManagedObjectConstants;
import com.tivoli.twg.engine.TWGManagedObjectFingerprint;
import com.tivoli.twg.engine.TWGObjectDestroyException;
import com.tivoli.twg.engine.TWGPersistentObjectDictionary;
import com.tivoli.twg.engine.TWGPersistentObjectRestoreException;
import com.tivoli.twg.engine.TWGPersistentObjectSaveException;
import com.tivoli.twg.libs.TWGByte;
import com.tivoli.twg.libs.TWGDate;
import com.tivoli.twg.libs.TWGIntegerArray;
import com.tivoli.twg.libs.TWGLong;

import com.tivoli.twg.libs.DataValue;
import com.tivoli.twg.libs.IntelByteBuffer;
import com.tivoli.twg.libs.MultiLocaleBundle;
import com.tivoli.twg.libs.QueueBatch;
import com.tivoli.twg.libs.TWGByteArray;
import com.tivoli.twg.libs.TWGInteger;
import com.tivoli.twg.libs.TWGString;
import com.tivoli.twg.libs.TWGStringArray;
import com.tivoli.twg.log.TWGRas;

import java.net.InetAddress;
import java.net.UnknownHostException;
import java.security.AccessController;
import java.sql.*;
import java.util.Collections;
import java.util.Enumeration;
import java.util.HashMap;
import java.util.Locale;
import java.util.MissingResourceException;
import java.util.Vector;

import java.io.NotSerializableException;
import java.io.UTFDataFormatException;


/**
 * This class extends the TWGManagedObject class to create a managed
 * object for systems that are defined by users
 *
 * @see com.tivoli.twg.engine.TWGManagedObject
 */

public class UserDefinedManagedSystem extends TWGManagedObject
{
  /** Our classname */
  public static final String CLASSNAME =
    "com.ibm.sysmgt.sdk.tools.UserDefinedManagedSystem";

  private MultiLocaleBundle mlb = null;

  /** Version numbers for saving and restoring */
  private static final int VERSION_THREE = 3;
  private static final int VERSION_TWO   = 2;
  private static final int VERSION_ONE   = 1;

  /** List of supported attribute IDs */
  private Vector attrib_Ids;

  /** List of attribute types */
  private Vector attrib_Types;

  /** List of attribute labels */
  private Vector attrib_Labels;

  /** List of attribute values */
  private Vector attrib_Values;

  /* The user object type id */
  private static String attrib_id = "USER_DEFINED_SYSTEM_SUBTYPE";

  /* Name of our Resource Bundle class                               */
  private static final String OURRESOURCEBUNDLE = "com.ibm.sysmgt.sdk.tools.SDKCLIResourceBundle";

  ///* User defined attribute type */
  //private int attrib_type = DataValue.STRING_ARRAY_TYPE;
  //
  ///* User defined attribute label */
  //private String attrib_label;
  //
  ///* User defined attribute description */
  //private String attrib_description;


  /** List of user defined objects that we've already created */
  protected static HashMap userDefinedObjectList = new HashMap();


  private long userDefinedObjectID;


  /**
   * Default constructor for user defined object managed object
   */
  public UserDefinedManagedSystem()
  {
    super();
    System.out.println("In restore constructor for user defined managed objects");
    try
    {
      System.out.println("Trying to get resource bundle for UserDefinedManagedSystem");
      mlb = MultiLocaleBundle.getSharedBundle(OURRESOURCEBUNDLE);
    }
    catch (MissingResourceException mrx)
    {
      System.out.println("MissingResourceException trying to get bundle for UserDefinedManagedSystem");
      mlb = null;
    }
    // set up our attribute vectors
    attrib_Ids = new Vector();
    attrib_Types = new Vector();
    attrib_Labels = new Vector();
    attrib_Values = new Vector();

    // set our user defined attributes to null;
    //attrib_label = null;
    //attrib description = null;

    // Allow this object to be deleted from the console
    setConsoleDeletable(true);
    // Allow this object to be renamed from the console
    setConsoleRenamable(true);

    TWGRas.entryExit(TWGRas.SDK,
                     "UserDefinedManagedSystem constructor: object name = "
                     + getName());
  }

  /**
   * Default constructor for user defined object managed object
   */
  public UserDefinedManagedSystem(String name)
  {
    super();
    System.out.println("In main constructor for user defined managed objects");
    try
    {
      System.out.println("Trying to get resource bundle for UserDefinedManagedSystem");
      mlb = MultiLocaleBundle.getSharedBundle(OURRESOURCEBUNDLE);
    }
    catch (MissingResourceException mrx)
    {
      System.out.println("MissingResourceException trying to get bundle for UserDefinedManagedSystem");
      mlb = null;
    }
    setName(name);

    // set up our attribute vectors
    attrib_Ids = new Vector();
    attrib_Types = new Vector();
    attrib_Labels = new Vector();
    attrib_Values = new Vector();

    attrib_Ids.add(new TWGString(attrib_id));
    attrib_Types.add(new TWGInteger(DataValue.STRING_TYPE));
    //attrib_Labels.add(new TWGString("User defined object"));
    attrib_Labels.add(new TWGString(getString("INFO.USER_DEFINED_ATTR_LABEL_1")));
    //attrib_Values.add((DataValue) new TWGString("Default definition"));
    attrib_Values.add((DataValue) new TWGString(getString("INFO.USER_DEFINED_ATTR_DESC_1")));

    // keep a local copy of our object ID so we don't have
    // to try and catch every time we want to use it.
    // debug statements
    try
    {
      userDefinedObjectID = this.ObjectID();
    }
    catch (InvalidObjectIDException ioie)
    {
      TWGRas.error(TWGRas.SDK,
                   "UserDefinedManagedSystem constructor: invalid object ID");
    }

    // Allow this object to be deleted from the console
    setConsoleDeletable(true);
    // Allow this object to be renamed from the console
    setConsoleRenamable(true);

    // save the data in the persistent store
    try
    {
      TWGRas.entryExit(TWGRas.SDK,
                       "UserDefinedManagedSystem constructor: object name = "
                       + getName());
      save();
    }
    catch (TWGPersistentObjectSaveException pose)
    {
      TWGRas.debug(TWGRas.SDK,
                   "UserDefinedManagedSystem constructor: TWGPersistentObjectSaveException, object name = "
                   + getName());
    }

    // add this user defined object to the list of user defined objects we've created
    TWGRas.debug(TWGRas.SDK,
                 "UserDefinedManagedSystem constructor:  adding user defined object to the server list: "
                 + this.getName() + ", oid: " + userDefinedObjectID);
    userDefinedObjectList.put(name, this);
  }

  /**
   * Copy instance data to another managed object.
   * @param managedObject target managed object.
   */
  protected void copyTo(TWGManagedObject managedObject)
  {
      super.copyTo(managedObject);
      if (managedObject instanceof UserDefinedManagedSystem)
      {
          UserDefinedManagedSystem userDefinedObject =
              (UserDefinedManagedSystem) managedObject;
          //userDefinedObject.attrib_Ids = new Vector(this.attrib_Ids.size());
          //this.attrib_Ids.copyInto(userDefinedObject.attrib_Ids);
          userDefinedObject.attrib_Ids = (Vector) this.attrib_Ids.clone();
          //userDefinedObject.attrib_Types = new Vector(this.attrib_Types.size());
          //this.attrib_Types.copyInto(userDefinedObject.attrib_Types);
          userDefinedObject.attrib_Types = (Vector) this.attrib_Types.clone();
          //userDefinedObject.attrib_Labels = new Vector(this.attrib_Labels.size());
          //this.attrib_Labels.copyInto(userDefinedObject.attrib_Labels);
          userDefinedObject.attrib_Labels = (Vector) this.attrib_Labels.clone();
          //userDefinedObject.attrib_Values = new Vector(this.attrib_Values.size());
          //this.attrib_Values.copyInto(userDefinedObject.attrib_Values);
          userDefinedObject.attrib_Values = (Vector) this.attrib_Values.clone();
      }
  }

  /**
   * Destructor for UserDefinedManagedSystem: remove object from lists
   */
  public void Delete()
  {
    userDefinedObjectList.remove(getName());
    super.Delete();
  }

  /**
   * Destroy for UserDefinedManagedSystem
   * @exception TWGObjectDestroyException if error during destroy
   */
  public void Destroy() throws TWGObjectDestroyException
  {
    super.Destroy();
  }

  /**
   * Method for saving persistent object data.
   * @param dictionary used to collect persistent object.
   * @exception TWGPersistentObjectSaveException
   *                   thrown to abort save procedure.
   */
  protected void saveData(TWGPersistentObjectDictionary dictionary)
    throws TWGPersistentObjectSaveException
  {
     int i;
     TWGRas.entryExit(TWGRas.SDK,
                      "UserDefinedManagedSystem.saveData(): entry, name = "
                      + getName());
     super.saveData(dictionary);

     TWGByteArrayOutputStream buffer = new TWGByteArrayOutputStream();

     // Save the version, host name, and IP address to
     // persistent storage.  We'll use this information
     // to rebuild our list of user defined objects
     // whenever Director is restarted.
     buffer.writeInt(VERSION_ONE);
     TWGRas.debug(TWGRas.SDK,
                  "UserDefinedManagedSystem.saveData(): version = " +
                  VERSION_ONE);
     buffer.writeLong(userDefinedObjectID);
     TWGRas.debug(TWGRas.SDK,
                  "UserDefinedManagedSystem.saveData(): oid = " +
                  userDefinedObjectID);
     //buffer.writeLong(attrib_Ids.length);
     buffer.writeInt(attrib_Ids.size());
     TWGRas.debug(TWGRas.SDK,
                  "UserDefinedManagedSystem.saveData(): attrib_Ids.size() = " +
                  attrib_Ids.size());

     // write out the attribute information.
     // to save code, we're writing out all of the
     // attribute info together, instead of writing
     // out each attribute array separately.
     for (i = 0; i < attrib_Ids.size(); i++)
     {
       TWGRas.debug(TWGRas.SDK,
                    "UserDefinedManagedSystem.saveData(): = writing attrib_Id: "
                      + ((TWGString) attrib_Ids.get(i)).getValue());
       buffer.writeUTF(((TWGString) attrib_Ids.get(i)).getValue());
       TWGRas.debug(TWGRas.SDK,
                    "UserDefinedManagedSystem.saveData(): = writing attrib_Type: "
                      + ((TWGInteger) attrib_Types.get(i)).getValue());
       buffer.writeInt(((TWGInteger) attrib_Types.get(i)).getValue());
       TWGRas.debug(TWGRas.SDK,
                    "UserDefinedManagedSystem.saveData(): = writing attrib_Labels: "
                      + ((TWGString) attrib_Labels.get(i)).getValue());
       buffer.writeUTF(((TWGString) attrib_Labels.get(i)).getValue());
       TWGRas.debug(TWGRas.SDK,
                    "UserDefinedManagedSystem.saveData(): = writing attrib_Values: "
                      + ((TWGString) attrib_Values.get(i)).getValue());
       buffer.writeUTF(((TWGString) attrib_Values.get(i)).getValue());
       //int len = attrib_Values.get(i).writeDataValueLength();
       //byte buf[] = new byte[len];
       //attrib_Values[i].writeDataValue(buf, 0);
       //buffer.writeUTF(new String(buf));
     }

     TWGRas.debug(TWGRas.SDK,
                  "UserDefinedManagedSystem.saveData(): byte buffer = " +
                  buffer.toString());

     /* Add record to dictionary */
     dictionary.put(CLASSNAME, buffer.toByteArray());
  }


  /**
   * Method for restoring persistent object data.
   *
   * @param dictionary used to restore persistent object.
   * @param resolveObjectReferences
   *                   used to indicate if object references should be
   *                   resolved during restore.
   *
   * @exception TWGPersistentObjectRestoreException
   *                   thrown to abort restore procedure.
   */
  protected void restoreData(TWGPersistentObjectDictionary dictionary,
                             boolean resolveObjectReferences)
      throws TWGPersistentObjectRestoreException
  {
     TWGRas.entryExit(TWGRas.SDK, "UserDefinedManagedSystem.restoreData(): entry, resolveObjectReferences = " +
                      resolveObjectReferences);

     /* Restore base class */
     super.restoreData(dictionary, resolveObjectReferences);

     if (resolveObjectReferences == false)
     {
        TWGRas.debug(TWGRas.SDK, "UserDefinedManagedSystem.restoreData(): getting buffer ");
        TWGByteArrayInputStream buffer = new TWGByteArrayInputStream(dictionary.get(CLASSNAME));
        if (buffer == null)
          throw new TWGPersistentObjectRestoreException("no data");

        // Get the version number so we know what other data is stored.
        // If the data that is saved to persistent storage is ever
        // changed, for example, if an additional variable is stored,
        // then the version number must change.
        int version = buffer.readInt();
        TWGRas.debug(TWGRas.SDK, "UserDefinedManagedSystem.restoreData(): version = " + version);
        switch (version)
        {
          case VERSION_ONE:
          {
            userDefinedObjectID = buffer.readLong();
            TWGRas.debug(TWGRas.SDK, "UserDefinedManagedSystem.restoreData(): oid = " + userDefinedObjectID);
            int length = buffer.readInt();
            TWGRas.debug(TWGRas.SDK, "UserDefinedManagedSystem.restoreData(): length = " + length);

            // retrieve the attribute information.
            // to save code, we're writing out all of the
            // attribute info together, instead of writing
            // out each attribute array separately.
            for (int i = 0; i < length; i++)
            {
              try
              {
                // get the attribute id
                TWGString tempString = new TWGString(buffer.readUTF());
                attrib_Ids.add(i, tempString);
                TWGRas.debug(TWGRas.SDK, "UserDefinedManagedSystem.restoreData(): adding attrib_Id " +
                             tempString);
                // get the attribute type
                attrib_Types.add(i, new TWGInteger(buffer.readInt()));
                attrib_Labels.add(i, new TWGString(buffer.readUTF()));

                switch (((TWGInteger) attrib_Types.get(i)).getValue())
                {
                  case DataValue.STRING_ARRAY_TYPE:
                    TWGStringArray tempStringArray = new TWGStringArray();
                    tempStringArray.readDataValue(buffer.readUTF().getBytes(), 0);
                    attrib_Values.add(i, (DataValue) tempStringArray);
                    break;
                  case DataValue.BYTE_ARRAY_TYPE:
                    TWGByteArray tempByteArray = new TWGByteArray();
                    tempByteArray.readDataValue(buffer.readUTF().getBytes(), 0);
                    attrib_Values.add(i, tempByteArray);
                    break;
                  case DataValue.BYTE_TYPE:
                    attrib_Values.add(i, new TWGByte((byte) buffer.readUTF().charAt(0)));
                    break;
                  case DataValue.DATE_TYPE:
                    attrib_Values.add(i, new TWGDate(new Date(buffer.readLong())));
                    break;
                  case DataValue.LONG_TYPE:
                    attrib_Values.add(i, new TWGLong(buffer.readLong()));
                    break;
                  case DataValue.INTEGER_ARRAY_TYPE:
                    attrib_Values.add(i, new TWGIntegerArray(buffer.readIntArray()));
                    break;
                  case DataValue.INTEGER_TYPE:
                    attrib_Values.add(i, new TWGInteger(buffer.readInt()));
                    break;
                  case DataValue.STRING_TYPE:
                  default:
                    TWGString temp = new TWGString(buffer.readUTF());
                    attrib_Values.add(i, temp);
                    TWGRas.debug(TWGRas.SDK, "UserDefinedManagedSystem.restoreData(): adding attrib_Value " +
                                 temp);
                    break;
                 }
               }
               catch (UTFDataFormatException exception)
               {
                 throw new TWGPersistentObjectRestoreException();
               }

            } // end for
            TWGRas.debug(TWGRas.SDK,
                         "UserDefinedManagedSystem.restore(): " + getName());

            break;
          } // end case
          case VERSION_TWO:
          case VERSION_THREE:
          default:
          {
            break;
          }
        }
        setState(TWGManagedObjectConstants.MOSTATE_UNKNOWN);
     }
     else
     {
        TWGRas.error(TWGRas.SDK,
                     "UserDefinedManagedSystem.restoreData(): adding restored object to user defined object list: "
                     + getName());
        userDefinedObjectList.put(getName(), this);
     }
  }

  /**
   * Get a user defined object with a given address.
   * @param address of the user defined object to find
   * @return UserDefinedManagedSystem the user defined object with the specified address
   */
  protected static UserDefinedManagedSystem getUserDefinedObject(String name)
  {
    TWGRas.entryExit(TWGRas.SDK,
                     "UserDefinedManagedSystem.getUserDefinedObject(): entry, name = "
                     + name);

    return (UserDefinedManagedSystem) userDefinedObjectList.get((Object) name);
  }

  /**
   * Get a list of all the user defined objects.
   * @param address of the user defined object to find
   * @return UserDefinedManagedSystem the user defined object with the specified address
   */
  protected static Enumeration getUserDefinedObjectList()
  {
    TWGRas.entryExit(TWGRas.SDK,
                     "UserDefinedManagedSystem.getUserDefinedObjectList(): entry");

    return (Collections.enumeration(userDefinedObjectList.values()));
  }

//  /**
//   * Get the IP address for this object
//   * @return address the IP address in the form of a String
//   */
//  protected String getAddress()
//  {
//    TWGRas.entryExit(TWGRas.SDK,
//                     "UserDefinedManagedSystem.getAddress(): entry");
//    ??
//    return userDefinedObjectID;
//  }


///**
// * Start ping sequence for user defined object managed objects
// * @return boolean TRUE: Ping function started
// */
//public boolean startPing()
//{
//   TWGRas.entryExit(TWGRas.SDK,
//                    "UserDefinedManagedSystem.startPing(): entry, "
//                    + getName() + ", " + getAddress());
//
//   UserDefinedPingRequest qe = new UserDefinedPingRequest(this);
//   queue.enqueue(qe);
//   return true;
//}

  /**
   * Start ping sequence for user defined object managed objects
   * @param mo The managed object to ping
   * @return boolean TRUE: Ping function started
   */
  public static boolean startPing(TWGManagedObject mo)
  {
     TWGRas.entryExit(TWGRas.SDK,
                      "UserDefinedManagedSystem.startPing(): entry with one param, "
                      + mo.getName() + ", " + mo.getAddressEntryRecord());

     return TWGManagedObject.startPing(mo);
  }

  /**
   * Start ping sequence for user defined object managed objects
   * @return boolean TRUE: Ping function started
   */
  public static boolean startPing(TWGManagedObject mo, boolean reauth)
  {
     TWGRas.entryExit(TWGRas.SDK,
                      "UserDefinedManagedSystem.startPing(): entry with two params, "
                      + mo.getName() + ", " + mo.getAddressEntryRecord());
     return TWGManagedObject.startPing(mo, reauth);
  }

  /**
   * Default ping with reauthenticate method for SNMP device.
   * If not overridden, just calls SNMP device's startPing method
   * which may have been overridden.
   *
   * @return TRUE: Ping function supported. FALSE: Function not supported
   *         for this instance.
   */
   public boolean startPingWithReauthenticate()
   {
     TWGRas.entryExit(TWGRas.SDK,
                      "UserDefinedManagedSystem.startPingWithReauthenticate(), "
                      + getName() + ", " + getAddressEntryRecord());

     return startPing();
   }


   /**
    * Attribute ID enumeration : this method is expected to return a
    * List of strings containing the String names of the attributes
    * supported for a given object.  Subclasses of base classes implementing
    * the interface should report additional attributes by adding them
    * to the list returned by the base class.
    *
    * @returns list of attribute ID strings
    */
   public String[] getAttributeIDList()
   {
      TWGRas.entryExit(TWGRas.SDK,
                       "UserDefinedManagedSystem.getAttributeIDList(), name = " + getName());
      String[] s = super.getAttributeIDList();
      String[] attrIdList = new String[attrib_Ids.size()];
      TWGRas.entryExit(TWGRas.SDK,
                       "UserDefinedManagedSystem.getAttributeIDList(), attrib size = " +
                       attrib_Ids.size());
      for (int i = 0; i < attrib_Ids.size(); i++)
      {
        TWGString tempString = (TWGString) attrib_Ids.get(i);
        attrIdList[i] = tempString.getValue();
      }
      String[] new_s = new String[s.length + attrib_Ids.size()];
      System.arraycopy(s, 0, new_s, 0, s.length);
      //System.arraycopy(attrib_Ids, 0, new_s, s.length, attrib_Ids.length);
      System.arraycopy(attrIdList, 0, new_s, s.length, attrIdList.length);
      return new_s;
   }

   /**
    * User defined attribute ID enumeration : this method returns a
    * list of strings containing the names of the attribute IDs
    * supported for a given user defined managed system.
    *
    * @returns list of user defined attribute ID strings
    */
   public String[] getUserDefinedAttributeIDList()
   {
      TWGRas.entryExit(TWGRas.SDK,
                       "UserDefinedManagedSystem.getUserDefinedAttributeIDList(), name = " + getName());
      String[] attrIdList = new String[attrib_Ids.size()];
      for (int i = 0; i < attrib_Ids.size(); i++)
      {
        TWGString tempString = (TWGString) attrib_Ids.get(i);
        attrIdList[i] = tempString.getValue();
      }
      return attrIdList;
   }

    /**
     * Get value of attribute with given ID
     * @param id - attribute ID requested
     * @param loc - locale to use for value, if applicable
     * @return value object, or null if no value
     */
    public DataValue getAttributeValue(String id, Locale loc)
    {
      TWGRas.entryExit(TWGRas.SDK,
                     "UserDefinedManagedSystem.getAttributeValue(): id = " + id +
                      ", name = " + getName());

      //for (int i = 0; i < attrib_Ids.length; i++)
      for (int i = 0; i < attrib_Ids.size(); i++)
      {
        if(id.equals(((TWGString) attrib_Ids.get(i)).getValue()))
        {
          TWGRas.debug(TWGRas.SDK,
                       "UserDefinedManagedSystem.getAttributeValue(): returning attribute: "
                       + attrib_Labels.get(i));
          return (TWGString) attrib_Values.get(i);
          //int type = attrib_Types[i];
          //switch (type)
          //{
          //  // These seem to be the data types that are used by the attributes
          //  case DataValue.STRING_ARRAY_TYPE:
          //    TWGStringArray twgStringArray = new TWGStringArray();
          //    twgStringArray.readDataValue(
          //    return new TWGStringArray(attrib_Values[i]);
          //  case DataValue.STRING_TYPE:
          //    return new TWGString((TWGString) attrib_Values[i]);
          //  case DataValue.BYTE_ARRAY_TYPE:
          //    return new TWGByteArray((TWGByteArray) attrib_Values[i]);
          //  case DataValue.BYTE_TYPE:
          //    return new TWGByte((TWGByte) attrib_Values[i]);
          //  case DataValue.DATE_TYPE:
          //    return new TWGDate((TWGDate) attrib_Values[i]);
          //  case DataValue.LONG_TYPE:
          //    return new TWGLong((TWGLong) attrib_Values[i]);
          //  case DataValue.INTEGER_ARRAY_TYPE:
          //    return new TWGIntegerArray((TWGIntegerArray) attrib_Values[i]);
          //  case DataValue.INTEGER_TYPE:
          //    return new TWGInteger((TWGInteger) attrib_Values[i]);
          //}
        }
      }
      // we shouldn't be here unless we couldn't find the attribute id in our list.
      // get the attribute value from our parent class.
      TWGRas.debug(TWGRas.SDK,
                   "UserDefinedManagedSystem.getAttributeValue(): getting attribute from parent class, id = " + id);
      return super.getAttributeValue(id, loc);
    }

    /**
     * Set value of attribute with given ID
     * @param id - attribute ID to be set
     * @param val - attribute value to be set
     * @return true if set successful, false if not successful
     */
    public boolean setAttributeValue(String id, DataValue val)
    {
      //for (int i = 0; i < attrib_Ids.length; i++)
      for (int i = 0; i < attrib_Ids.size(); i++)
      {
        if (id.equals(((TWGString) attrib_Ids.get(i)).getValue()))
        {
          // remove the old value
          attrib_Values.remove(i);
          // add the new value
          attrib_Values.set(i, val);
          return true;
        }
      }
      return super.setAttributeValue(id, val);
    }

    /**
     * Get type of attribute value for given attribute
     * @param id - attribute ID
     * @return DataValue type of attribute, or DataValue.NONE_TYPE if
     * undefined
     */
    public int getAttributeType(String id)
    {
      for (int i = 0; i < attrib_Types.size(); i++)
      {
        if (id.equals(((TWGString) attrib_Ids.get(i)).getValue()))
        {
          return ((TWGInteger) attrib_Types.get(i)).getValue();
        }
      }
      return super.getAttributeType(id);
    }

    /**
     * Get description string for given attribute in given locale
     * @param id - attribute ID
     * @param loc - locale to use for description
     * @return description string, or null if not available
     */
    public String getAttributeDescription(String id, Locale loc)
    {
      for (int i = 0; i < attrib_Types.size(); i++)
      {
        if (id.equals(((TWGString) attrib_Ids.get(i)).getValue()))
        {
          return ((TWGString) attrib_Labels.get(i)).getValue();
        }
      }
      return super.getAttributeDescription(id, loc);
    }

    /**
     * Add a new attribute
     * @param id - attribute ID
     * @param label - text description for the attribute
     * @param value - the value of the attribute
     * @param loc - locale to use for description
     * @return true - the attribute was added successfully
     */
    public boolean addAttribute(String id, String label, DataValue value, Locale locale)
    {
      // verify we don't already have one of these defined.
      //for (int i = 0; i < attrib_Types.length; i++)
      for (int i = 0; i < attrib_Ids.size(); i++)
      {
        if (id.equals(((TWGString) attrib_Ids.get(i)).getValue()))
        {
          // we've already got one of these defined.
          return false;
        }
      }

      // add the new attribute to the end of the array
      attrib_Ids.add(new TWGString(id));
      attrib_Types.add(new TWGInteger(DataValue.STRING_TYPE));
      attrib_Labels.add(new TWGString(label));
      attrib_Values.add(value);
      return true;
    }

    /**
     * Add a new attribute
     * @param id - attribute ID
     * @param label - text description for the attribute
     * @param value - the value of the attribute
     * @param loc - locale to use for description
     * @return true - the attribute was added successfully
     */
    public boolean removeAttribute(String id, Locale loc)
    {
      // find the attribute we want to remove
      for (int i = 0; i < attrib_Types.size(); i++)
      {
        if (id.toUpperCase().equals(((TWGString) attrib_Ids.get(i)).getValue().toUpperCase()))
        {
          attrib_Ids.remove(i);
          attrib_Types.remove(i);
          attrib_Labels.remove(i);
          attrib_Values.remove(i);
          return true;
        }
      }
      return false;
    }

  public String getString(String id)
  {
    String name = id;
    if (mlb != null)
    {
      name = mlb.getString(Locale.getDefault(), id, id);
    }
    return name;
  }
}
