/******************************************************************************
 * Licensed Materials - Property of IBM                            *
 *                                                                            *
 * PID (5697-TKT)                                                             *
 *                                                                            *
 * (C) Copyright by IBM 1999. All Rights Reserved. *
 *****************************************************************************/
package com.BobCo.EventHandler ;

import com.tivoli.twg.alertmgr.* ;

import com.tivoli.twg.alertmgr.TWGEvent;
import com.tivoli.twg.alertmgr.TWGActionInfo;
import com.tivoli.twg.alertmgr.TWGEventHandler;
import com.tivoli.twg.log.*;
import com.tivoli.twg.libs.*;
import com.tivoli.twg.snmp.*;
import com.tivoli.twg.snmputils.*;
import java.util.*;
import java.net.*;

import java.io.* ;


// Class for new Action handler BobCoEventHandler
public class WriteEventToFileEventHandler implements TWGEventHandler 
{

	private static EContainer theContainer ; 
	private final String BUNDLE_NAME = "com.BobCo.EventHandler.WriteEventToFileBundle" ;
	private final int FILE_NAME_ID = 0 ;
	private final int EVENT_DATA_FORMAT_ID = 1 ;
	private final int MAX_EVENTS_ID = 2 ;
	private final int ACTION_ID = 3 ;
	
	public static final int APPEND = 0 ;
	public static final int RESET  = 1 ;
	
	public WriteEventToFileEventHandler() 
	{
		theContainer = new EContainer() ;
	}

	// returns information about the action handler, its customizable parameters etc
	public boolean GetInfo(TWGActionInfo info) 
	{
		String[] choices = {"RESET_FILE","APPEND_EVENT_RECORDS"};
		int[] defaultChoice = { 1,2 } ;


		info.setTitle("TITLE", BUNDLE_NAME);
		info.addParm(new TWGActionParmTextBox(FILE_NAME_ID,"FILENAME", BUNDLE_NAME,null, 
					null, "Events.txt"));

		info.addParm(new TWGActionParmTextArea(EVENT_DATA_FORMAT_ID,"EVENT_DATA_FORMAT", BUNDLE_NAME, null, 
		null, "DATE=&Date, TIME=&time, EVENT TYPE=&type, TEXT=&text,MANAGED OBJECT NAME=&system,SEVERITY=&severity, CATEGORY=&category, GROUP NAME=&group,SENDER NAME=&sender " ) ) ;

		info.addParm(new TWGActionParmLongSpinner(MAX_EVENTS_ID,"MAX_EVENTS", BUNDLE_NAME,
					null,null,0,100,1,20));

		info.addParm(new TWGActionParmDropDown(ACTION_ID,"ACTION",BUNDLE_NAME,
					null,null,choices,null,1));

		/*info.addParm(new TWGActionParmMultiSelList(4,"MultSelList 6", BUNDLE_NAME,
					null,null,choices,null));

		info.addParm(new TWGActionParmStringSpinner(5,"String Spinner 4",	BUNDLE_NAME,
					null,null,choices,null,1));
		*/
		info.setPersistent();
		info.setExtendedLogging();
		return true;
	}


	// ProcessEvent is the place where the actual code the handling the event takes place
	public boolean ProcessEvent(TWGEvent alert, TWGActionInfo info) 
	{
		boolean launched = true;

		// get the first parameter of the customized action
		TWGActionParm aprm = info.getParm(FILE_NAME_ID) ;
		String fname = ((TWGActionParmTextBox)aprm).getValue();
		
		// get the second parameter of the customized action
		aprm = info.getParm(EVENT_DATA_FORMAT_ID) ;
		String sampleText = ((TWGActionParmTextArea)aprm).getValue();

        Locale localLanguage = info.getTargetLocale();
        TimeZone tZ = info.getTargetTimeZone();

		// expand the substitution variables
		sampleText = alert.expandString(sampleText) ;
		

		// get the third parameter of the customized action
		aprm = info.getParm(MAX_EVENTS_ID) ;
		long maxEvents = ((TWGActionParmLongSpinner)aprm).getValue();

		// get the fourth parameter of the customized action
		aprm = info.getParm(ACTION_ID) ;
		TWGActionParmDropDown sp1 = (TWGActionParmDropDown)aprm ;
		String parm = sp1.getItem(sp1.getSelected()) ;
		
		int actn = WriteEventToFileEventHandler.APPEND ; // APPEND ;
		if ( parm.equals("RESET_FILE") ) 
			actn = WriteEventToFileEventHandler.RESET ; // RESET ;

		// add the filename to the list if its not present
		// if the filename is already present then retrieve the store object.

		Store st = theContainer.AddStore(fname, maxEvents,actn) ;
		TWGRas.entryExit(TWGRas.EX0,"SampleText::" + sampleText);		
		st.write(sampleText) ;
	
		long key = info.getLogKey() ;
		TWGActionLogManager.actionComplete(new TWGActionEndInfo(key,TWGActionEndInfo.ACTION_SUCCESSFUL)) ;

		return launched;

	}
 
	public TWGMessage ValidateParm(TWGActionParm ap) 
	{
		TWGRas.entryExit(TWGRas.EX0,"BobCoEventHandler::ValidateParm() Entered ");
		String label = ap.getLabel();
		int parmIndex = ap.getIndex();

		InetAddress anAddress = null;

		switch (parmIndex) 
		{
			case FILE_NAME_ID:
				String parm = ((TWGActionParmTextBox)ap).getValue();
				if ( (parm != null) && parm.equals("") ) 
					return new TWGMessage("INVALID_FILENAME", BUNDLE_NAME);	
				else
					return null ;

			case EVENT_DATA_FORMAT_ID:
				parm = ((TWGActionParmTextArea)ap).getValue();
				if ( parm != null && parm.equals("") )
					return new TWGMessage("INVALID_EVENT_DATAFORMAT", BUNDLE_NAME);
				else
					return null ;

			case MAX_EVENTS_ID:
				long maxEvents = ((TWGActionParmLongSpinner)ap).getValue();
				if ( maxEvents == 0 ) 
					return new TWGMessage("VALUE_CANNOT_BE_ZERO", BUNDLE_NAME);					
				else
					return null ;

			case ACTION_ID:
				TWGActionParmDropDown sp1 = (TWGActionParmDropDown)ap ;
				parm = sp1.getItem(sp1.getSelected()) ;
				return null ;

			/*
			case 4:
				TWGActionParmMultiSelList sel = (TWGActionParmMultiSelList)ap ;

				if ( sel.getNumSelectedItems() <= 0 )
					return new TWGMessage("Please Select atleast one Item for MultiSelList", BUNDLE_NAME);

				int[] aSelection = sel.getSelectedItems() ;

				for ( int i = 0 ; i < sel.getNumSelectedItems() ; i++ )
				{
					TWGRas.entryExit(TWGRas.EX0,"MultiSel "+i + " " + sel.getItem(aSelection[i]) ) ;
				}			
			
			case 5:
				TWGActionParmStringSpinner sp = (TWGActionParmStringSpinner)ap ;
				parm = sp.getItem(sp.getSelected()) ;
				if ( (parm != null) && parm.equals("choices4")  ) 
					return null;
				else 
					return new TWGMessage("Please Select choices4", BUNDLE_NAME);
			*/

			default:
				break;
		}
		return null;
	}

	public boolean Init()
	{
		return true;
	}

	public boolean Term()
	{
		theContainer = null ;
		return true ;
	}

	protected void finalize() throws IOException
	{
		theContainer = null ;
	}

} 


/* a Container class for storing the list of files and its associated parameters */
class EContainer
{
	private Vector List ;
	
	public EContainer()
	{
		List = new Vector() ;
	}

	/* creates a new Store object if the filename is not present or returns the store
	   Store object if the filename is already present. */
	
	public Store AddStore(String filename, long Maxcnt, int actn )
	{
		Store st ;
		int num = List.size() ;  
		String fname = null ;
		
		for (int i = 0 ; i < num ; i++ ) 
		{
			st = (Store) List.elementAt(i) ;
			fname = st.getFilename() ;
			if ( fname.equals(filename)  )
			{	
				st.setMaxCount(Maxcnt) ;
				st.setAction(actn) ;
				return st ;
			}
		}

		st = new Store(filename,Maxcnt,actn) ;
		List.addElement(st) ;
		st.Print() ;
		return st ;
	}
	
	protected void finalize() throws IOException
	{
		Store st ;
		int num = List.size() ;  
		for (int i = 0 ; i < num ; i++ ) 
		{
			st = (Store) List.elementAt(i) ;
			st.Close() ;
		}
	}
}

/** class for storing the customized action handler parameters, this object is
    identified by filename which is the key */

class Store extends Object 
{
	private String Filename ;  
	private long    Count ;    
	private java.io.PrintWriter file ;
	private FileOutputStream ostream ;
	private long    MaxCount ;
	private int Action ;

	/*  constructor, does initilization of the members. This object is created when
	    processEvent() gets fired */
	public Store ( String fname, long Maxcnt, int actn)
	{
		Filename = new String( fname ) ;
		MaxCount = Maxcnt ;
		Action = actn ;
		Count = 0 ;

		// open handler for file stream
		try
		{
			ostream = new FileOutputStream(".\\..\\data\\" + Filename);
			file = new PrintWriter(ostream);	
			getFilename() ;
		}
		catch( IOException exp )
		{
			TWGRas.entryExit(TWGRas.EX0,"========== File could not be opened ========== ");					
			TWGRas.entryExit(TWGRas.EX0,"========== " + Filename + " ========== ");					
		}
	}
	
	public String getFilename()  
	{
		return Filename ;
	}
	public void setFilename(String file)  
	{
		Filename = new String( file ) ;
	}
	public long getCount()
	{
		return Count ;
	}
	public void setCount(long cnt)
	{
		Count = cnt;
	}
	public long getMaxCount()
	{
		return MaxCount ;
	}
	public void setMaxCount(long mxcnt)
	{
		MaxCount = mxcnt ;
	}
	public PrintWriter getFileStream()
	{
		return file ;
	}
	public int getAction()
	{
		return Action ;
	}
	public void setAction(int actn)
	{
		Action = actn ;
	}

	public void Print()
	{
		TWGRas.entryExit(TWGRas.EX0,"from Print: filename= " + getFilename() );
		TWGRas.entryExit(TWGRas.EX0,"count= " + getCount() );
		TWGRas.entryExit(TWGRas.EX0,"MaxCount= " + getMaxCount() );
		TWGRas.entryExit(TWGRas.EX0,"State= " + getAction() );
	}
	
	// reset the file by opening the file for write again when the number
	// of records exceed the limit.
	public void Reset()
	{	
		try
		{
			file.flush();	
			ostream.close();
			ostream = new FileOutputStream(".\\..\\data\\" + Filename);
			file = new PrintWriter(ostream);	
			Count = 0 ;
		}
		catch( IOException exp )
		{
			TWGRas.entryExit(TWGRas.EX0,"=== File could not be opened ==== ");					
		}
	}

	public void Close() throws IOException
	{
		file.flush();	
		ostream.close();
	}

	public boolean isActionStatusChanged(int actn)
	{
		if ( Action == actn )
			return false ;
		else
			return true ;
	}

	public boolean equals( Store st )
	{
		if ( Filename.equals(st.getFilename()) )
			return true ;
		else
			return false ;
	}

	
	/*  write the parameter details and event details into the file */
	public boolean write(String sampleText )
	{
		//try
		//{
			if ( Action == WriteEventToFileEventHandler.RESET || Count >= MaxCount )
				Reset() ;

			// printing the data before writing it..
			TWGRas.entryExit(TWGRas.EX0,"####Sample Text : " + sampleText );
			TWGRas.entryExit(TWGRas.EX0,"####Count : " + Count );

			file.println(sampleText) ;
			//file.writeUTF(sampleText) ;
			file.flush();	
			Count++ ;
		//}
		//catch( IOException exp )
		//{
		//	return false ;
		//}
		return true ;
	}
}

