// Director Event Source SDK Sample
// Licensed Materials - Property of IBM
// PID (5697-TKT)
// (C) Copyright by IBM 1999.  All Rights Reserved.

#define DLISTT_INCLUDE_OUTLINES // enables expansion of methods!

/*

 Watcom: do not include stddef.h or
         stdlib.h before tchar.h

*/

#include <stdio.h>
#include <time.h>
#if defined( __WIN32__ )
#include <windows.h>
#include "nfwincom.hpp"
#elif defined( __OS2__ )
#include <wchar.h>
#define INCL_DOS
#define INCL_DOSERRORS
#include <os2.h>
#include "int64.hpp"
typedef ULARGE_INTEGER DWORDLONG;
#elif defined( __NETWARE__ )
#include <tchar.h>
#include <errno.h>
#include <nwtypes.h>
#include <nwenvrn.h>
#include <conio.h>
#include <new.h>
#include <process.h>

#undef BYTE
#undef LONG

#define INCL_DOS
#define INCL_DOSERRORS
#include <os2.h>
#include "int64.hpp"
typedef ULARGE_INTEGER DWORDLONG;
#include "nfstart.h"
#include "dosclls.hpp"
#include "nounload.hpp"
#else
#error "unsupported operating system"
#endif
#include "dlistt.hpp"
#include "string.hpp"
#include "evtsub_sdk.hpp"
#include "essdk.h"
#include "essdk.hpp"

/* defines */

#define NIL 0

#if defined( __OS2__ ) || defined( __NETWARE__ )
#define MAX_PATH 260
#endif

/* constants */

#if defined( __WIN32__ ) || defined( __OS2__ )
const char * const LIBRARY_NAME = "essdk.dll";
#elif defined( __NETWARE__ )
const char * const LIBRARY_NAME = "essdk.nlm";
#endif

#if defined( __NETWARE__ )
const char * const NLM_NAME = "ESSDK";
#endif

const wchar_t * const QUALIFIER_ROOT = L"SDK";
const wchar_t * const QUALIFIER_SAMPLE = L"sample";

/* function prototypes */

#if defined( __WIN32__ ) || defined( __OS2__ )
BOOL APIENTRY initialize( const RAS * const log, const EventSubscriber * const subscriber );
const wchar_t * APIENTRY rootQualifier( void );
void APIENTRY eventSourceDescriptors( DListT< EventSourceDescriptor * > & eventSourceDescriptorList );
BOOL APIENTRY run( void );
BOOL APIENTRY finalize( void );
void APIENTRY updateIndication( void );
#elif defined( __NETWARE__ )
extern "C" NLMInitData * APIENTRY ESSDK_NLMInit( void );
extern "C" void APIENTRY ESSDK_NLMTerm( void );

extern "C" BOOL APIENTRY ESSDK_initialize( const RAS * const log, const EventSubscriber * const subscriber );
extern "C" const wchar_t * APIENTRY ESSDK_rootQualifier( void );
extern "C" void APIENTRY ESSDK_eventSourceDescriptors( DListT< EventSourceDescriptor * > & eventSourceDescriptorList );
extern "C" BOOL APIENTRY ESSDK_run( void );
extern "C" BOOL APIENTRY ESSDK_finalize( void );
extern "C" void APIENTRY ESSDK_updateIndication( void );
#endif

/* external variables */

#if defined( __NETWARE__ )
extern "C" NLMInitData  __NLMInitData;
extern NewHandler       handler;
#endif

/* global variables */

#if defined( __WIN32__ )
HINSTANCE          handle = 0;
#elif defined( __OS2__ )
unsigned long      handle = 0;
#elif defined( __NETWARE__ )
static int         state = 0;
int                threadIdentifier = 0;
int                users = 0;
#endif
EventSubscriber *  eventSubscriber = NIL;
RAS *              ras = NIL;
StrNLSBundle       resourceBundle( (char * ) LIBRARY_NAME, NIL );
EventSemaphore     semaphore( FALSE );
EventThread *      thread = NIL;

#if defined( __WIN32__ )

/* DllMain */

BOOL WINAPI DllMain( HINSTANCE instance, unsigned long reason, void * reserved )

  { /* DllMain */

  if ( reason == DLL_PROCESS_ATTACH )
    handle = instance;
  else
    handle = 0;

  return ( TRUE );
  } /* DllMain */

#elif defined( __OS2__ )

extern "C" void __ctordtorInit( void );
extern "C" void __ctordtorTerm( void );

/* _DLL_InitTerm */

unsigned long _System _DLL_InitTerm( unsigned long module, unsigned long flag )

  { /* _DLL_InitTerm */

  unsigned long    result = 1;

  switch ( flag )
    { /* switch */
    case 0:
      __ctordtorInit();
      handle = module;
      break;
    case 1:
      __ctordtorTerm();
      break;
    default:
      result = 0;
      break;
    } /* switch */

  return ( result );
  } /* _DLL_InitTerm */

#elif defined( __NETWARE__ )

/* ESSDK_NLMInit */

NLMInitData * APIENTRY ESSDK_NLMInit( void )

  { /* ESSDK_NLMInit */

  users++;
  __NLMInitData.closeFile = 1;

  return ( &__NLMInitData );
  } /* ESSDK_NLMInit */

/* ESSDK_NLMTerm */

void APIENTRY ESSDK_NLMTerm( void )

  { /* ESSDK_NLMTerm */

  if ( users > 0 )
    { /* decrement users */
    users--;
    if ( users == 0 )
      ResumeThread( threadIdentifier );
    } /* decrement users */

  } /* ESSDK_NLMTerm */

/* main */

void main( void )

  { /* main */

  APIRET           rc = NO_ERROR;

#if defined( _DEBUG )
  cprintf( "%s: main: nlm identifier = X\'%X\'\n", NLM_NAME, GetNLMID() );
#endif

  rc = DOSCLLS_InitLibrary();
  if ( rc == NO_ERROR )
    state = 1;
  else
    { /* library initialization failed */
    cprintf( "%s: unable to initialize DOSCALLS library; rc = X\'%lX\'\n", NLM_NAME, rc );
    state = -1;
    } /* library initialization failed */

  threadIdentifier = GetThreadID();
  SuspendThread( threadIdentifier );

#if defined( _DEBUG )
  cprintf( "%s: exit main\n", NLM_NAME );
#endif

  } /* main */

#endif

#if defined( __WIN32__ ) || defined( __OS2__ )

/* initialize */

BOOL APIENTRY initialize( const RAS * const log, const EventSubscriber * const subscriber )

  { /* initialize */

  BOOL             successful = FALSE;

  // save parameters in global variables

  ras = (RAS *) log;
  eventSubscriber = (EventSubscriber *) subscriber;

  ras->entry( "sdk.initialize" );

  // allocate thread object

  thread = new EventThread();
  if ( thread != NIL )
    successful = TRUE;
  else
    ras->error( "sdk.initialize: unable to allocate thread object" );

  ras->exit( "sdk.initialize: return ( %s )", ( successful ? "TRUE" : "FALSE" ) );

  return ( successful );
  } /* initialize */

#elif defined( __NETWARE__ )

/* initialize */

BOOL APIENTRY ESSDK_initialize( const RAS * const log, const EventSubscriber * const subscriber )

  { /* initialize */

  BOOL             successful = FALSE;

  // wait for main to finish

  while ( state == 0 )
	  ThreadSwitchWithDelay();

  // save parameters in global variables

  ras = (RAS *) log;
  eventSubscriber = (EventSubscriber *) subscriber;

  ras->entry( "sdk.initialize" );

  if ( state > 0 )
    { /* allocate thread */

    // allocate thread object

    thread = new EventThread();
    if ( thread != NIL )
      successful = TRUE;
    else
      ras->error( "sdk.initialize: unable to allocate thread object" );

    } /* allocate thread */

  ras->exit( "sdk.initialize: return ( %s )", ( successful ? "TRUE" : "FALSE" ) );

  return ( successful );
  } /* initialize */

#endif

/* rootQualifier */

#if defined( __WIN32__ ) || defined( __OS2__ )
const wchar_t * APIENTRY rootQualifier( void )
#elif defined( __NETWARE__ )
const wchar_t * APIENTRY ESSDK_rootQualifier( void )
#endif

  { /* rootQualifier */
  return ( QUALIFIER_ROOT );
  } /* rootQualifier */

/* eventSourceDescriptors */

#if defined( __WIN32__ ) || defined( __OS2__ )
void APIENTRY eventSourceDescriptors( DListT< EventSourceDescriptor * > & eventSourceDescriptorList )
#elif defined( __NETWARE__ )
void APIENTRY ESSDK_eventSourceDescriptors( DListT< EventSourceDescriptor * > & eventSourceDescriptorList )
#endif

  { /* eventSourceDescriptors */

  EventSourceDescriptor *    eventSourceDescriptor = NIL;
  wchar_t *                  externalQualifierList[ 1 ] = { NIL };
  wchar_t *                  internalQualifierList[ 1 ] = { NIL };

  ras->entry( "sdk.eventSourceDescriptors" );

  // initialize qualifier lists

  internalQualifierList[ 0 ] = (wchar_t *) QUALIFIER_SAMPLE;
  externalQualifierList[ 0 ] = getString( resourceBundle, RESOURCE_IDENTIFIER_QUALIFIER_SAMPLE );

  // allocate event descriptor object

  eventSourceDescriptor = new EventSourceDescriptor( QUALIFIER_ROOT, 1UL, (const wchar_t **) internalQualifierList, (const wchar_t **) externalQualifierList, resourceBundle.getActualLocale() );

  // add details to event source descriptor

  eventSourceDescriptor->addDetail( L"datetime", getString( resourceBundle, RESOURCE_IDENTIFIER_DETAIL_DATETIME ), EventSourceDescriptor::DETAIL_DATATYPE_DATETIME );
  eventSourceDescriptor->addDetail( L"integer", getString( resourceBundle, RESOURCE_IDENTIFIER_DETAIL_INTEGER ), EventSourceDescriptor::DETAIL_DATATYPE_INT32 );
  eventSourceDescriptor->addDetail( L"octet", getString( resourceBundle, RESOURCE_IDENTIFIER_DETAIL_OCTET ), EventSourceDescriptor::DETAIL_DATATYPE_OCTET );
  eventSourceDescriptor->addDetail( L"string", getString( resourceBundle, RESOURCE_IDENTIFIER_DETAIL_STRING ), EventSourceDescriptor::DETAIL_DATATYPE_STRING16 );

  // append event source descriptor to list

  eventSourceDescriptorList.append( eventSourceDescriptor );
  eventSourceDescriptor = NIL;

  // allocate and append any additional event source descriptors

  ras->exit( "sdk.eventSourceDescriptors" );
  } /* eventSourceDescriptors */

/* run */

#if defined( __WIN32__ ) || defined( __OS2__ )
BOOL APIENTRY run( void )
#elif defined( __NETWARE__ )
BOOL APIENTRY ESSDK_run( void )
#endif

  { /* run */
  ras->entry( "sdk.run" );

  // start processing thread

  if ( thread != NIL )
    thread->StartThread();

  ras->exit( "sdk.run" );

  return ( TRUE );
  } /* run */

/* finalize */

#if defined( __WIN32__ ) || defined( __OS2__ )
BOOL APIENTRY finalize( void )
#elif defined( __NETWARE__ )
BOOL APIENTRY ESSDK_finalize( void )
#endif

  { /* finalize */
  ras->entry( "sdk.finalize" );

  if ( thread != NIL )
    { /* deallocate thread */

    // stop thread

    thread->SignalStop();
    semaphore.Post();
    thread->WaitForStop();

    delete thread;
    thread = NIL;
    } /* deallocate thread */

  ras->exit( "sdk.finalize" );

  return ( TRUE );
  } /* finalize */

/* updateIndication */

#if defined( __WIN32__ ) || defined( __OS2__ )
void APIENTRY updateIndication( void )
#elif defined( __NETWARE__ )
void APIENTRY ESSDK_updateIndication( void )
#endif

  { /* updateIndication */
  ras->entry( "sdk.updateIndication" );

  // invoke getServerList with event descriptor(s) to determine if event is "wanted" by any server

  ras->exit( "sdk.updateIndication" );
  } /* updateIndication */

/* EventThread::EventThread */

EventThread::EventThread( void ) : Thread()

  { /* EventThread::EventThread */
  ras->entry( "sdk.EventThread::EventThread" );
  ras->exit( "sdk.EventThread::EventThread" );
  } /* EventThread::EventThread */

/* EventThread::~EventThread */

EventThread::~EventThread( void )

  { /* EventThread::~EventThread */
  ras->entry( "sdk.EventThread::~EventThread" );
  ras->exit( "sdk.EventThread::~EventThread" );
  } /* EventThread::~EventThread */

/* EventThread::DoProcessing */

void EventThread::DoProcessing( void )

  { /* EventThread::DoProcessing */

#if defined( __OS2__ )
  unsigned long    postCount = 0;
#endif

  ras->entry( "sdk.EventThread::DoProcessing" );

  while ( ! StopSignalled() )
    { /* loop */
    indicate( 1, L"text: %1 %2 %3" );
    semaphore.Wait( 60UL * 1000UL );
    } /* loop */

  ras->exit( "sdk.EventThread::DoProcessing" );
  } /* EventThread::DoProcessing */

/* EventThread::indicate */

void EventThread::indicate( unsigned long identifier, const wchar_t * message )

  { /* EventThread::indicate */

  Event *                    event = NIL;
  EventDescriptor *          eventDescriptor = NIL;
  wchar_t                    identifierString[ 32 + 1 ] = { L"" };
#if defined( __WIN32__ )
  DWORDLONG                  milliseconds = 0;
#elif defined( __OS2__ ) || defined( __NETWARE__ )
  DWORDLONG                  milliseconds = { 0UL, 0UL };
#endif
  wchar_t *                  qualifierList[ 2 ] = { NIL, NIL };
  Server *                   server = NIL;
  DListT< Server * >         serverList;
  Event::Severity            severity = Event::harmless;
  time_t                     timestamp = 0;
  wchar_t *                  variableList[ 3 ] = { NIL, NIL, NIL };

  ras->entry( "sdk.EventThread::indicate: identifier = %ld, message = \"%ls\"", identifier, message );

  memset( identifierString, 0x00, sizeof( identifierString ) );
#if defined( __WIN32__ )
   swprintf( identifierString, L"%lu", identifier );
#elif defined( __OS2__ )
   swprintf( identifierString, sizeof( identifierString ), L"%lu", identifier );
#elif defined( __NETWARE__ )
  _ultow( identifier, identifierString, 10 );
#endif

  qualifierList[ 0 ] = (wchar_t *) QUALIFIER_SAMPLE;
  qualifierList[ 1 ] = identifierString;

  eventDescriptor = new EventDescriptor( QUALIFIER_ROOT, 2, (const wchar_t **) qualifierList );
  eventSubscriber->getServerList( eventDescriptor, serverList );
  if ( serverList.len() > 0 )
    { /* indicate */

    // initialize substitution strings

    variableList[ 0 ] = L"alpha";
    variableList[ 1 ] = L"beta";
    variableList[ 2 ] = L"gamma";

    // create event object

    event = new Event( QUALIFIER_ROOT,
                       2,
                       (const wchar_t **) qualifierList,
                       Event::alert,
                       severity,
                       ( message != NIL ? message : L" " ),
                       sizeof( variableList ) / sizeof( variableList[ 0 ] ),
                       (const wchar_t **) variableList,
                       resourceBundle.getActualLocale() );

    // add details to event

#if defined( __WIN32__ )
    milliseconds = (DWORDLONG) time( &timestamp ) * 1000Ui64;
#elif defined( __OS2__ ) || defined( __NETWARE__ )
    Uint64 date( (unsigned long) time( &timestamp ) );
    date = date * 1000UL;
    milliseconds = (DWORDLONG) date;
#endif
    event->addDateDetail( L"datetime", NIL, NIL, milliseconds );
    event->addLongDetail( L"integer", NIL, NIL, timestamp );
    event->addOctetDetail( L"octet", NIL, NIL, 0, NIL );
    event->addStringDetail( L"string", NIL, NIL, L"sample" );

    // send event to server(s)

    eventSubscriber->send( serverList, event );
    delete event;
    event = NIL;

    server = serverList.removeFirst();
    while ( server != NIL )
      { /* clear */
      delete server;
      server = NIL;
      server = serverList.removeFirst();
      } /* clear */
    } /* indicate */
  else
    ras->debug( "sdk.EventThread::indicate: no servers" );

  if ( eventDescriptor != NIL )
    delete eventDescriptor;
  eventDescriptor = NIL;

  ras->exit( "sdk.EventThread::indicate" );
  } /* EventThread::indicate */

