/*****************************************************************************
 * Licensed Materials - Property of IBM                            
 * PID (5697-TKT)                                                             
 * (C) Copyright by IBM 1999. All Rights Reserved.
 ****************************************************************************/
#ifndef _LINKLIST_
#define _LINKLIST_

template <class T> 
class ListElement
{
public:
   ListElement();
   T
   fElement;
   ListElement<T>
   *fNextElement;
   ListElement<T>
   *fPrevElement;
};

template <class T> 
class LinkListIterator;

template <class T> 
class LinkList
{
public:
   LinkList(); 
   ~LinkList();
   void 
   addElementToEndOfList( const T& newNode ),
   insertElementBeforeCursor( const T& newNode, LinkListIterator<T> &cursor ),
   insertElementAfterCursor( const T& newNode, LinkListIterator<T> &cursor );
   void
   removeElement( LinkListIterator<T> &cursor );
   unsigned long
   numberOfElements();
   T
   elementAt( LinkListIterator<T> &cursor);
private:
   ListElement<T>
   *fHead;
   unsigned long 
   fNumberOfElements;
   friend class LinkListIterator<T>;
};

template <class T> class LinkListIterator
{
public:
   LinkListIterator( LinkList<T> *list );

   void
   setToFirst(),
   setToLast(),
   setToNext();
   BOOL
   isValid( );
private:
   LinkList<T>
   *fLinkList;
   ListElement<T>
   *fCurrent;
   friend class LinkList<T>;
};

template<class T>
LinkList<T>::~LinkList()
{ 
   ListElement<T> *fCurrent = fHead;
   ListElement<T> *fTmp;
   while ( fCurrent )
   {
      fTmp = fCurrent;
      fCurrent = fCurrent->fNextElement;
      delete fTmp;
   }
   fHead = 0;
}


template<class T>
ListElement<T>::ListElement( )
: fNextElement( 0 )
{
}

template<class T>
LinkList<T>::LinkList() 
: fHead( 0 )
, fNumberOfElements(0) 
{
}

template<class T>
void LinkList<T>::addElementToEndOfList( const T& newNode )
{
   ListElement<T> *fListElement = new ListElement<T>;
   fListElement->fElement = newNode;
   fListElement->fNextElement = 0;

   if ( fHead )
   {
      ListElement<T>
      *fTemp = fHead;
      while ( fTemp->fNextElement != 0 )
         fTemp = fTemp->fNextElement;
      fTemp->fNextElement = fListElement;
      fListElement->fPrevElement = fTemp;
   }
   else
   {
      // first element added to the list
      fHead = fListElement;
      fListElement->fPrevElement = 0;
   }
   fNumberOfElements++;
}

template<class T>
void LinkList<T>::insertElementBeforeCursor( const T& newNode, 
                                             LinkListIterator<T> &cursor )
{
   ListElement<T> *fListElement = new ListElement<T>;
   fListElement->fElement = newNode;

   ListElement<T>
   *fTemp;

   fTemp = cursor.fCurrent;
   if ( fTemp )
   {
      if ( fTemp == fHead )
         fHead = fListElement;
      if ( fTemp->fPrevElement )
         fTemp->fPrevElement->fNextElement = fListElement;
      fListElement->fPrevElement = fTemp->fPrevElement;
      fListElement->fNextElement = fTemp;
      fTemp->fPrevElement = fListElement;
   }
   else
   {
      // first element added to the list
      fHead = fListElement;
      fListElement->fPrevElement = 0;
      fListElement->fNextElement = 0;
   }
   fNumberOfElements++;
}

template<class T>
void LinkList<T>::insertElementAfterCursor( const T& newNode, 
                                            LinkListIterator<T> &cursor )
{
   ListElement<T> *fListElement = new ListElement<T>;
   fListElement->fElement = newNode;

   ListElement<T>
   *fTemp;

   fTemp = cursor.fCurrent;
   if ( fTemp )
   {
      fListElement->fNextElement = fTemp->fNextElement;
      fListElement->fPrevElement = fTemp;
      if ( fTemp->fNextElement )
         fTemp->fNextElement->fPrevElement = fListElement;
      fTemp->fNextElement = fListElement;
   }
   else
   {
      // first element added to the list
      fHead = fListElement;
      fListElement->fPrevElement = 0;
      fListElement->fNextElement = 0;
   }
   fNumberOfElements++;
}

template<class T>
void LinkList<T>::removeElement( LinkListIterator<T> &cursor )
{
   ListElement<T>
   *fTemp;

   fTemp = cursor.fCurrent;

   if ( fTemp )
   {
      if ( fTemp->fPrevElement )
      {
         fTemp->fPrevElement->fNextElement = fTemp->fNextElement;
      }
      if ( fTemp->fNextElement )
      {
         fTemp->fNextElement->fPrevElement = fTemp->fPrevElement;
      }
      if ( fTemp == fHead )
         fHead = fTemp->fNextElement;
      delete fTemp;
      fNumberOfElements--;
   }
}

template<class T>
T LinkList<T>::elementAt(LinkListIterator<T> &cursor) 
{ 
   return cursor.fCurrent->fElement;
}

template<class T>
inline unsigned long LinkList<T>::numberOfElements()
{
   return fNumberOfElements;
}

template<class T>
LinkListIterator<T>::LinkListIterator(LinkList<T> *linkList)
{
   fLinkList = linkList;
}


template<class T>
void LinkListIterator<T>::setToFirst()
{
   fCurrent = fLinkList->fHead;
}

template<class T>
void LinkListIterator<T>::setToLast()
{
   fCurrent = fLinkList->fHead;
   while ( fCurrent->fNextElement != 0 )
      fCurrent = fCurrent->fNextElement;
}

template<class T>
void LinkListIterator<T>::setToNext()
{
   if ( isValid() )
   {
      fCurrent = fCurrent->fNextElement;
   }
}

template<class T>
inline BOOL 
LinkListIterator<T>::isValid() 
{
   if ( fCurrent )
      return TRUE;
   else
      return FALSE;
}

#endif
