/*****************************************************************************
 * Licensed Materials - Property of IBM
 * PID (5697-TKT)
 * (C) Copyright by IBM 1999. All Rights Reserved.
 ****************************************************************************/
extern "C"
{
//  #include <monitor.h>
#include <windows.h>
#include <mon_sdk.h>
}
#include <fstream.h>
#include <cp_conv.hpp>
#include <dirgraph.h>
#include <monitlib.hpp>
#include <keyinfob.hpp>
#include <valuegen.hpp>

/**
 * Private Class used to hide implementation details from public
 * interface.
 */
class MonitorLibraryPrivate
{
public:
  MonitorLibraryPrivate();
  ~MonitorLibraryPrivate();
  ValueGenerator
   *fValueGenerator;
  DirectedGraph<KeyInfoBlock>
    fTree;
  KeyInfoBlock
   *keyInfoBlockWith( const KeyInfoBlock& fKIB, unsigned long handle );
  KeyInfoBlock
   *getParentOf( const KeyInfoBlock& fParent, const KeyInfoBlock& fKIB );
};

MonitorLibraryPrivate::MonitorLibraryPrivate()
: fValueGenerator(0)
{
}

MonitorLibraryPrivate::~MonitorLibraryPrivate()
{
  if ( fValueGenerator )
    delete fValueGenerator;
}

/**
 * Monitor Library constructor
 */
MonitorLibrary::MonitorLibrary()
: fData(new MonitorLibraryPrivate )
{
}

/**
 * MonitorLibrary destructor
 */
MonitorLibrary::~MonitorLibrary()
{
#ifdef _DEBUG
  ofstream to("C:\\temp\\test.out", ios::out | ios::app );
  to << "MonitorLibrary::~MonitorLibrary " << endl;
#endif
  if ( fData )
    delete fData;
}

/**
 *
 */
void MonitorLibrary::addKeyInfoBlock( const KeyInfoBlock& root )
{
#ifdef _DEBUG
  ofstream to("C:\\temp\\test.out", ios::out | ios::app );
  to << "MonitorLibrary::addKeyInfoBlock::Root " << endl;
#endif
  fData->fTree.addNode( root );
}

/**
 *
 */
void MonitorLibrary::addKeyInfoBlock( const KeyInfoBlock& parent,
                                      const KeyInfoBlock& child )
{
#ifdef _DEBUG
  ofstream to("C:\\temp\\test.out", ios::out | ios::app );
  to << "MonitorLibrary::addKeyInfoBlock::Parent Child " << endl;
#endif
  fData->fTree.addNode( parent, child );
}

/**
 *
 */
void MonitorLibrary::adoptValueGenerator( ValueGenerator *valueGenerator )
{
  fData->fValueGenerator = valueGenerator;
}

/**
 *
 */
unsigned long MonitorLibrary::getKeyFor( AttributePath *queriedPath )
{
#ifdef _DEBUG
  ofstream to("C:\\temp\\test.out", ios::out | ios::app );
  to << "MonitorLibrary::getKeyFor" << endl;
#endif
  unsigned long fHandle( INVALID_HANDLE );

  KeyInfoBlock* fKIB = getKeyInfoBlockFor( queriedPath );
  if ( fKIB )
  {
    fHandle = fKIB->getHandle();
    delete fKIB;
  }
  return fHandle;
}

/**
 *
 */
KeyInfoBlock* MonitorLibrary::getKeyInfoBlockWith( unsigned long handle )
{
#ifdef _DEBUG
  ofstream to("C:\\temp\\test.out", ios::out | ios::app );
  to << "MonitorLibrary::getKeyInfoBlockWith" << endl;
#endif
  KeyInfoBlock fKIB = fData->fTree.rootOfTree();

  if ( fKIB.getHandle() == handle )
    return new KeyInfoBlock( fKIB );

  return fData->keyInfoBlockWith( fKIB, handle );
}

/**
 *
 */
//KeyInfoBlock* MonitorLibrary::getParentOf( const KeyInfoBlock &fKIB)
//{
//#ifdef _DEBUG
//  ofstream to("C:\\temp\\test.out", ios::out | ios::app );
//  to << "MonitorLibrary::getParentOf" << endl;
//#endif
//  KeyInfoBlock fRoot = fData->fTree.rootOfTree();
//
//  // if this is the
//  if ( fRoot == fKib )
//    return 0;
//
//  return fData->getParentOf( fRoot, fKIB );
//}

/**
 *
 */
KeyInfoBlock* MonitorLibraryPrivate::getParentOf( const KeyInfoBlock &fParent,
                                                  const KeyInfoBlock &fKIB )
{
#ifdef _DEBUG
  ofstream to("C:\\temp\\test.out", ios::out | ios::app );
  to << "MonitorLibraryPrivate::getParentOf" << endl;
#endif

  DirectedGraphIterator<KeyInfoBlock> fCursor( &fTree, fParent );
  for ( fCursor.setToFirst();
      fCursor.isValid();
      fCursor.setToNext() )
  {
    KeyInfoBlock fChildKIB = fTree.elementAt( fCursor );
    if ( fChildKIB == fKIB )
      return new KeyInfoBlock( fParent );

    KeyInfoBlock *fFoundKIB = getParentOf( fChildKIB, fKIB );
    if ( fFoundKIB )
      return fFoundKIB;
  }
  return 0;
}

/**
 *
 */
KeyInfoBlock* MonitorLibraryPrivate::keyInfoBlockWith( const KeyInfoBlock &fKIB,
                                                       unsigned long handle )
{
#ifdef _DEBUG
  ofstream to("C:\\temp\\test.out", ios::out | ios::app );
  to << "MonitorLibrary::getKeyInfoBlockWith" << endl;
#endif

  DirectedGraphIterator<KeyInfoBlock> fCursor( &fTree, fKIB );
  for ( fCursor.setToFirst();
      fCursor.isValid();
      fCursor.setToNext() )
  {
    KeyInfoBlock fChildKIB = fTree.elementAt( fCursor );
    if ( fChildKIB.getHandle() == handle )
      return new KeyInfoBlock( fChildKIB );

    KeyInfoBlock *fFoundKIB = keyInfoBlockWith( fChildKIB, handle );
    if ( fFoundKIB )
      return fFoundKIB;
  }
  return 0;
}


/**
 * Create our real nodeList to return
 */
AttributeNodeList* MonitorLibrary::orphanAttributeNodeListFor(
                                                             AttributePath *queriedPath)
{
#ifdef _DEBUG
  ofstream to("C:\\temp\\test.out", ios::out | ios::app );
  to << "MonitorLibrary::orphanAttributeNodeListFor " << endl;
#endif

  unsigned long
  fTotalSize;
  static unsigned long
  fAttributeNodeListSize( sizeof(AttributeNodeList));
  AttributeNodeList
  *fAttributeNodeList = 0;

  // the temp info block only contains the programmatic name of the block
  // for comparison.
  KeyInfoBlock* fKIB = getKeyInfoBlockFor( queriedPath );

  if ( fKIB )
  {
#ifdef _DEBUG
    to << "  Key info block " << fKIB->getProgrammaticName() << endl;
#endif
    // this will create a cursor for the children of queriedPath
    DirectedGraphIterator<KeyInfoBlock> fCursor( &fData->fTree,
                                                 *fKIB );

    // we need to step through this data structure twice, first to allocate
    // the needed size, and then to copy the data into it.
    fTotalSize = fAttributeNodeListSize;
    for ( fCursor.setToFirst( );
        fCursor.isValid( );
        fCursor.setToNext( ) )
    {
      KeyInfoBlock fChild = fData->fTree.elementAt( fCursor );
#ifdef _DEBUG
      to << "  found child " << fChild.getProgrammaticName() << endl;
#endif
      // by default nodes are assumed to be leaf nodes. We only need to check
      // to see if the node has children, and then we know the node is a folder.
      if ( fData->fTree.numberOfChildrenFor( fChild ) )
      {
        fChild.setLeafNode( FALSE );
      }
      fTotalSize += fChild.getAttributeNodeSize();
    }
#ifdef _DEBUG
    to << "  Total calculated size is " << fTotalSize << endl
    << "  Attribute Node List Size " << fAttributeNodeListSize << endl;
#endif
    // create the memory to return
    fAttributeNodeList =
    (AttributeNodeList *) new char[fTotalSize];

#ifdef _DEBUG
    to << "  AttributeNodeList " << (void *) fAttributeNodeList << endl;
#endif
    // if new fails then return
    if ( fAttributeNodeList != NULL )
    {
      // clear out the memory
      memset( fAttributeNodeList, 0, fTotalSize);

#ifdef _DEBUG
      to << "  initialize fields " << endl;
#endif
      // initialize the fields of AttributeNodeList
      fAttributeNodeList->size = fAttributeNodeListSize;
      fAttributeNodeList->totalSize = fTotalSize;
      fAttributeNodeList->version = NODE_LIST_VERSION_2;
      fAttributeNodeList->members =
      fData->fTree.numberOfChildrenFor(*fKIB);


      // Now copy the key info blocks into the attribute node list.
      AttributeNode *fAttributeNode;
      fAttributeNode = (AttributeNode *)( (char *)fAttributeNodeList +
                                          fAttributeNodeListSize );
#ifdef _DEBUG
      to << "  fAttribute Node " << (void *) fAttributeNode << endl;
#endif

      for ( fCursor.setToFirst( );
          fCursor.isValid( );
          fCursor.setToNext() )
      {
#ifdef _DEBUG
        to << "  Attribute Node " << (void *) fAttributeNode << endl;
#endif
        KeyInfoBlock fChild = fData->fTree.elementAt( fCursor );
#ifdef _DEBUG
        to << "  Copy data to " << endl;
#endif
        fChild.copyDataTo( fAttributeNode );

        // update copy offset
        fAttributeNode = (AttributeNode *)
                         ((char *)fAttributeNode + fAttributeNode->size);

      }
#ifdef _DEBUG
      to << "  return the attribute node list " << endl;
#endif
    }
    delete fKIB;
  }
  else
  {
#ifdef _DEBUG
    to << "  Key Info Block not found" << endl;
#endif
  }
  return fAttributeNodeList;
}

KeyInfoBlock* MonitorLibrary::getKeyInfoBlockFor( AttributePath *queriedPath )
{
  ofstream to("C:\\temp\\test.out", ios::out | ios::app );
  to << "MonitorLibrary::getKeyInfoBlockFor " << endl;
  if ( queriedPath == 0 )
    return 0;
  if ( queriedPath->members )
  {
    // first we need to verify the root of the path name.
    char *fChildName = 0;
    unsigned long fNameLength =
    NFConvertCompUnicodeToLocal(
                               (char *)(((char *)(queriedPath))+queriedPath->offsets[0]),
                               NULL, 0, L'*', NULL);
    fChildName = new char[fNameLength + 1];
    NFConvertCompUnicodeToLocal(
                               (char *)(((char *)(queriedPath))+queriedPath->offsets[0]),
                               fChildName,
                               fNameLength, L'*', NULL);

    // traverse down the path to find the keyinfo block
    KeyInfoBlock fKIB = fData->fTree.rootOfTree();
#ifdef _DEBUG
    to << "  Root name is " << fChildName << " and root KIB " <<
    fKIB.getProgrammaticName() << endl;
#endif
    if ( strcmp(fKIB.getProgrammaticName(), fChildName) != 0 )
    {
#ifdef _DEBUG
      to << "  Root test failed" << endl;
#endif
      // not in our list, return an invalid handle.
      return 0;
    }
    else if ( queriedPath->members == 1 )
    {
#ifdef _DEBUG
      to << "  Found root" << endl;
#endif
      return new KeyInfoBlock( fKIB );
    }

    // we matched the root name, now we need to continue down the path.
    for ( int i = 1; i < queriedPath->members; i++ )
    {
      if ( fChildName )
      {
        delete [] fChildName;
        fChildName = 0;
      }
      fNameLength =
      NFConvertCompUnicodeToLocal(
                                 (char *)(((char *)(queriedPath))+queriedPath->offsets[i]),
                                 NULL, 0, L'*', NULL);
      fChildName = new char[fNameLength + 1];
      NFConvertCompUnicodeToLocal(
                                 (char *)(((char *)(queriedPath))+queriedPath->offsets[i]),
                                 fChildName,
                                 fNameLength, L'*', NULL);
#ifdef _DEBUG
      to << "Look for child " << fChildName << endl;
#endif
      // Now cursor through
      BOOL fFound( FALSE );
      DirectedGraphIterator<KeyInfoBlock> fCursor( &fData->fTree, fKIB );
      for ( fCursor.setToFirst();
          fCursor.isValid() && !fFound;
          fCursor.setToNext() )
      {
        fKIB = fData->fTree.elementAt( fCursor );
        if ( strcmp( fKIB.getProgrammaticName(), fChildName ) == 0 )
        {
          // found it, now see if we are done.
          if ( i == (queriedPath->members - 1) )
          {
            // then we are done
            if ( fChildName )
            {
              delete [] fChildName;
              fChildName = 0;
            }
            return new KeyInfoBlock(fKIB );
          }
          else
          {
            fFound = TRUE;
          }
        }
      }
      if ( !fFound )
        break;
    }
    if ( fChildName )
      delete [] fChildName;
  }
  return 0;
}

void MonitorLibrary::setValue( KeyInfoBlock& block )
{
  if ( fData->fValueGenerator )
    fData->fValueGenerator->setValueFor( block );
}


void MonitorLibrary::addNodes( )
{
  if ( fData->fValueGenerator )
    fData->fValueGenerator->addNodesTo( *this );
}

/**
 *
 */
void MonitorLibrary::updateNodesFor( AttributePath *queriedPath)
{
#ifdef _DEBUG
  ofstream to("C:\\temp\\test.out", ios::out | ios::app );
  to << "MonitorLibrary::updateNodes " << endl;
#endif
  if ( fData->fValueGenerator )
  {
    KeyInfoBlock *queriedBlock = getKeyInfoBlockFor( queriedPath );
    if ( queriedBlock )
    {
#ifdef _DEBUG
      to << " for block " << queriedBlock->getDisplayableName() << endl;
#endif
      fData->fValueGenerator->updateNodesFor( *queriedBlock, *this );
      delete queriedBlock;
    }
  }
}
