/*****************************************************************************
 * Licensed Materials - Property of IBM                            
 * PID (5697-TKT)                                                             
 * (C) Copyright by IBM 1999. All Rights Reserved.
 ****************************************************************************/
#include <valuegen.hpp>

class KeyInfoBlock;

/**             
 * This class is a derived class from ValueGenerator. An object of this class is
 * new'd by monsamp.cpp and adopted by the MonitorLibrary object. Monsamp then
 * calls member functions of this object through the MonitorLibrary object. The
 * two abstract member functions of ValueGenerator that must be overriden are
 * setValueFor and addNodesTo.
 */
class MyValueGenerator : public ValueGenerator
{
public:
   /**
    * Constructor, usually KeyInfoBlock objects will be created in the ctr, as
    * they are needed in both the addNodesTo and setValueFor member functions.
    */
   MyValueGenerator();
   /**
    * Destructor for MyValueGenerator objects.
    */
   virtual
   ~MyValueGenerator();

   /**
    * Called to set the value for the passed KeyInfoBlock. This member function
    * is called based on the pulse information set in the KeyInfoBlock object.
    *
    * @param fBlock - KeyInfoBlock that needs a value set.
    */
   void
   setValueFor( KeyInfoBlock& fBlock);
   /**
    * Called when heirarchal nodes need to be added to the MonitorLibrary. The
    * MonitorLibrary object has the member functions for adding the KeyInfoBlock
    * objects in a tree structure.
    *
    * @param fMonitor - MonitorLibrary object that the KeyInfoBlock nodes are
    *                   added to.
    */
   void
   addNodesTo( MonitorLibrary& fMonitor );
   /**
     * Called when a user expands an folder and the AttributeNodeList is to be
     * returned. This allows nodes to be added to support dynamic 
     * monitors.
     */
   void
   updateNodesFor( KeyInfoBlock& fBlock,
                   MonitorLibrary& fMonitor);
private:
   KeyInfoBlock
     fSampleMonitor,
     fFolder2,
     fFolder3,
     fExpandableFolder,
     fPI,
     fTime,
     fPrinter1,
     fPrinter2;
};
